@@ -10,7 +10,6 @@ import os
 import sys
 import glob
 import time
-import argparse
 import warnings
 
 from mintpy.defaults import auto_path
@@ -28,26 +27,39 @@ from mintpy.objects.stackDict import (
     ifgramDict,
 )
 from mintpy.utils import readfile, ptime, utils as ut
+from mintpy.utils.arg_utils import create_argument_parser
 from mintpy import subset
 
 
 #################################################################
 PROCESSOR_LIST = ['isce', 'aria', 'hyp3', 'gmtsar', 'snap', 'gamma', 'roipac', 'cosicorr']
 
-datasetName2templateKey = {
+# primary observation dataset names
+OBS_DSET_NAMES = ['unwrapPhase', 'rangeOffset', 'azimuthOffset']
+
+IFG_DSET_NAME2TEMPLATE_KEY = {
     'unwrapPhase'     : 'mintpy.load.unwFile',
     'coherence'       : 'mintpy.load.corFile',
     'connectComponent': 'mintpy.load.connCompFile',
     'wrapPhase'       : 'mintpy.load.intFile',
-    'ionoPhase'       : 'mintpy.load.ionoFile',
     'magnitude'       : 'mintpy.load.magFile',
+}
 
+ION_DSET_NAME2TEMPLATE_KEY = {
+    'unwrapPhase'     : 'mintpy.load.ionUnwFile',
+    'coherence'       : 'mintpy.load.ionCorFile',
+    'connectComponent': 'mintpy.load.ionConnCompFile',
+}
+
+OFF_DSET_NAME2TEMPLATE_KEY = {
     'azimuthOffset'   : 'mintpy.load.azOffFile',
     'azimuthOffsetStd': 'mintpy.load.azOffStdFile',
     'rangeOffset'     : 'mintpy.load.rgOffFile',
     'rangeOffsetStd'  : 'mintpy.load.rgOffStdFile',
     'offsetSNR'       : 'mintpy.load.offSnrFile',
+}
 
+GEOM_DSET_NAME2TEMPLATE_KEY = {
     'height'          : 'mintpy.load.demFile',
     'latitude'        : 'mintpy.load.lookupYFile',
     'longitude'       : 'mintpy.load.lookupXFile',
@@ -79,26 +91,42 @@ NOTE = """NOTE:
 """
 
 EXAMPLE = """example:
-  load_data.py -t GalapagosSenDT128.tempalte
-  load_data.py -t smallbaselineApp.cfg
-  load_data.py -t smallbaselineApp.cfg GalapagosSenDT128.tempalte --project GalapagosSenDT128
-  load_data.py -H #Show example input template for ISCE/ROI_PAC/GAMMA products
+  # MUST run in the mintpy working directory!
 
-  # load geometry only
-  # fill metaFile, baselineDir and geometry datasets in the template and run load_data.py
+  # show example template file for ISCE/ROI_PAC/GAMMA products
+  load_data.py -H
+
+  # load & write the following HDF5 files:
+  # ./inputs/ifgramStack.h5   for interferogram        stack
+  # ./inputs/ionStack.h5      for ionosphere           stack
+  # ./inputs/offsetStack.h5   for range/azimuth offset stack
+  # ./inputs/geometryRadar.h5 for geometry in radar coordinates
+  # ./inputs/geometryGeo.h5   for geometry in geo   coordinates
+  load_data.py -t smallbaselineApp.cfg
+  load_data.py -t smallbaselineApp.cfg GalapagosSenDT128.txt --project GalapagosSenDT128
+
+  # load geometry ONLY
+  smallbaselineApp.py SaltonSeaSenDT173.txt -g
+  load_data.py -t smallbaselineApp.cfg --geom
 """
 
 
-def create_parser():
+def create_parser(subparsers=None):
     """Create command line parser."""
-    parser = argparse.ArgumentParser(description='Saving a stack of Interferograms to an HDF5 file',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=TEMPLATE+'\n'+NOTE+'\n'+EXAMPLE)
+    synopsis = 'Load stacks of interferograms to HDF5 files'
+    epilog = TEMPLATE + '\n' + NOTE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
+
     parser.add_argument('-H', dest='print_example_template', action='store_true',
                         help='Print/Show the example template file for loading.')
-    parser.add_argument('-t', '--template', type=str, nargs='+',
-                        dest='template_file', help='template file with path info.')
+    parser.add_argument('-t', '--template', dest='template_file', type=str, nargs='+',
+                        help='template file(s) with path info.')
+    parser.add_argument('--geom','--geometry', dest='only_load_geometry', action='store_true',
+                        help='Load the geometry file(s) ONLY.')
 
+    # options from template file name & content
     parser.add_argument('--project', type=str, dest='PROJECT_NAME',
                         help='project name of dataset for INSARMAPS Web Viewer')
     parser.add_argument('--processor', type=str, dest='processor', choices=PROCESSOR_LIST,
@@ -108,11 +136,6 @@ def create_parser():
     parser.add_argument('--compression', choices={'gzip', 'lzf', None}, default=None,
                         help='compress loaded geometry while writing HDF5 file, default: None.')
 
-    parser.add_argument('-o', '--output', type=str, nargs=3, dest='outfile',
-                        default=['./inputs/ifgramStack.h5',
-                                 './inputs/geometryRadar.h5',
-                                 './inputs/geometryGeo.h5'],
-                        help='output HDF5 file')
     return parser
 
 
@@ -136,21 +159,21 @@ def cmd_line_parse(iargs=None):
         print('{} -H to show the example template file'.format(os.path.basename(__file__)))
         sys.exit(1)
 
-    inps.outfile = [os.path.abspath(i) for i in inps.outfile]
-    inps.outdir = os.path.dirname(inps.outfile[0])
-
     return inps
 
 
 #################################################################
 def read_inps2dict(inps):
-    """Read input Namespace object info into iDict
+    """Read input Namespace object info into iDict.
 
     It grab the following contents into iDict
     1. inps & all template files
     2. configurations: processor, autoPath, updateMode, compression, x/ystep
     3. extra metadata: PLATFORM, PROJECT_NAME,
     4. translate autoPath
+
+    Parameters: inps  - namespace, input arguments from command line & template file
+    Returns:    iDict - dict,      input arguments from command line & template file
     """
     # Read input info into iDict
     iDict = vars(inps)
@@ -167,14 +190,13 @@ def read_inps2dict(inps):
     if 'processor' in template.keys():
         template['mintpy.load.processor'] = template['processor']
 
+    # group - load
     prefix = 'mintpy.load.'
     key_list = [i.split(prefix)[1] for i in template.keys() if i.startswith(prefix)]
     for key in key_list:
         value = template[prefix+key]
         if key in ['processor', 'autoPath', 'updateMode', 'compression']:
             iDict[key] = template[prefix+key]
-        elif key in ['xstep', 'ystep']:
-            iDict[key] = int(template[prefix+key])
         elif value:
             iDict[prefix+key] = template[prefix+key]
     print('processor : {}'.format(iDict['processor']))
@@ -182,8 +204,17 @@ def read_inps2dict(inps):
     if iDict['compression'] == False:
         iDict['compression'] = None
 
-    iDict['xstep'] = iDict.get('xstep', 1)
-    iDict['ystep'] = iDict.get('ystep', 1)
+    # group - multilook
+    prefix = 'mintpy.multilook.'
+    key_list = [i.split(prefix)[1] for i in template.keys() if i.startswith(prefix)]
+    for key in key_list:
+        value = template[prefix+key]
+        if key in ['xstep', 'ystep', 'method']:
+            iDict[key] = template[prefix+key]
+
+    iDict['xstep']  = int(iDict.get('xstep', 1))
+    iDict['ystep']  = int(iDict.get('ystep', 1))
+    iDict['method'] = str(iDict.get('method', 'nearest'))
 
     # PROJECT_NAME --> PLATFORM
     if not iDict['PROJECT_NAME']:
@@ -203,51 +234,47 @@ def read_inps2dict(inps):
     if iDict.get('autoPath', False):
         print('use auto path defined in mintpy.defaults.auto_path for options in auto')
         iDict = auto_path.get_auto_path(processor=iDict['processor'],
-                                        work_dir=os.path.dirname(iDict['outdir']),
+                                        work_dir=os.getcwd(),
                                         template=iDict)
 
-    # copy global var dsName2templateKey to iDict as a local var
-    iDict['ds_name2key'] = dict()
-    for key, value in datasetName2templateKey.items():
-        iDict['ds_name2key'][key] = value
-
     return iDict
 
 
 def read_subset_box(iDict):
     """read the following items:
-    geocoded
-    box
-    box4geo_lut
+    geocoded - bool, if the stack of observations geocoded or not
+    box      - tuple of 4 int, pixel box for stackObj and geomRadarObj, for obs in geo & radar coordinates
+    box4geo  - tuple of 4 int, pixel box for geomGeoObj, box4geo is the same as box, except for:
+               obs in radar coordinate with lookup table [for gamma and roipac], where box4geo is
+               the geo bounding box of the box above.
     """
     # Read subset info from template
     iDict['box'] = None
-    iDict['box4geo_lut'] = None
+    iDict['box4geo'] = None
     pix_box, geo_box = subset.read_subset_template2box(iDict['template_file'][0])
 
     # Grab required info to read input geo_box into pix_box
-    try:
-        lookupFile = [glob.glob(str(iDict['mintpy.load.lookupYFile']))[0],
-                      glob.glob(str(iDict['mintpy.load.lookupXFile']))[0]]
-    except:
-        lookupFile = None
+    lookup_y_files = glob.glob(str(iDict['mintpy.load.lookupYFile']))
+    lookup_x_files = glob.glob(str(iDict['mintpy.load.lookupXFile']))
+    if len(lookup_y_files) > 0 and len(lookup_x_files) > 0:
+        lookup_file = [lookup_y_files[0], lookup_x_files[0]]
+    else:
+        lookup_file = None
 
-    try:
-        pathKey = [i for i in iDict['ds_name2key'].values()
-                   if i in iDict.keys()][0]
-        file = glob.glob(str(iDict[pathKey]))[0]
-        atr = readfile.read_attribute(file)
-    except:
+    # use DEM file attribute as reference, because
+    # 1) it is required AND
+    # 2) it is in the same coordinate type as observation files
+    dem_files = glob.glob(iDict['mintpy.load.demFile'])
+    if len(dem_files) > 0:
+        atr = readfile.read_attribute(dem_files[0])
+    else:
         atr = dict()
 
-    geocoded = None
-    if 'Y_FIRST' in atr.keys():
-        geocoded = True
-    else:
-        geocoded = False
+    geocoded = True if 'Y_FIRST' in atr.keys() else False
+    iDict['geocoded'] = geocoded
 
     # Check conflict
-    if geo_box and not geocoded and lookupFile is None:
+    if geo_box and not geocoded and lookup_file is None:
         geo_box = None
         print(('WARNING: mintpy.subset.lalo is not supported'
                ' if 1) no lookup file AND'
@@ -266,7 +293,7 @@ def read_subset_box(iDict):
             return iDict
 
     # geo_box --> pix_box
-    coord = ut.coordinate(atr, lookup_file=lookupFile)
+    coord = ut.coordinate(atr, lookup_file=lookup_file)
     if geo_box is not None:
         pix_box = coord.bbox_geo2radar(geo_box)
         pix_box = coord.check_box_within_data_coverage(pix_box)
@@ -275,24 +302,24 @@ def read_subset_box(iDict):
 
     # Get box for geocoded lookup table (for gamma/roipac)
     box4geo_lut = None
-    if lookupFile is not None:
-        atrLut = readfile.read_attribute(lookupFile[0])
+    if lookup_file is not None:
+        atrLut = readfile.read_attribute(lookup_file[0])
         if not geocoded and 'Y_FIRST' in atrLut.keys():
             geo_box = coord.bbox_radar2geo(pix_box)
             box4geo_lut = ut.coordinate(atrLut).bbox_geo2radar(geo_box)
             print('box to read for geocoded lookup file in y/x: {}'.format(box4geo_lut))
 
-    iDict['geocoded'] = geocoded
     iDict['box'] = pix_box
-    iDict['box4geo_lut'] = box4geo_lut
+    iDict['box4geo'] = box4geo_lut if box4geo_lut else pix_box
     return iDict
 
 
+#################################################################
 def update_box4files_with_inconsistent_size(fnames):
     """Check the size (row / column number) of a list of files
     For SNAP geocoded products has one line missing in some interferograms, Andre, 2019-07-16
-    Parameters: fnames  : list of path for interferogram files
-    Returns:    pix_box : None if all files are in same size
+    Parameters: fnames  - list of path for interferogram files
+    Returns:    pix_box - None if all files are in same size
                           (0, 0, min_width, min_length) if not.
     """
     atr_list = [readfile.read_attribute(fname) for fname in fnames]
@@ -379,38 +406,51 @@ def skip_files_with_inconsistent_size(dsPathDict, pix_box=None, dsName='unwrapPh
     return dsPathDict
 
 
-def read_inps_dict2ifgram_stack_dict_object(iDict):
-    """Read input arguments into dict of ifgramStackDict object"""
+def read_inps_dict2ifgram_stack_dict_object(iDict, ds_name2template_key):
+    """Read input arguments into ifgramStackDict object.
+
+    Parameters: iDict                - dict, input arguments from command line & template file
+                ds_name2template_key - dict, to relate the HDF5 dataset name to the template key
+    Returns:    stackObj             - ifgramStackDict object or None
+    """
+    if iDict['only_load_geometry']:
+        return None
+
+    if 'mintpy.load.unwFile' in ds_name2template_key.values():
+        obs_type = 'interferogram'
+    elif 'mintpy.load.ionUnwFile' in ds_name2template_key.values():
+        obs_type = 'ionosphere'
+    elif 'mintpy.load.azOffFile' in ds_name2template_key.values():
+        obs_type = 'offset'
+
     # iDict --> dsPathDict
     print('-'*50)
-    print('searching interferometric pairs info')
+    print(f'searching {obs_type} pairs info')
     print('input data files:')
-    maxDigit = max([len(i) for i in list(iDict['ds_name2key'].keys())])
+    max_digit = max([len(i) for i in list(ds_name2template_key.keys())])
     dsPathDict = {}
-    for dsName in [i for i in ifgramDatasetNames
-                   if i in iDict['ds_name2key'].keys()]:
-        key = iDict['ds_name2key'][dsName]
+    for dsName in [i for i in ifgramDatasetNames if i in ds_name2template_key.keys()]:
+        key = ds_name2template_key[dsName]
         if key in iDict.keys():
             files = sorted(glob.glob(str(iDict[key])))
             if len(files) > 0:
                 dsPathDict[dsName] = files
-                print('{:<{width}}: {path}'.format(dsName,
-                                                   width=maxDigit,
-                                                   path=iDict[key]))
+                print(f'{dsName:<{max_digit}}: {iDict[key]}')
 
     # Check 1: required dataset
-    dsName0s = ['unwrapPhase', 'rangeOffset', 'azimuthOffset']
+    dsName0s = [x for x in OBS_DSET_NAMES if x in ds_name2template_key.keys()]
     dsName0 = [i for i in dsName0s if i in dsPathDict.keys()]
     if len(dsName0) == 0:
-        print('WARNING: No reqired {} data files found!'.format(dsName0s))
+        print(f'WARNING: No data files found for the required dataset: {dsName0s}! Skip loading for {obs_type} stack.')
         return None
     else:
         dsName0 = dsName0[0]
 
     # Check 2: data dimension for unwrapPhase files
-    dsPathDict = skip_files_with_inconsistent_size(dsPathDict,
-                                                   pix_box=iDict['box'],
-                                                   dsName=dsName0)
+    dsPathDict = skip_files_with_inconsistent_size(
+        dsPathDict=dsPathDict,
+        pix_box=iDict['box'],
+        dsName=dsName0)
 
     # Check 3: number of files for all dataset types
     # dsPathDict --> dsNumDict
@@ -418,7 +458,7 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
     for key in dsPathDict.keys():
         num_file = len(dsPathDict[key])
         dsNumDict[key] = num_file
-        print('number of {:<{width}}: {num}'.format(key, width=maxDigit, num=num_file))
+        print(f'number of {key:<{max_digit}}: {num_file}')
 
     dsNumList = list(dsNumDict.values())
     if any(i != dsNumList[0] for i in dsNumList):
@@ -456,7 +496,6 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
         # ifgramPathDict1 = {
         #     'unwrapPhase': /dirPathToFile/filt_fine.unw,
         #     'coherence'  : /dirPathToFile/filt_fine.cor,
-        #     'ionoPhase'  : /dirPathToFile/iono.bil,
         #     ...
         # }
         # All path of data file must contain the reference and secondary date, in file/dir name.
@@ -478,7 +517,6 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
 
                 if len(dsPath2) > 0:
                     ifgramPathDict[dsName] = dsPath2[0]
-
                 else:
                     print('WARNING: {:>18} file missing for pair {}'.format(dsName, date6s))
 
@@ -496,17 +534,23 @@ def read_inps_dict2ifgram_stack_dict_object(iDict):
     return stackObj
 
 
-def read_inps_dict2geometry_dict_object(iDict):
+def read_inps_dict2geometry_dict_object(iDict, dset_name2template_key):
+    """Read input arguments into geometryDict object(s).
+
+    Parameters: iDict        - dict, input arguments from command line & template file
+    Returns:    geomGeoObj   - geometryDict object in geo   coordinates or None
+                geomRadarObj - geometryDict object in radar coordinates or None
+    """
 
     # eliminate lookup table dsName for input files in radar-coordinates
     if iDict['processor'] in ['isce', 'doris']:
         # for processors with lookup table in radar-coordinates, remove azimuth/rangeCoord
-        iDict['ds_name2key'].pop('azimuthCoord')
-        iDict['ds_name2key'].pop('rangeCoord')
+        dset_name2template_key.pop('azimuthCoord')
+        dset_name2template_key.pop('rangeCoord')
     elif iDict['processor'] in ['roipac', 'gamma']:
         # for processors with lookup table in geo-coordinates, remove latitude/longitude
-        iDict['ds_name2key'].pop('latitude')
-        iDict['ds_name2key'].pop('longitude')
+        dset_name2template_key.pop('latitude')
+        dset_name2template_key.pop('longitude')
     elif iDict['processor'] in ['aria', 'gmtsar', 'hyp3', 'snap', 'cosicorr']:
         # for processors with geocoded products support only, do nothing for now.
         # check again when adding products support in radar-coordiantes
@@ -518,11 +562,10 @@ def read_inps_dict2geometry_dict_object(iDict):
     print('-'*50)
     print('searching geometry files info')
     print('input data files:')
-    maxDigit = max([len(i) for i in list(iDict['ds_name2key'].keys())])
+    max_digit = max([len(i) for i in list(dset_name2template_key.keys())])
     dsPathDict = {}
-    for dsName in [i for i in geometryDatasetNames
-                   if i in iDict['ds_name2key'].keys()]:
-        key = iDict['ds_name2key'][dsName]
+    for dsName in [i for i in geometryDatasetNames if i in dset_name2template_key.keys()]:
+        key = dset_name2template_key[dsName]
         if key in iDict.keys():
             files = sorted(glob.glob(str(iDict[key])))
             if len(files) > 0:
@@ -532,33 +575,30 @@ def read_inps_dict2geometry_dict_object(iDict):
                         date = ptime.yyyymmdd(os.path.basename(os.path.dirname(file)))
                         bperpDict[date] = file
                     dsPathDict[dsName] = bperpDict
-                    print('{:<{width}}: {path}'.format(dsName,
-                                                       width=maxDigit,
-                                                       path=iDict[key]))
-                    print('number of bperp files: {}'.format(len(list(bperpDict.keys()))))
+                    print(f'{dsName:<{max_digit}}: {iDict[key]}')
+                    print(f'number of bperp files: {len(list(bperpDict.keys()))}')
                 else:
                     dsPathDict[dsName] = files[0]
-                    print('{:<{width}}: {path}'.format(dsName,
-                                                       width=maxDigit,
-                                                       path=files[0]))
+                    print(f'{dsName:<{max_digit}}: {files[0]}')
 
     # Check required dataset
     dsName0 = geometryDatasetNames[0]
     if dsName0 not in dsPathDict.keys():
         print('WARNING: No reqired {} data files found!'.format(dsName0))
 
-    # metadata
-    ifgramMetaGeo = None
-    ifgramMetaRadar = None
-    ifgramKey = iDict['ds_name2key']['unwrapPhase']
-    if ifgramKey in iDict.keys():
-        ifgramFiles = glob.glob(str(iDict[ifgramKey]))
-        if len(ifgramFiles) > 0:
-            atr = readfile.read_attribute(ifgramFiles[0])
+    # extra metadata from observations
+    # e.g. EARTH_RADIUS, HEIGHT, etc.
+    obsMetaGeo = None
+    obsMetaRadar = None
+    for obsName in OBS_DSET_NAMES:
+        obsFiles = sorted(glob.glob(iDict[dset_name2template_key[obsName]]))
+        if len(obsFiles) > 0:
+            atr = readfile.read_attribute(obsFiles[0])
             if 'Y_FIRST' in atr.keys():
-                ifgramMetaGeo = atr.copy()
+                obsMetaGeo = atr.copy()
             else:
-                ifgramMetaRadar = atr.copy()
+                obsMetaRadar = atr.copy()
+            break
 
     # dsPathDict --> dsGeoPathDict + dsRadarPathDict
     dsNameList = list(dsPathDict.keys())
@@ -574,56 +614,93 @@ def read_inps_dict2geometry_dict_object(iDict):
         else:
             dsRadarPathDict[dsName] = dsPathDict[dsName]
 
-    geomRadarObj = None
     geomGeoObj = None
-    if len(dsRadarPathDict) > 0:
-        geomRadarObj = geometryDict(processor=iDict['processor'],
-                                    datasetDict=dsRadarPathDict,
-                                    extraMetadata=ifgramMetaRadar)
+    geomRadarObj = None
     if len(dsGeoPathDict) > 0:
-        geomGeoObj = geometryDict(processor=iDict['processor'],
-                                  datasetDict=dsGeoPathDict,
-                                  extraMetadata=ifgramMetaGeo)
-    return geomRadarObj, geomGeoObj
+        geomGeoObj = geometryDict(
+            processor=iDict['processor'],
+            datasetDict=dsGeoPathDict,
+            extraMetadata=obsMetaGeo)
+    if len(dsRadarPathDict) > 0:
+        geomRadarObj = geometryDict(
+            processor=iDict['processor'],
+            datasetDict=dsRadarPathDict,
+            extraMetadata=obsMetaRadar)
+
+    return geomGeoObj, geomRadarObj
 
 
-def update_object(outFile, inObj, box, updateMode=True, xstep=1, ystep=1):
-    """Do not write h5 file if: 1) h5 exists and readable,
-                                2) it contains all date12 from ifgramStackDict,
-                                            or all datasets from geometryDict"""
-    write_flag = True
-    if updateMode and ut.run_or_skip(outFile, check_readable=True) == 'skip':
+#################################################################
+def run_or_skip(outFile, inObj, box, updateMode=True, xstep=1, ystep=1, geom_obj=None):
+    """Check if re-writing is necessary.
+
+    Do not write HDF5 file if ALL the following meet:
+        1. HDF5 file exists and is readable,
+        2. HDF5 file constains all the datasets and in the same size
+        3. For ifgramStackDict, HDF5 file contains all date12.
+
+    Parameters: outFile    - str, path to the output HDF5 file
+                inObj      - ifgramStackDict or geometryDict, object to write
+                box        - tuple of int, bounding box in (x0, y0, x1, y1)
+                updateMode - bool
+                x/ystep    - int
+                geom_obj   - geometryDict object or None, for ionosphere only
+    Returns:    flag       - str, run or skip
+    """
+
+    flag = 'run'
+
+    # skip if there is no dict object to write
+    if not inObj:
+        flag = 'skip'
+        return flag
+
+    # run if not in update mode
+    if not updateMode:
+        return flag
+
+    if ut.run_or_skip(outFile, check_readable=True) == 'skip':
+        kwargs = dict(box=box, xstep=xstep, ystep=ystep)
+
         if inObj.name == 'ifgramStack':
-            in_size = inObj.get_size(box=box, xstep=xstep, ystep=ystep)[1:]
+            in_size = inObj.get_size(geom_obj=geom_obj, **kwargs)[1:]
+            in_dset_list = inObj.get_dataset_list()
             in_date12_list = inObj.get_date12_list()
 
             outObj = ifgramStack(outFile)
-            out_size = outObj.get_size()[1:]
-            out_date12_list = outObj.get_date12_list(dropIfgram=False)
+            outObj.open(print_msg=False)
+            out_size = (outObj.length, outObj.width)
+            out_dset_list = outObj.datasetNames
+            out_date12_list = outObj.date12List
 
-            if out_size == in_size and set(in_date12_list).issubset(set(out_date12_list)):
+            if (out_size == in_size
+                    and set(in_dset_list).issubset(set(out_dset_list))
+                    and set(in_date12_list).issubset(set(out_date12_list))):
                 print(('All date12   exists in file {} with same size as required,'
                        ' no need to re-load.'.format(os.path.basename(outFile))))
-                write_flag = False
+                flag = 'skip'
 
         elif inObj.name == 'geometry':
-            in_size = inObj.get_size(box=box, xstep=xstep, ystep=ystep)
+            in_size = inObj.get_size(**kwargs)
             in_dset_list = inObj.get_dataset_list()
 
             outObj = geometry(outFile)
             outObj.open(print_msg=False)
-            out_size = outObj.get_size()
+            out_size = (outObj.length, outObj.width)
             out_dset_list = outObj.datasetNames
 
-            if out_size == in_size and set(in_dset_list).issubset(set(out_dset_list)):
+            if (out_size == in_size
+                    and set(in_dset_list).issubset(set(out_dset_list))):
                 print(('All datasets exists in file {} with same size as required,'
                        ' no need to re-load.'.format(os.path.basename(outFile))))
-                write_flag = False
+                flag = 'skip'
 
-    return write_flag
+    return flag
 
 
 def prepare_metadata(iDict):
+    """Prepare metadata via prep_{insar_processor}.py scripts."""
+
     processor = iDict['processor']
     script_name = 'prep_{}.py'.format(processor)
     print('-'*50)
@@ -643,7 +720,10 @@ def prepare_metadata(iDict):
             from mintpy import prep_cosicorr as prep_module
 
         # run prep_{processor} module
-        for key in [i for i in iDict.keys() if (i.startswith('mintpy.load.') and i.endswith('File') and i != 'mintpy.load.metaFile')]:
+        for key in [i for i in iDict.keys()
+                    if (i.startswith('mintpy.load.')
+                        and i.endswith('File')
+                        and i != 'mintpy.load.metaFile')]:
             if len(glob.glob(str(iDict[key]))) > 0:
                 # print command line
                 script_name = '{}.py'.format(os.path.basename(prep_module.__name__).split('.')[-1])
@@ -652,15 +732,15 @@ def prepare_metadata(iDict):
                     iargs += ['--sensor', iDict['PLATFORM'].lower()]
                 elif processor == 'cosicorr':
                     iargs += ['--metadata', iDict['mintpy.load.metaFile']]
-                print(script_name, ' '.join(iargs))
+                ut.print_command_line(script_name, iargs)
                 # run
                 prep_module.main(iargs)
 
     elif processor == 'isce':
         from mintpy import prep_isce
-        from mintpy.utils.isce_utils import get_processor
+        from mintpy.utils import s1_utils, isce_utils
 
-        # metadata
+        # --meta-file
         meta_files = sorted(glob.glob(iDict['mintpy.load.metaFile']))
         if len(meta_files) > 0:
             meta_file = meta_files[0]
@@ -668,53 +748,58 @@ def prepare_metadata(iDict):
             warnings.warn('No input metadata file found: {}'.format(iDict['mintpy.load.metaFile']))
             meta_file = 'auto'
 
-        # auxliary data
+        # --baseline-dir / --geometry-dir
         baseline_dir = iDict['mintpy.load.baselineDir']
         geom_dir = os.path.dirname(iDict['mintpy.load.demFile'])
 
-        # observation
-        obs_keys = ['mintpy.load.unwFile', 'mintpy.load.rgOffFile', 'mintpy.load.azOffFile']
-        obs_keys = [i for i in obs_keys if i in iDict.keys()]
+        # --dset-dir / --file-pattern
+        obs_keys = [
+            'mintpy.load.unwFile',
+            'mintpy.load.ionUnwFile',
+            'mintpy.load.rgOffFile',
+            'mintpy.load.azOffFile',
+        ]
         obs_paths = [iDict[key] for key in obs_keys if iDict[key].lower() != 'auto']
-        if len(obs_paths) > 0:
-            processor = get_processor(meta_file) if os.path.isfile(meta_file) else 'topsStack'
-            if processor == 'alosStack':
-                obs_dir = os.path.dirname(obs_paths[0])
-            else:
-                obs_dir = os.path.dirname(os.path.dirname(obs_paths[0]))
-            obs_file = os.path.basename(obs_paths[0])
-        else:
-            obs_dir = None
-            obs_file = None
+        obs_paths = [x for x in obs_paths if len(glob.glob(x)) > 0]
 
-        # geometry
+        # --geom-files for the basenames only
         geom_names = ['dem', 'lookupY', 'lookupX', 'incAngle', 'azAngle', 'shadowMask', 'waterMask']
         geom_keys = ['mintpy.load.{}File'.format(i) for i in geom_names]
         geom_files = [os.path.basename(iDict[key]) for key in geom_keys
-                      if (iDict[key] and iDict[key].lower() != 'auto')]
+                      if iDict.get(key, 'auto') not in ['auto', 'None', 'no',  None, False]]
 
         # compose list of input arguments
         iargs = ['-m', meta_file, '-g', geom_dir]
         if baseline_dir:
             iargs += ['-b', baseline_dir]
-        if obs_dir is not None:
-            iargs += ['-d', obs_dir, '-f', obs_file]
+        if len(obs_paths) > 0:
+            iargs += ['-f'] + obs_paths
         if geom_files:
             iargs += ['--geom-files'] + geom_files
 
         # run module
-        print('prep_isce.py', ' '.join(iargs))
+        ut.print_command_line(script_name, iargs)
         try:
             prep_isce.main(iargs)
         except:
             warnings.warn('prep_isce.py failed. Assuming its result exists and continue...')
 
+        # [optional] for topsStack: SAFE_files.txt --> S1A/B_date.txt
+        if os.path.isfile(meta_file) and isce_utils.get_processor(meta_file) == 'topsStack':
+            safe_list_file = os.path.join(os.path.dirname(os.path.dirname(meta_file)), 'SAFE_files.txt')
+            if os.path.isfile(safe_list_file):
+                s1_utils.get_s1ab_date_list_file(
+                    mintpy_dir=os.getcwd(),
+                    safe_list_file=safe_list_file,
+                    print_msg=True)
+
     elif processor == 'aria':
         from mintpy import prep_aria
 
         ## compose input arguments
         # use the default template file if exists & input
-        default_temp_files = [fname for fname in iDict['template_file'] if fname.endswith('smallbaselineApp.cfg')]
+        default_temp_files = [fname for fname in iDict['template_file']
+                              if fname.endswith('smallbaselineApp.cfg')]
         if len(default_temp_files) > 0:
             temp_file = default_temp_files[0]
         else:
@@ -750,7 +835,7 @@ def prepare_metadata(iDict):
             iargs += ['--update']
 
         ## run
-        print('prep_aria.py', ' '.join(iargs))
+        ut.print_command_line(script_name, iargs)
         try:
             prep_aria.main(iargs)
         except:
@@ -760,13 +845,14 @@ def prepare_metadata(iDict):
         from mintpy import prep_gmtsar
 
         # use the custom template file if exists & input
-        custom_temp_files = [fname for fname in iDict['template_file'] if not fname.endswith('smallbaselineApp.cfg')]
+        custom_temp_files = [fname for fname in iDict['template_file']
+                             if not fname.endswith('smallbaselineApp.cfg')]
         if len(custom_temp_files) == 0:
             raise FileExistsError('Custom template file NOT found and is required for GMTSAR!')
 
         # run prep_*.py
-        iargs = [custom_temp_files[0], '--mintpy-dir', os.path.dirname(iDict['outdir'])]
-        print('prep_gmtsar.py', ' '.join(iargs))
+        iargs = [custom_temp_files[0]]
+        ut.print_command_line(script_name, iargs)
         try:
             prep_gmtsar.main(iargs)
         except:
@@ -780,28 +866,12 @@ def prepare_metadata(iDict):
     return
 
 
-def print_write_setting(iDict):
-    updateMode = iDict['updateMode']
-    comp = iDict['compression']
-    print('-'*50)
-    print('updateMode : {}'.format(updateMode))
-    print('compression: {}'.format(comp))
-    print('x/ystep: {}/{}'.format(iDict['xstep'], iDict['ystep']))
-
-    # box
-    box = iDict['box']
-    # box for geometry file in geo-coordinates
-    if not iDict.get('geocoded', False):
-        boxGeo = iDict['box4geo_lut']
-    else:
-        boxGeo = box
-
-    return updateMode, comp, box, boxGeo
-
-
 def get_extra_metadata(iDict):
     """Extra metadata with key names in MACRO_CASE to be written into stack file.
-    E.g.: PROJECT_NAME, PLATFORM, ORBIT_DIRECTION, SUBSET_X/YMIN, etc.
+
+    Parameters: iDict     - dict, input arguments from command lines & template file
+                extraDict - dict, extra metadata from template file:
+                            E.g. PROJECT_NAME, PLATFORM, ORBIT_DIRECTION, SUBSET_X/YMIN, etc.
     """
     extraDict = {}
     # all keys in MACRO_CASE
@@ -824,70 +894,91 @@ def main(iargs=None):
     # read input options
     iDict = read_inps2dict(inps)
 
-    # prepare metadata
+    ## prepare metadata
     prepare_metadata(iDict)
+    extraDict = get_extra_metadata(iDict)
 
     # skip data writing for aria as it is included in prep_aria
     if iDict['processor'] == 'aria':
         return
 
+    ## search & write data files
+    print('-'*50)
+    print('updateMode : {}'.format(iDict['updateMode']))
+    print('compression: {}'.format(iDict['compression']))
+    print('multilook x/ystep: {}/{}'.format(iDict['xstep'], iDict['ystep']))
+    print('multilook method : {}'.format(iDict['method']))
+    kwargs = dict(updateMode=iDict['updateMode'], xstep=iDict['xstep'], ystep=iDict['ystep'])
+
+    # read subset info [need the metadata from above]
     iDict = read_subset_box(iDict)
-    extraDict = get_extra_metadata(iDict)
 
-    # initiate objects
-    stackObj = read_inps_dict2ifgram_stack_dict_object(iDict)
-    geomRadarObj, geomGeoObj = read_inps_dict2geometry_dict_object(iDict)
+    # geometry in geo / radar coordinates
+    geom_dset_name2template_key = {
+        **GEOM_DSET_NAME2TEMPLATE_KEY,
+        **IFG_DSET_NAME2TEMPLATE_KEY,
+        **OFF_DSET_NAME2TEMPLATE_KEY,
+    }
+    geom_geo_obj, geom_radar_obj = read_inps_dict2geometry_dict_object(iDict, geom_dset_name2template_key)
+    geom_geo_file = os.path.abspath('./inputs/geometryGeo.h5')
+    geom_radar_file = os.path.abspath('./inputs/geometryRadar.h5')
 
-    # prepare write
-    updateMode, comp, box, boxGeo = print_write_setting(iDict)
-    if any([stackObj, geomRadarObj, geomGeoObj]) and not os.path.isdir(inps.outdir):
-        os.makedirs(inps.outdir)
-        print('create directory: {}'.format(inps.outdir))
+    if run_or_skip(geom_geo_file, geom_geo_obj, iDict['box4geo'], **kwargs) == 'run':
+        geom_geo_obj.write2hdf5(
+            outputFile=geom_geo_file,
+            access_mode='w',
+            box=iDict['box4geo'],
+            xstep=iDict['xstep'],
+            ystep=iDict['ystep'],
+            compression='lzf')
 
-    # write
-    if stackObj and update_object(inps.outfile[0], stackObj, box,
-                                  updateMode=updateMode,
-                                  xstep=iDict['xstep'],
-                                  ystep=iDict['ystep']):
-        print('-'*50)
-        stackObj.write2hdf5(outputFile=inps.outfile[0],
-                            access_mode='w',
-                            box=box,
-                            xstep=iDict['xstep'],
-                            ystep=iDict['ystep'],
-                            compression=comp,
-                            extra_metadata=extraDict)
+    if run_or_skip(geom_radar_file, geom_radar_obj, iDict['box'], **kwargs) == 'run':
+        geom_radar_obj.write2hdf5(
+            outputFile=geom_radar_file,
+            access_mode='w',
+            box=iDict['box'],
+            xstep=iDict['xstep'],
+            ystep=iDict['ystep'],
+            compression='lzf',
+            extra_metadata=extraDict)
 
-    if geomRadarObj and update_object(inps.outfile[1], geomRadarObj, box,
-                                      updateMode=updateMode,
-                                      xstep=iDict['xstep'],
-                                      ystep=iDict['ystep']):
-        print('-'*50)
-        geomRadarObj.write2hdf5(outputFile=inps.outfile[1],
-                                access_mode='w',
-                                box=box,
-                                xstep=iDict['xstep'],
-                                ystep=iDict['ystep'],
-                                compression='lzf',
-                                extra_metadata=extraDict)
+    # observations: ifgram, ion or offset
+    # loop over obs stacks
+    stack_ds_name2tmpl_key_list = [
+        IFG_DSET_NAME2TEMPLATE_KEY,
+        ION_DSET_NAME2TEMPLATE_KEY,
+        OFF_DSET_NAME2TEMPLATE_KEY,
+    ]
+    stack_files = ['ifgramStack.h5', 'ionStack.h5', 'offsetStack.h5']
+    stack_files = [os.path.abspath(os.path.join('./inputs', x)) for x in stack_files]
+    for ds_name2tmpl_opt, stack_file in zip(stack_ds_name2tmpl_key_list, stack_files):
 
-    if geomGeoObj and update_object(inps.outfile[2], geomGeoObj, boxGeo,
-                                    updateMode=updateMode,
-                                    xstep=iDict['xstep'],
-                                    ystep=iDict['ystep']):
-        print('-'*50)
-        geomGeoObj.write2hdf5(outputFile=inps.outfile[2],
-                              access_mode='w',
-                              box=boxGeo,
-                              xstep=iDict['xstep'],
-                              ystep=iDict['ystep'],
-                              compression='lzf')
+        # initiate dict objects
+        stack_obj = read_inps_dict2ifgram_stack_dict_object(iDict, ds_name2tmpl_opt)
+
+        # use geom_obj as size reference while loading ionosphere
+        geom_obj = None
+        if os.path.basename(stack_file).startswith('ion'):
+            geom_obj = geom_geo_obj if iDict['geocoded'] else geom_radar_obj
+
+        # write dict objects to HDF5 files
+        if run_or_skip(stack_file, stack_obj, iDict['box'], geom_obj=geom_obj, **kwargs) == 'run':
+            stack_obj.write2hdf5(
+                outputFile=stack_file,
+                access_mode='w',
+                box=iDict['box'],
+                xstep=iDict['xstep'],
+                ystep=iDict['ystep'],
+                mli_method=iDict['method'],
+                compression=iDict['compression'],
+                extra_metadata=extraDict,
+                geom_obj=geom_obj)
 
     # time info
     m, s = divmod(time.time()-start_time, 60)
     print('time used: {:02.0f} mins {:02.1f} secs.\n'.format(m, s))
 
-    return inps.outfile
+    return
 
 
 #################################################################
