@@ -12,34 +12,36 @@
 
 import os
 import sys
-import argparse
 import numpy as np
+
 from mintpy.objects import timeseries
 from mintpy.defaults.template import get_template_content
 from mintpy.utils import readfile, writefile, ptime
+from mintpy.utils.arg_utils import create_argument_parser
 
 
 #########################################################################################
 TEMPLATE = get_template_content('correct_LOD')
 
-EXAMPLE = """example:
-  local_oscilator_drift.py  timeseries.h5                 inputs/geometryRadar.h5
-  local_oscilator_drift.py  filt_101020_110220_4rlks.unw  inputs/geometryRadar.h5
-"""
-
 REFERENCE = """reference:
   Marinkovic, P., and Y. Larsen (2013), Consequences of long-term ASAR local oscillator 
   frequency decay - An empirical study of 10 years of data, in Living Planet Symposium,
   Edinburgh, U.K.
 """
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Local Oscilator Drift (LOD) correction of Envisat',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog='{}\n{}\n{}'.format(REFERENCE, TEMPLATE, EXAMPLE))
+EXAMPLE = """example:
+  local_oscilator_drift.py  timeseries.h5                 inputs/geometryRadar.h5
+  local_oscilator_drift.py  filt_101020_110220_4rlks.unw  inputs/geometryRadar.h5
+"""
 
-    parser.add_argument(
-        dest='file', help='timeseries / interferograms file, i.e. timeseries.h5')
+def create_parser(subparsers=None):
+    synopsis = 'Local Oscilator Drift (LOD) correction of Envisat'
+    epilog = REFERENCE + '\n' + TEMPLATE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
+
+    parser.add_argument(dest='file', help='timeseries / interferograms file, i.e. timeseries.h5')
     parser.add_argument(dest='range_dist_file',
                         help='Slant range distance file, i.e. inputs/geometryRadar.h5, inputs/geometryGeo.h5\n' +
                         'or use range_distance.py to generate it.')
@@ -55,11 +57,11 @@ def cmd_line_parse(iargs=None):
 
 
 #########################################################################################
-def get_relative_range_distance(metadata):
-    length, width = int(metadata['LENGTH']), int(metadata['WIDTH'])
-    range_dist_1d = float(metadata['RANGE_PIXEL_SIZE']) * np.linspace(0, width-1, width)
+def get_relative_range_distance(meta):
+    length, width = int(meta['LENGTH']), int(meta['WIDTH'])
+    range_dist_1d = float(meta['RANGE_PIXEL_SIZE']) * np.linspace(0, width-1, width)
     range_dist = np.tile(range_dist_1d, (length, 1))
-    range_dist -= range_dist[int(atr['REF_Y']), int(atr['REF_X'])]
+    range_dist -= range_dist[int(meta['REF_Y']), int(meta['REF_X'])]
     return range_dist
 
 
@@ -88,6 +90,7 @@ def correct_local_oscilator_drift(fname, rg_dist_file=None, out_file=None):
         print('read range distance from file: %s' % (rg_dist_file))
         rg_dist = readfile.read(rg_dist_file, datasetName='slantRangeDistance', print_msg=False)[0]
         rg_dist -= rg_dist[int(atr['REF_Y']), int(atr['REF_X'])]
+
     ramp_rate = np.array(rg_dist * 3.87e-7, np.float32)
 
     # Correct LOD Ramp for Input fname
