@@ -9,9 +9,9 @@
 import os
 import sys
 import shutil
-import argparse
 import numpy as np
 from mintpy.utils import readfile, writefile
+from mintpy.utils.arg_utils import create_argument_parser
 
 
 ############################################################
@@ -25,10 +25,12 @@ EXAMPLE = """example:
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Mask file',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Mask file'
+    epilog = EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('file', help='File to be masked')
     parser.add_argument('-m', '--mask', dest='mask_file', required=True,
@@ -220,10 +222,10 @@ def main(iargs=None):
     inps = cmd_line_parse(iargs)
 
     ext = os.path.splitext(inps.file)[1]
-    if os.path.isfile(inps.file+'.xml') and ext in ['.unw','.int','.cor','.conncomp']:
-        mask_isce_file(inps.file, inps.mask_file, inps.outfile)
-    else:
-        mask_file(inps.file, inps.mask_file, inps.outfile, inps)
+    #if os.path.isfile(inps.file+'.xml') and ext in ['.unw','.int','.cor','.conncomp']:
+    #    mask_isce_file(inps.file, inps.mask_file, inps.outfile)
+    #else:
+    mask_file(inps.file, inps.mask_file, inps.outfile, inps)
 
     print('Done.')
     return
