@@ -8,7 +8,6 @@
 
 import os
 import sys
-import argparse
 import h5py
 import numpy as np
 from matplotlib import pyplot as plt, dates as mdates
@@ -22,9 +21,12 @@ from mintpy.utils import (
     network as pnet,
     plot as pp,
 )
+from mintpy.utils.arg_utils import create_argument_parser
 
 
 ###############################  Usage  ################################
+TEMPLATE = get_template_content('modify_network')
+
 REFERENCE = """reference:
   Yunjun, Z., Fattahi, H. and Amelung, F. (2019), Small baseline InSAR time series analysis:
   Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
@@ -40,8 +42,6 @@ REFERENCE = """reference:
   California. Remote Sensing of Environment, 258, 112400. doi:10.1016/j.rse.2021.112400
 """
 
-TEMPLATE = get_template_content('modify_network')
-
 EXAMPLE = """example:
   modify_network.py inputs/ifgramStack.h5 -t smallbaselineApp.cfg
   modify_network.py inputs/ifgramStack.h5 --reset
@@ -49,17 +49,20 @@ EXAMPLE = """example:
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Modify the network of interferograms',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=REFERENCE+'\n'+TEMPLATE+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Modify the network of interferograms'
+    epilog = REFERENCE + '\n' + TEMPLATE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
+
     parser.add_argument('file', help='Files to modify/drop network, e.g. inputs/ifgramStack.h5.')
     parser.add_argument('-t', '--template', dest='template_file',
                         help='Template file with input options')
     parser.add_argument('--reset', action='store_true',
                         help='restore all interferograms in the file, by marking all dropIfgram=True')
     parser.add_argument('--noaux', dest='update_aux', action='store_false',
-                        help='Do not update auxilary files, e.g.\n' +
+                        help='Do not update auxiliary files, e.g.\n' +
                              'maskConnComp.h5 or avgSpatialCoh.h5 from ifgramStack.h5')
 
     # 1. temp/perp baseline, num of conn., dates, pair index, etc.
@@ -169,7 +172,7 @@ def read_template2inps(template_file, inps=None):
         inps = cmd_line_parse()
     inpsDict = vars(inps)
     print('read options from template file: '+os.path.basename(template_file))
-    template = readfile.read_template(inps.template_file)
+    template = readfile.read_template(inps.template_file, skip_chars=['[', ']'])
     template = ut.check_template_auto_value(template)
 
     # Update inps if key existed in template file
@@ -187,12 +190,12 @@ def read_template2inps(template_file, inps=None):
             elif key in ['maskFile', 'referenceFile']:
                 inpsDict[key] = value
             elif key == 'aoiYX':
-                tmp = [i.replace('[','').replace(']','').strip() for i in value.split(',')]
+                tmp = [i.strip() for i in value.split(',')]
                 sub_y = sorted([int(i.strip()) for i in tmp[0].split(':')])
                 sub_x = sorted([int(i.strip()) for i in tmp[1].split(':')])
                 inps.aoi_pix_box = (sub_x[0], sub_y[0], sub_x[1], sub_y[1])
             elif key == 'aoiLALO':
-                tmp = [i.replace('[','').replace(']','').strip() for i in value.split(',')]
+                tmp = [i.strip() for i in value.split(',')]
                 sub_lat = sorted([float(i.strip()) for i in tmp[0].split(':')])
                 sub_lon = sorted([float(i.strip()) for i in tmp[1].split(':')])
                 inps.aoi_geo_box = (sub_lon[0], sub_lat[1], sub_lon[1], sub_lat[0])
@@ -204,11 +207,9 @@ def read_template2inps(template_file, inps=None):
             elif key in ['startDate', 'endDate']:
                 inpsDict[key] = ptime.yyyymmdd(value)
             elif key == 'excludeDate':
-                value = value.replace('[','').replace(']','').replace(',', ' ')
-                inpsDict[key] = ptime.yyyymmdd(value.split())
+                inpsDict[key] = ptime.yyyymmdd(value.split(','))
             elif key == 'excludeIfgIndex':
-                value = value.replace('[','').replace(']','').replace(',', ' ')
-                inpsDict[key] += value.split()
+                inpsDict[key] += value.split(',')
                 inpsDict[key] = read_input_index_list(inpsDict[key], stackFile=inps.file)
 
     # Turn reset on if 1) no input options found to drop ifgram AND 2) there is template input
