@@ -23,10 +23,11 @@ from mintpy.utils import ptime, time_func, readfile, utils1 as ut
 unr_site_list_file = 'http://geodesy.unr.edu/NGLStationPages/DataHoldings.txt'
 
 
-def dload_site_list(print_msg=True):
+def dload_site_list(out_file=None, print_msg=True):
     """download DataHoldings.txt"""
     url = unr_site_list_file
-    out_file = os.path.basename(url)
+    if not out_file:
+        out_file = os.path.basename(url)
     if print_msg:
         print('downloading site list from UNR Geod Lab: {}'.format(url))
     urlretrieve(url, out_file)
@@ -47,8 +48,9 @@ def search_gps(SNWE, start_date=None, end_date=None, site_list_file=None, min_nu
     # download site list file if it's not found in current directory
     if site_list_file is None:
         site_list_file = os.path.basename(unr_site_list_file)
-        if not os.path.isfile(site_list_file):
-            dload_site_list(print_msg=print_msg)
+
+    if not os.path.isfile(site_list_file):
+        dload_site_list(site_list_file, print_msg=print_msg)
 
     txt_data = np.loadtxt(site_list_file,
                           dtype=bytes,
@@ -159,6 +161,16 @@ def get_gps_los_obs(meta, obs_type, site_names, start_date, end_date, gps_comp='
         fc = np.genfromtxt(csv_file, dtype=col_types, delimiter=',', names=True)
         site_obs = fc[col_names[obs_ind]]
 
+        # get obs for the input site names only
+        # in case the site_names are not consistent with the CSV file.
+        if num_row != num_site:
+            temp_names = fc[col_names[0]]
+            temp_obs = np.array(site_obs, dtype=np.float32)
+            site_obs = np.zeros(num_site, dtype=np.float32) * np.nan
+            for i, site_name in enumerate(site_names):
+                if site_name in temp_names:
+                    site_obs[i] = temp_obs[temp_names == site_name][0]
+
     else:
         # calculate and save to CSV file
         data_list = []
@@ -451,11 +463,11 @@ class GPS:
         elif gps_comp in ['u2los', 'up2los']:
             unit_vec[0] = 0.
             unit_vec[1] = 0.
-        elif gps_comp in ['horz']:
+        elif gps_comp in ['horz','horizontal']:
             unit_vec[0] = np.sin(horz_az_angle) * -1
             unit_vec[1] = np.cos(horz_az_angle)
             unit_vec[2] = 0.
-        elif gps_comp in ['vertical']:
+        elif gps_comp in ['vert','vertical']:
             unit_vec[0] = 0.
             unit_vec[1] = 0.
             unit_vec[2] = 1.
