@@ -7,16 +7,15 @@
 #     from mintpy.objects.resample import resample
 
 
-try:
-    import pyresample as pr
-except ImportError:
-    raise ImportError('Can not import pyresample!')
-
 import os
+import warnings
 import h5py
 import numpy as np
 from scipy import ndimage
 from scipy.interpolate import RegularGridInterpolator as RGI
+with warnings.catch_warnings():
+    warnings.simplefilter('ignore', UserWarning)
+    import pyresample as pr
 from mintpy.objects.cluster import split_box2sub_boxes
 from mintpy.utils import readfile, ptime, utils0 as ut
 
@@ -339,8 +338,19 @@ class resample:
 
             # parameter 1 - lalo_step (output grid)
             if self.lalo_step is None:
-                self.lalo_step = ((src_lat1 - src_lat0) / (lut_lat.shape[0] - 1),
-                                  (src_lon1 - src_lon0) / (lut_lat.shape[1] - 1))
+                try:
+                    # ensure the same pixel area before / after geocoding
+                    merged_meta = {**self.lut_meta, **self.src_meta}
+                    lat_c = (src_lat0 + src_lat1) / 2.
+                    lat_step, lon_step = ut.auto_lat_lon_step_size(merged_meta, lat_c)
+
+                except KeyError:
+                    # ensure the same matrix shape before / after geocoding
+                    # if not enough metadata found for the above
+                    lat_step = (src_lat1 - src_lat0) / (lut_lat.shape[0] - 1)
+                    lon_step = (src_lon1 - src_lon0) / (lut_lat.shape[1] - 1)
+                self.lalo_step = (abs(lat_step) * -1., abs(lon_step))
+
             else:
                 # ensure lat/lon step sign
                 self.lalo_step = (abs(self.lalo_step[0]) * -1.,
