@@ -6,13 +6,12 @@
 ############################################################
 
 
-import os
 import sys
 import argparse
 import numpy as np
 from matplotlib import pyplot as plt, ticker
 
-from mintpy.utils import arg_group, readfile, utils as ut, plot as pp
+from mintpy.utils import arg_utils, readfile, utils as ut, plot as pp
 from mintpy import view
 
 
@@ -29,17 +28,21 @@ EXAMPLE = """example:
   plot_transection.py velocity.h5 --start-lalo 30.125 129.988 --end-lalo 30.250 130.116
   plot_transection.py velocity.h5 --line-file  transect_lonlat.xy --dem gsi10m.dem
 
-  # Multiple files
+  # multiple files
   plot_transection.py AlosA*/velocity.h5 AlosD*/velocity.h5 --off 2
   plot_transection.py Kirishima2017*.h5 Kirishima2008*.h5 --off 0 0 10 10
   plot_transection.py Kirishima2017*.h5 Kirishima2008*.h5 --off 0 0 10 10 --lalo0 31.947 130.843 --lalo1 31.947 130.860
+
+  # interactive plot: click two points to draw a profile
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Generate transect/profile along a line',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Generate transect/profile along a line'
+    epilog = EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = arg_utils.create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('file', nargs='+',
                         help='input file to show transection')
@@ -71,14 +74,19 @@ def create_parser():
     lines.add_argument('--ms', '--markersize', dest='marker_size', type=float, default=2.0,
                        help='Point marker size. Default: 2.0')
 
-    parser = arg_group.add_figure_argument(parser)
-    parser = arg_group.add_save_argument(parser)
+    parser = arg_utils.add_figure_argument(parser)
+    parser = arg_utils.add_save_argument(parser)
     return parser
 
 
 def cmd_line_parse(iargs=None):
     inps = create_parser().parse_args(args=iargs)
 
+    # save argv (to check the manually specified arguments)
+    # use iargs        for python call
+    # use sys.argv[1:] for command line call
+    inps.argv = iargs if iargs else sys.argv[1:]
+
     if inps.outfile or not inps.disp_fig:
         inps.save_fig = True
 
@@ -158,8 +166,8 @@ def get_view_cmd(iargs):
                         help='Display limits for matrix plotting.')
     parser.add_argument('--noverbose', dest='print_msg', action='store_false',
                         help='Disable the verbose message printing.')
-    parser = arg_group.add_figure_argument(parser)
-    parser = arg_group.add_save_argument(parser)
+    parser = arg_utils.add_figure_argument(parser)
+    parser = arg_utils.add_save_argument(parser)
 
     # get args that are applicable to view.py
     unique_args = parser.parse_known_args(iargs)[1]
@@ -198,7 +206,7 @@ class transectionViewer():
         self.ax_txn = None
 
         self.img = None
-        self.line = None
+        self.line_ann = None
         self.pts_idx = 0
         return
 
@@ -209,7 +217,7 @@ class transectionViewer():
             setattr(self, key, value)
 
         # copy inps from view.py to self object
-        view_cmd = get_view_cmd(self.iargs)
+        view_cmd = get_view_cmd(self.argv)
         self.data_img, atr, inps_view = view.prep_slice(view_cmd)
         for key, value in inps_view.__dict__.items():
             setattr(self, key, value)
@@ -253,7 +261,7 @@ class transectionViewer():
             self.draw_line(self.start_yx, self.end_yx)
             self.draw_transection(self.start_yx, self.end_yx, self.start_lalo, self.end_lalo)
 
-        self.fig.subplots_adjust(left=0.05, wspace=0.25)
+        self.fig.subplots_adjust(left=0.08, wspace=0.25, bottom=0.15)
 
         # save
         if self.save_fig:
@@ -268,69 +276,8 @@ class transectionViewer():
             plt.show()
         return
 
-    def draw_line(self, start_yx, end_yx):
-        """Draw the transect line in the map axes"""
-        # erase existing line
-        if self.line is not None:
-            self.ax_img.lines.remove(self.line[0])
-
-        # convert coordinates accordingly
-        if 'Y_FIRST' in self.atr.keys():
-            ys = self.coord.yx2lalo([self.start_yx[0], self.end_yx[0]], coord_type='y')
-            xs = self.coord.yx2lalo([self.start_yx[1], self.end_yx[1]], coord_type='x')
-        else:
-            ys = [start_yx[0], end_yx[0]]
-            xs = [start_yx[1], end_yx[1]]
-
-        # plot
-        self.line = self.ax_img.plot(xs, ys, 'k--')
-        self.fig.canvas.draw()
-        return
-
-    def draw_transection(self, start_yx, end_yx, start_lalo=None, end_lalo=None):
-        """Plot the transect as dots"""
-        self.ax_txn.cla()
-
-        # loop for all input files
-        for i in range(self.num_file):
-            # get transection data
-            if start_lalo is not None:
-                # use lat/lon whenever it's possible to support files with different resolutions
-                txn = ut.transect_lalo(self.data_list[i],
-                                       self.atr_list[i],
-                                       start_lalo, end_lalo,
-                                       interpolation=self.interpolation)
-            else:
-                txn = ut.transect_yx(self.data_list[i],
-                                     self.atr_list[i],
-                                     start_yx, end_yx,
-                                     interpolation=self.interpolation)
-            # plot
-            self.ax_txn.scatter(txn['distance']/1000.0,
-                                txn['value'] - self.offset[i],
-                                c=pp.mplColors[i],
-                                s=self.marker_size**2)
-
-        self.outfile_base = 'transect_Y{}X{}_Y{}X{}'.format(start_yx[0], start_yx[1], end_yx[0], end_yx[1])
-
-        # title
-        msg = 'y/x: ({}, {}) --> ({}, {})'.format(start_yx[0], start_yx[1], end_yx[0], end_yx[1])
-        if 'Y_FIRST' in self.atr.keys():
-            lat0, lon0 = self.coord.radar2geo(start_yx[0], start_yx[1])[0:2]
-            lat1, lon1 = self.coord.radar2geo(end_yx[0], end_yx[1])[0:2]
-            msg += '\nlat/lon: ({:.4f}, {:.4f}) --> ({:.4f}, {:.4f})'.format(lat0, lon0, lat1, lon1)
-        self.ax_txn.set_title(msg, fontsize=self.font_size)
-
-        # axis format
-        self.ax_txn.yaxis.set_minor_locator(ticker.AutoMinorLocator(10))
-        self.ax_txn.set_xlabel('Distance (km)', fontsize=self.font_size)
-        self.ax_txn.set_ylabel(self.disp_unit, fontsize=self.font_size)
-        self.ax_txn.tick_params(which='both', direction='in', labelsize=self.font_size,
-                                bottom=True, top=True, left=True, right=True)
-        self.ax_txn.set_xlim(0, txn['distance'][-1]/1000.0)
-        self.fig.canvas.draw()
-        return
 
+    ##---------- event function
     def select_point(self, event):
         """Event handling function for points selection"""
         if event.inaxes == self.ax_img:
@@ -354,12 +301,104 @@ class transectionViewer():
             # update transection for every two clicks
             self.pts_idx += 1
             if self.pts_idx >= 2:
+                self.pts_idx = 0
                 self.draw_line(self.start_yx, self.end_yx)
                 self.draw_transection(self.start_yx, self.end_yx, self.start_lalo, self.end_lalo)
-                self.pts_idx = 0
         return
 
 
+    ##---------- plot functions
+    def draw_line(self, start_yx, end_yx):
+        """Draw the transect line in the map axes"""
+        # erase existing line
+        if self.line_ann is not None:
+            self.line_ann.remove()
+
+        # convert coordinates accordingly
+        if 'Y_FIRST' in self.atr.keys():
+            ys = self.coord.yx2lalo([start_yx[0], end_yx[0]], coord_type='y')
+            xs = self.coord.yx2lalo([start_yx[1], end_yx[1]], coord_type='x')
+        else:
+            ys = [start_yx[0], end_yx[0]]
+            xs = [start_yx[1], end_yx[1]]
+
+        # plot
+        line = self.ax_img.plot(xs, ys, 'k--', alpha=0)[0]
+        self.line_ann = pp.add_arrow(line, position=xs[1])
+
+        self.fig.canvas.draw_idle()
+        self.fig.canvas.flush_events()
+        return
+
+
+    def draw_transection(self, start_yx, end_yx, start_lalo=None, end_lalo=None):
+        """Plot the transect as dots"""
+        self.ax_txn.cla()
+
+        # loop - extract transection data
+        txn_list = []
+        min_dist = 0
+        for i in range(self.num_file):
+            # get transection data
+            if start_lalo is not None:
+                # use lat/lon whenever it's possible to support files with different resolutions
+                txn = ut.transect_lalo(self.data_list[i],
+                                       self.atr_list[i],
+                                       start_lalo, end_lalo,
+                                       interpolation=self.interpolation)
+            else:
+                txn = ut.transect_yx(self.data_list[i],
+                                     self.atr_list[i],
+                                     start_yx, end_yx,
+                                     interpolation=self.interpolation)
+
+            # save txn
+            txn_list.append(txn)
+            min_dist = max(min_dist, txn['distance'][0])
+
+        # loop - plot transection
+        for i, txn in enumerate(txn_list):
+            # distance unit and scaling
+            if txn.get('distance_unit', 'm') == 'pixel':
+                dist_scale = 1.0
+                dist_unit = 'pixel'
+            else:
+                dist_scale = 0.001
+                dist_unit = 'km'
+
+            # plot
+            # update distance values by excluding the commonly masked out pixels in the begining
+            self.ax_txn.scatter(x=(txn['distance'] - min_dist) * dist_scale,
+                                y=txn['value'] - self.offset[i],
+                                c=pp.mplColors[i],
+                                s=self.marker_size**2)
+
+        y0, x0, y1, x1 = start_yx + end_yx
+        self.outfile_base = f'transect_Y{y0}X{x0}_Y{y1}X{x1}'
+
+        # title
+        msg = f'y/x: ({y0}, {x0}) --> ({y1}, {x1})'
+        if 'Y_FIRST' in self.atr.keys():
+            lat0, lon0 = self.coord.radar2geo(start_yx[0], start_yx[1])[0:2]
+            lat1, lon1 = self.coord.radar2geo(end_yx[0], end_yx[1])[0:2]
+            msg += f'\nlat/lon: ({lat0:.4f}, {lon0:.4f}) --> ({lat1:.4f}, {lon1:.4f})'
+        self.ax_txn.set_title(msg, fontsize=self.font_size)
+
+        # axis format
+        self.ax_txn.tick_params(which='both', direction='in', labelsize=self.font_size,
+                                bottom=True, top=True, left=True, right=True)
+        self.ax_txn.yaxis.set_minor_locator(ticker.AutoMinorLocator(10))
+        self.ax_txn.set_ylabel(self.disp_unit, fontsize=self.font_size)
+        self.ax_txn.set_xlabel(f'Distance [{dist_unit}]', fontsize=self.font_size)
+        self.ax_txn.set_xlim(0, (txn['distance'][-1] - min_dist) * dist_scale)
+
+        # update figure
+        self.fig.canvas.draw_idle()
+        self.fig.canvas.flush_events()
+        return
+
+
+
 ############################ Main ###################################
 def main(iargs=None):
     obj = transectionViewer(iargs=iargs)
