@@ -8,13 +8,14 @@
 
 import os
 import sys
-import argparse
 import h5py
 import numpy as np
 import random
+
 from mintpy.objects import timeseries
 from mintpy.defaults.template import get_template_content
-from mintpy.utils import readfile, writefile, utils as ut
+from mintpy.utils import ptime, readfile, writefile, utils as ut
+from mintpy.utils.arg_utils import create_argument_parser
 
 
 #########################################  Usage  ##############################################
@@ -23,7 +24,7 @@ TEMPLATE = get_template_content('reference_point')
 NOTE = """note: Reference value cannot be nan, thus, all selected reference point must be:
   a. non zero in mask, if mask is given
   b. non nan  in data (stack)
-  
+
   Priority:
       input reference_lat/lon
       input reference_y/x
@@ -55,10 +56,12 @@ EXAMPLE = """example:
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Reference to the same pixel in space.',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=NOTE+'\n'+TEMPLATE+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Reference to the same pixel in space.'
+    epilog = NOTE + '\n' + TEMPLATE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('file', type=str, help='file to be referenced.')
     parser.add_argument('-t', '--template', dest='template_file',
@@ -66,7 +69,7 @@ def create_parser():
     parser.add_argument('-m', '--mask', dest='maskFile', help='mask file')
 
     parser.add_argument('-o', '--outfile', type=str, default=None,
-                        help='output file name (default: %(default)s). This option is diabled for ifgramStack file.\n'
+                        help='output file name (default: %(default)s). This option is disabled for ifgramStack file.\n'
                              'None (default) for update data value directly without writing to a new file.\n')
 
     parser.add_argument('--write-data', dest='write_data', action='store_true',
@@ -129,7 +132,7 @@ def read_template_file2inps(template_file, inps=None):
     if not inps:
         inps = cmd_line_parse([''])
     inps_dict = vars(inps)
-    template = readfile.read_template(template_file)
+    template = readfile.read_template(template_file, skip_chars=['[', ']'])
     template = ut.check_template_auto_value(template)
 
     prefix = 'mintpy.reference.'
@@ -147,14 +150,12 @@ def read_template_file2inps(template_file, inps=None):
     if key in template.keys():
         value = template[key]
         if value:
-            value = value.replace('[','').replace(']','')
             inps.ref_y, inps.ref_x = [int(i) for i in value.split(',')]
 
     key = prefix+'lalo'
     if key in template.keys():
         value = template[key]
         if value:
-            value = value.replace('[','').replace(']','')
             inps.ref_lat, inps.ref_lon = [float(i) for i in value.split(',')]
 
     return inps
@@ -180,8 +181,8 @@ def reference_file(inps):
     atr = readfile.read_attribute(inps.file)
 
     # update_mode
-    if (not inps.force 
-            and inps.ref_y is not None and inps.ref_y == int(atr.get('REF_Y', -999)) 
+    if (not inps.force
+            and inps.ref_y is not None and inps.ref_y == int(atr.get('REF_Y', -999))
             and inps.ref_x is not None and inps.ref_x == int(atr.get('REF_X', -999))):
         print('SAME reference pixel is already selected/saved in file, skip updating.')
         return inps.file
@@ -206,9 +207,10 @@ def reference_file(inps):
     else:
         # Find reference y/x
         if inps.method == 'maxCoherence':
-            inps.ref_y, inps.ref_x = select_max_coherence_yx(coh_file=inps.coherenceFile,
-                                                             mask=mask,
-                                                             min_coh=inps.minCoherence)
+            inps.ref_y, inps.ref_x = select_max_coherence_yx(
+                coh_file=inps.coherenceFile,
+                mask=mask,
+                min_coh=inps.minCoherence)
         elif inps.method == 'random':
             inps.ref_y, inps.ref_x = random_select_reference_yx(mask)
         elif inps.method == 'manual':
@@ -234,13 +236,22 @@ def reference_file(inps):
 
         if fext == '.h5':
             if inps.outfile == inps.file:
-                print('updating data value without re-writing to a new file')
+                print('updating dataset values without re-writing to a new file')
 
                 if k == 'ifgramStack':
                     with h5py.File(inps.file, 'r+') as f:
                         ds = f['unwrapPhase']
-                        for i in range(ds.shape[0]):
-                            ds[i, :, :] -= ds[i, inps.ref_y, inps.ref_x]
+                        num_date12 = ds.shape[0]
+                        prog_bar = ptime.progressBar(maxValue=num_date12)
+                        for i in range(num_date12):
+                            prog_bar.update(i+1, suffix=f'{i+1} / {num_date12}')
+
+                            # make a copy of ds[i] because h5py allows fancy indexing for 1D arrays only.
+                            data_2d = ds[i, :, :]
+                            # apply spatial referencing (skip pixels with no-data-value)
+                            data_2d[data_2d != 0.] -= data_2d[inps.ref_y, inps.ref_x]
+                            ds[i, :, :] = data_2d
+                        prog_bar.close()
 
                         print('update metadata')
                         f.attrs.update(atrNew)
@@ -265,7 +276,7 @@ def reference_file(inps):
                 print('writing the referenced data into file: {}'.format(inps.outfile))
 
                 # 1. read and update data value
-                data, atr = readfile.read(inps.file)
+                data, atr = readfile.read(inps.file, datasetName=k)
                 if len(data.shape) == 3:
                     # 3D matrix
                     for i in range(data.shape[0]):
@@ -321,7 +332,7 @@ def manual_select_reference_yx(data, inps, mask=None):
     """
     from matplotlib import pyplot as plt
     print('\nManual select reference point ...')
-    print('Click on a pixel that you want to choose as the refernce ')
+    print('Click on a pixel that you want to choose as the reference ')
     print('    pixel in the time-series analysis;')
     print('Then close the displayed window to continue.\n')
     if mask is not None:
@@ -453,7 +464,9 @@ def read_reference_input(inps):
         # Do not use ref_y/x in masked out area
         if inps.maskFile and os.path.isfile(inps.maskFile):
             print('mask: '+inps.maskFile)
-            mask = readfile.read(inps.maskFile, datasetName='mask')[0]
+            ds_names = readfile.get_dataset_list(inps.maskFile)
+            ds_name = [x for x in ds_names if x in ['mask', 'waterMask']][0]
+            mask = readfile.read(inps.maskFile, datasetName=ds_name)[0]
             if mask[inps.ref_y, inps.ref_x] == 0:
                 inps.ref_y, inps.ref_x = None, None
                 msg = 'input reference point is in masked OUT area defined by {}!'.format(inps.maskFile)
@@ -464,9 +477,9 @@ def read_reference_input(inps):
         print('no input reference y/x.')
         if not inps.method:
             # Use existing REF_Y/X if 1) no ref_y/x input and 2) no method input and 3) ref_yx is in coverage
-            if (not inps.force 
+            if (not inps.force
                     and 'REF_X' in atr.keys()
-                    and 0 <= float(atr['REF_Y']) <= length 
+                    and 0 <= float(atr['REF_Y']) <= length
                     and 0 <= float(atr['REF_X']) <= width):
                 print('REF_Y/X exists in input file, skip updating.')
                 print('REF_Y: '+atr['REF_Y'])
@@ -502,6 +515,7 @@ def main(iargs=None):
 
     if inps.go_reference:
         reference_file(inps)
+
     print('Done.')
     return
 
