@@ -6,10 +6,8 @@
 ############################################################
 
 
-
 import os
 import sys
-import argparse
 import numpy as np
 import scipy.io as sio
 import matplotlib.pyplot as plt
@@ -19,14 +17,10 @@ warnings.filterwarnings("ignore", category=UserWarning, module="matplotlib")
 
 from mintpy.objects import sensor
 from mintpy.utils import ptime, readfile, utils as ut
+from mintpy.utils.arg_utils import create_argument_parser
 
 
-EXAMPLE = """example:
-  save_gbis.py velocity.h5 -g inputs/geometryGeo.h5 -o AlosDT73_20081012_20100302.mat
-  save_gbis.py 20150223_20161031_msk.unw -g inputs/geometryGeo.h5 -o Alos2DT23_20150223_20161031.mat
-  save_gbis.py 20150223_20161031.unw -g inputs/geometryGeo.h5 --out-data ../Model/data --ellipsoid2geoid
-"""
-
+##############################################################################
 REFERENCE = """references:
   Bagnardi, M., and A. Hooper (2018), Inversion of Surface Deformation Data for Rapid Estimates of Source 
   Parameters and Uncertainties: A Bayesian Approach, Geochemistry, Geophysics, Geosystems, 19, 
@@ -36,10 +30,18 @@ REFERENCE = """references:
   with L-band InSAR time series, Geophysical Research Letters, 48(11), e2021GL092879. doi:10.1029/2021GL092879
 """
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Convert MintPy product to GBIS .mat format.',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=REFERENCE+'\n'+EXAMPLE)
+EXAMPLE = """example:
+  save_gbis.py velocity.h5 -g inputs/geometryGeo.h5 -o AlosDT73_20081012_20100302.mat
+  save_gbis.py 20150223_20161031_msk.unw -g inputs/geometryGeo.h5 -o Alos2DT23_20150223_20161031.mat
+  save_gbis.py 20150223_20161031.unw -g inputs/geometryGeo.h5 --out-data ../Model/data --ellipsoid2geoid
+"""
+
+def create_parser(subparsers=None):
+    synopsis = 'Convert MintPy product to GBIS .mat format.'
+    epilog = REFERENCE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('file', help='deformation file.')
     parser.add_argument('dset', nargs='?',
@@ -64,7 +66,10 @@ def create_parser():
 def cmd_line_parse(iargs=None):
     parser = create_parser()
     inps = parser.parse_args(args=iargs)
-    print('{} {}'.format(os.path.basename(__file__), ' '.join(iargs)))
+
+    inps.argv = iargs if iargs else sys.argv[1:]
+    print('{} {}'.format(os.path.basename(__file__), ' '.join(inps.argv)))
+
     inps.file = os.path.abspath(inps.file)
 
     # Backend setting
@@ -74,6 +79,7 @@ def cmd_line_parse(iargs=None):
     return inps
 
 
+##############################################################################
 def read_data(inps):
     """
     Returns: defo: 2D np.array with in-valid/masked-out pixel in NaN
@@ -128,7 +134,7 @@ def read_data(inps):
     print('number of pixels after excluding zero phase value: {}'.format(np.sum(inps.mask)))
 
     # read geometry
-    inps.lat, inps.lon = ut.get_lat_lon(inps.metadata)
+    inps.lat, inps.lon = ut.get_lat_lon(inps.metadata, geom_file=inps.geom_file)
     inps.inc_angle = readfile.read(inps.geom_file, datasetName='incidenceAngle')[0]
     inps.head_angle = np.ones(inps.inc_angle.shape, dtype=np.float32) * float(inps.metadata['HEADING'])
     inps.height = readfile.read(inps.geom_file, datasetName='height')[0]
@@ -243,7 +249,8 @@ def main(iargs=None):
     if inps.disp_fig:
         print('showing...')
         plt.show()
-    return inps.outfile
+
+    return
 
 
 ##############################################################################
