@@ -8,7 +8,6 @@
 
 import os
 import sys
-import argparse
 import datetime as dt
 import h5py
 import numpy as np
@@ -16,6 +15,7 @@ import numpy as np
 from mintpy.objects import timeseries, geometry, sensor
 from mintpy.defaults.template import get_template_content
 from mintpy.utils import ptime, readfile
+from mintpy.utils.arg_utils import create_argument_parser
 from mintpy import info
 
 
@@ -34,13 +34,17 @@ EXAMPLE = """example:
   save_hdfeos5.py timeseries_ERA5_ramp_demErr.h5 --tc temporalCoherence.h5 --asc avgSpatialCoh.h5 -m maskTempCoh.h5 -g inputs/geometryGeo.h5
 """
 
+NOTE = """
+  https://earthdata.nasa.gov/esdis/eso/standards-and-references/hdf-eos5
+  https://mintpy.readthedocs.io/en/latest/hdfeos5/
+"""
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Convert MintPy timeseries product into HDF-EOS5 format\n' +
-                                     '  https://earthdata.nasa.gov/esdis/eso/standards-and-references/hdf-eos5\n' +
-                                     '  https://mintpy.readthedocs.io/en/latest/hdfeos5/',
-                                     formatter_class=argparse.RawDescriptionHelpFormatter,
-                                     epilog=TEMPALTE+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Convert MintPy timeseries product into HDF-EOS5 format'
+    epilog = TEMPALTE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis+NOTE, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('ts_file', default='timeseries.h5', help='Timeseries file')
     parser.add_argument('-t', '--template', dest='template_file',
@@ -52,6 +56,7 @@ def create_parser():
                         help='Average spatial coherence file, i.e. avgSpatialCoh.h5')
     parser.add_argument('-m', '--mask', dest='mask_file', help='Mask file')
     parser.add_argument('-g', '--geometry', dest='geom_file', help='geometry file')
+    parser.add_argument('--suffix', dest='suffix', help='suffix to be appended to file name (e.g. PS).')
 
     parser.add_argument('--update', action='store_true',
                         help='Enable update mode, a.k.a. put XXXXXXXX as endDate in filename if endDate < 1 year')
@@ -242,7 +247,7 @@ def metadata_mintpy2unavco(meta_in, dateList):
     return unavco_meta
 
 
-def get_output_filename(metadata, update_mode=False, subset_mode=False):
+def get_output_filename(metadata, suffix=None, update_mode=False, subset_mode=False):
     """Get output file name of HDF-EOS5 time-series file."""
     SAT = metadata['mission']
     SW = metadata['beam_mode']
@@ -263,7 +268,10 @@ def get_output_filename(metadata, update_mode=False, subset_mode=False):
         print('Update mode is ON, put endDate as XXXXXXXX.')
         DATE2 = 'XXXXXXXX'
 
-    outName = SAT+'_'+SW+'_'+RELORB+'_'+FRAME+'_'+DATE1+'_'+DATE2+'.he5'
+    if not suffix:
+       outName = SAT+'_'+SW+'_'+RELORB+'_'+FRAME+'_'+DATE1+'_'+DATE2+'.he5'
+    else:
+       outName = SAT+'_'+SW+'_'+RELORB+'_'+FRAME+'_'+DATE1+'_'+DATE2+'_'+suffix+'.he5'
 
     if subset_mode:
         print('Subset mode is enabled, put subset range info in output filename.')
@@ -457,24 +465,29 @@ def main(iargs=None):
     inps, template = read_template2inps(inps.template_file, inps)
 
     # Prepare Metadata
-    meta = prep_metadata(ts_file=inps.ts_file,
-                         template=template,
-                         print_msg=True)
+    meta = prep_metadata(
+        ts_file=inps.ts_file,
+        template=template,
+        print_msg=True)
 
     # Get output filename
-    out_file = get_output_filename(metadata=meta,
-                                   update_mode=inps.update,
-                                   subset_mode=inps.subset)
+    out_file = get_output_filename(
+        metadata=meta,
+        suffix=inps.suffix,
+        update_mode=inps.update,
+        subset_mode=inps.subset)
 
     # Open HDF5 File
-    write_hdf5_file(metadata=meta,
-                    out_file=out_file,
-                    ts_file=inps.ts_file,
-                    tcoh_file=inps.tcoh_file,
-                    scoh_file=inps.scoh_file,
-                    mask_file=inps.mask_file,
-                    geom_file=inps.geom_file)
-    return out_file
+    write_hdf5_file(
+        metadata=meta,
+        out_file=out_file,
+        ts_file=inps.ts_file,
+        tcoh_file=inps.tcoh_file,
+        scoh_file=inps.scoh_file,
+        mask_file=inps.mask_file,
+        geom_file=inps.geom_file)
+
+    return
 
 
 ################################################################
