@@ -14,12 +14,12 @@ import glob
 import time
 import datetime
 import shutil
-import argparse
 import numpy as np
 
 import mintpy
 from mintpy.objects import sensor, cluster, RAMP_LIST
 from mintpy.utils import readfile, writefile, utils as ut
+from mintpy.utils.arg_utils import create_argument_parser
 from mintpy.defaults.template import STEP_LIST
 import mintpy.workflow   # dynamic import of modules for smallbaselineApp
 
@@ -36,6 +36,12 @@ previous run was done using one of the steps options to process at least
 through the step immediately preceding the starting step of the current run.
 """.format(STEP_LIST[0:5], STEP_LIST[5:11], STEP_LIST[11:])
 
+REFERENCE = """reference:
+  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis:
+  Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
+  doi:10.1016/j.cageo.2019.104331.
+"""
+
 EXAMPLE = """example:
   smallbaselineApp.py                         #run with default template 'smallbaselineApp.cfg'
   smallbaselineApp.py <custom_template>       #run with default and custom templates
@@ -43,24 +49,20 @@ EXAMPLE = """example:
   smallbaselineApp.py -H                      #print    default template options
   smallbaselineApp.py -g                      #generate default template if it does not exist
   smallbaselineApp.py -g <custom_template>    #generate/update default template based on custom template
-  smallbaselineApp.py --plot                  #plot results without run
+  smallbaselineApp.py --plot                  #plot results w/o run [to populate the 'pic' folder after failed runs]
 
   # Run with --start/stop/dostep options
   smallbaselineApp.py GalapagosSenDT128.template --dostep velocity  #run at step 'velocity' only
   smallbaselineApp.py GalapagosSenDT128.template --end load_data    #end after step 'load_data'
 """
 
-REFERENCE = """reference:
-  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis: 
-  Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
-  doi:10.1016/j.cageo.2019.104331.
-"""
 
-
-def create_parser():
-    parser = argparse.ArgumentParser(description='Routine Time Series Analysis for Small Baseline InSAR Stack',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=REFERENCE+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Routine Time Series Analysis for Small Baseline InSAR Stack'
+    epilog = REFERENCE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('customTemplateFile', nargs='?',
                         help='custom template with option settings.\n' +
@@ -93,6 +95,11 @@ def cmd_line_parse(iargs=None):
     parser = create_parser()
     inps = parser.parse_args(args=iargs)
 
+    # save argv (to check the manually specified arguments)
+    # use iargs        for python call
+    # use sys.argv[1:] for command line call
+    inps.argv = iargs if iargs else sys.argv[1:]
+
     template_file = os.path.join(os.path.dirname(mintpy.__file__), 'defaults/smallbaselineApp.cfg')
 
     # -H (print default template)
@@ -136,7 +143,7 @@ def cmd_line_parse(iargs=None):
             inps.customTemplateFile = None
 
     # check --plot
-    if iargs == ['--plot']:
+    if inps.argv == ['--plot']:
         plot_only = True
         print('plot smallbaselineApp results without run.')
     else:
@@ -289,6 +296,10 @@ class TimeSeriesAnalysis:
             self.templateFile = ut.update_template_file(self.templateFile, self.customTemplate)
 
         # 2) backup custome/default template file in inputs/pic folder
+        flen = len(os.path.basename(self.templateFile))
+        if self.customTemplateFile:
+            flen = max(flen, len(os.path.basename(self.customTemplateFile)))
+
         for backup_dirname in ['inputs', 'pic']:
             backup_dir = os.path.join(self.workDir, backup_dirname)
             # create directory
@@ -301,8 +312,9 @@ class TimeSeriesAnalysis:
                                              check_readable=False,
                                              print_msg=False) == 'run':
                     shutil.copy2(tfile, backup_dir)
-                    print('copy {} to {:<8} directory for backup.'.format(os.path.basename(tfile),
-                                                                          os.path.basename(backup_dir)))
+                    print('copy {f:<{l}} to {d:<8} directory for backup.'.format(f=os.path.basename(tfile),
+                                                                                 l=flen,
+                                                                                 d=os.path.basename(backup_dir)))
 
         # 3) read default template file
         print('read default template file:', self.templateFile)
@@ -347,24 +359,18 @@ class TimeSeriesAnalysis:
         os.chdir(self.workDir)
 
         # 3) check loading result
-        load_complete, stack_file, geom_file = ut.check_loaded_dataset(self.workDir, print_msg=True)[0:3]
+        stack_file, geom_file, _, ion_file = ut.check_loaded_dataset(self.workDir, print_msg=True)[:4]
 
         # 4) add custom metadata (optional)
         if self.customTemplateFile:
-            print('updating {}, {} metadata based on custom template file: {}'.format(
-                os.path.basename(stack_file),
-                os.path.basename(geom_file),
-                os.path.basename(self.customTemplateFile)))
             # use ut.add_attribute() instead of add_attribute.py because of
             # better control of special metadata, such as SUBSET_X/YMIN
-            ut.add_attribute(stack_file, self.customTemplate)
-            ut.add_attribute(geom_file, self.customTemplate)
+            msg = f'updating metadata based on custom template file {os.path.basename(self.customTemplateFile)}'
+            for fname in [stack_file, ion_file, geom_file]:
+                if fname:
+                    print(f'{msg} for file: {os.path.basename(fname)}')
+                    ut.add_attribute(fname, self.customTemplate)
 
-        # 5) if not load_complete, plot and raise exception
-        if not load_complete:
-            self.plot_result(print_aux=False)
-            self.close(normal_end=False)
-            raise RuntimeError('step {}: NOT all required dataset found, exit.'.format(step_name))
         return
 
 
@@ -386,7 +392,7 @@ class TimeSeriesAnalysis:
     def run_network_modification(self, step_name):
         """Modify network of interferograms before the network inversion."""
         # check the existence of ifgramStack.h5
-        stack_file, geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1:3]
+        stack_file, geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[:2]
         coh_txt = os.path.join(self.workDir, 'coherenceSpatialAvg.txt')
         net_fig = [os.path.join(self.workDir, i, 'network.pdf') for i in ['', 'pic']]
         try:
@@ -439,11 +445,11 @@ class TimeSeriesAnalysis:
 
     def generate_ifgram_aux_file(self):
         """Generate auxiliary files from ifgramStack file"""
-        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
+        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[0]
         dsNames = readfile.get_dataset_list(stack_file)
         mask_file = os.path.join(self.workDir, 'maskConnComp.h5')
         coh_file = os.path.join(self.workDir, 'avgSpatialCoh.h5')
-        snr_file = os.path.join(self.workDir, 'avgSpatialSnr.h5')
+        snr_file = os.path.join(self.workDir, 'avgSpatialSNR.h5')
 
         # 1) generate mask file from the common connected components
         if any('phase' in i.lower() for i in dsNames):
@@ -471,7 +477,7 @@ class TimeSeriesAnalysis:
         self.generate_ifgram_aux_file()
 
         # 3) add REF_X/Y(/LAT/LON) of the reference point
-        stack_file, _, lookup_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1:4]
+        stack_file, _, lookup_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[:3]
         coh_file = os.path.join(self.workDir, 'avgSpatialCoh.h5')
 
         iargs = [stack_file, '-t', self.templateFile, '-c', coh_file]
@@ -488,7 +494,7 @@ class TimeSeriesAnalysis:
             2) numTriNonzeroIntAmbiguity.h5: phase unwrapping errors through the integer ambiguity of phase closure
         """
         # check the existence of ifgramStack.h5
-        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
+        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[0]
 
         # 1) stack interferograms
         pha_vel_file = os.path.join(self.workDir, 'avgPhaseVelocity.h5')
@@ -512,7 +518,7 @@ class TimeSeriesAnalysis:
             return
 
         # check required input files
-        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
+        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[0]
         mask_file = os.path.join(self.workDir, 'maskConnComp.h5')
 
         iargs_bridge = [stack_file, '--template', self.templateFile, '--update']
@@ -548,7 +554,7 @@ class TimeSeriesAnalysis:
         2) temporalCoherence.h5 --> maskTempCoh.h5
         """
         # check the existence of ifgramStack.h5
-        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
+        stack_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[0]
 
         # 1) invert ifgramStack for time-series
         iargs = [stack_file, '-t', self.templateFile, '--update']
@@ -562,7 +568,7 @@ class TimeSeriesAnalysis:
 
     def generate_temporal_coherence_mask(self):
         """Generate reliable pixel mask from temporal coherence"""
-        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
         tcoh_file = os.path.join(self.workDir, 'temporalCoherence.h5')
         mask_file = os.path.join(self.workDir, 'maskTempCoh.h5')
         tcoh_min = self.template['mintpy.networkInversion.minTempCoh']
@@ -611,6 +617,12 @@ class TimeSeriesAnalysis:
             msg += "Try the following:\n"
             msg += "1) Check the reference pixel and make sure it's not in areas with unwrapping errors\n"
             msg += "2) Check the network and make sure it's fully connected without subsets"
+            print(f'ERROR: {msg}')
+
+            # populate the pic folder to facilate the trouble shooting
+            self.plot_result(print_aux=False)
+
+            # terminate the program
             raise RuntimeError(msg)
         return
 
@@ -712,7 +724,7 @@ class TimeSeriesAnalysis:
         Automatically applied for Envisat data.
         Automatically skipped for all the other data.
         """
-        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
         fnames = self.get_timeseries_filename(self.template, self.workDir)[step_name]
         in_file = fnames['input']
         out_file = fnames['output']
@@ -730,7 +742,7 @@ class TimeSeriesAnalysis:
 
     def run_solid_earth_tides_correction(self, step_name):
         """Correct solid Earth tides (SET)."""
-        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
         fnames = self.get_timeseries_filename(self.template, self.workDir)[step_name]
         in_file = fnames['input']
         out_file = fnames['output']
@@ -748,7 +760,7 @@ class TimeSeriesAnalysis:
 
     def run_tropospheric_delay_correction(self, step_name):
         """Correct tropospheric delays."""
-        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
         mask_file = os.path.join(self.workDir, 'maskTempCoh.h5')
 
         fnames = self.get_timeseries_filename(self.template, self.workDir)[step_name]
@@ -845,7 +857,7 @@ class TimeSeriesAnalysis:
         """step - correct_topography
         Topographic residual (DEM error) correction (optional).
         """
-        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+        geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
         fnames = self.get_timeseries_filename(self.template, self.workDir)[step_name]
         in_file = fnames['input']
         out_file = fnames['output']
@@ -924,7 +936,7 @@ class TimeSeriesAnalysis:
                 out_dir = os.path.join(self.workDir, 'geo')
                 os.makedirs(out_dir, exist_ok=True)
 
-                geom_file, lookup_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2:4]
+                geom_file, lookup_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1:3]
                 in_files = [geom_file, 'temporalCoherence.h5', 'avgSpatialCoh.h5', ts_file, 'velocity.h5']
                 iargs = in_files + ['-l', lookup_file, '-t', self.templateFile, '--outdir', out_dir, '--update']
                 print('\ngeocode.py', ' '.join(iargs))
@@ -938,7 +950,7 @@ class TimeSeriesAnalysis:
 
                 iargs = [tcoh_file, '-m', tcoh_min, '-o', mask_file]
                 # exclude pixels in shadow if shadowMask dataset is available
-                if (self.template['mintpy.networkInversion.shadowMask'] is True 
+                if (self.template['mintpy.networkInversion.shadowMask'] is True
                         and 'shadowMask' in readfile.get_dataset_list(geom_file)):
                     iargs += ['--base', geom_file, '--base-dataset', 'shadowMask', '--base-value', '1']
                 print('\ngenerate_mask.py', ' '.join(iargs))
@@ -995,7 +1007,7 @@ class TimeSeriesAnalysis:
             tcoh_file = os.path.join(self.workDir, 'temporalCoherence.h5')
             scoh_file = os.path.join(self.workDir, 'avgSpatialCoh.h5')
             mask_file = os.path.join(self.workDir, 'maskTempCoh.h5')
-            geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[2]
+            geom_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1]
             if 'geo' in ts_file:
                 tcoh_file = os.path.join(self.workDir, 'geo/geo_temporalCoherence.h5')
                 scoh_file = os.path.join(self.workDir, 'geo/geo_avgSpatialCoh.h5')
@@ -1094,7 +1106,10 @@ class TimeSeriesAnalysis:
         max_memory = abs(float(self.template['mintpy.compute.maxMemory']))
         fig_dpi = int(self.template['mintpy.plot.dpi'])
 
-        stack_file, geom_file, lookup_file = ut.check_loaded_dataset(self.workDir, print_msg=False)[1:]
+        (stack_file,
+         geom_file,
+         lookup_file,
+         ion_file) = ut.check_loaded_dataset(self.workDir, print_msg=False)[:4]
         mask_file = os.path.join(self.workDir, 'maskTempCoh.h5')
         geo_dir = os.path.join(self.workDir, 'geo')
         pic_dir = os.path.join(self.workDir, 'pic')
@@ -1153,6 +1168,12 @@ class TimeSeriesAnalysis:
             ['numInvIfgram.h5',           '--mask', 'no'],
         ]
 
+        if ion_file:
+            iargs_list0 += [
+                [ion_file, 'unwrapPhase-', '--zero-mask'],
+                [ion_file, 'coherence-',   '--mask', 'no', '-v', '0', '1'],
+            ]
+
         # translate element list whose file path has *
         iargs_list = []
         for iargs in iargs_list0:
@@ -1234,13 +1255,13 @@ class TimeSeriesAnalysis:
         msg = """Explore more info & visualization options with the following scripts:
         info.py                    #check HDF5 file structure and metadata
         view.py                    #2D map view
-        tsview.py                  #1D point time-series (interactive)   
+        tsview.py                  #1D point time-series (interactive)
         transect.py                #1D profile (interactive)
         plot_coherence_matrix.py   #plot coherence matrix for one pixel (interactive)
-        plot_network.py            #plot network configuration of the dataset    
+        plot_network.py            #plot network configuration of the dataset
         plot_transection.py        #plot 1D profile along a line of a 2D matrix (interactive)
         save_kmz.py                #generate Google Earth KMZ file in raster image
-        save_kmz_timeseries.py     #generate Goodle Earth KMZ file in points for time-series (interactive)
+        save_kmz_timeseries.py     #generate Google Earth KMZ file in points for time-series (interactive)
         """
         if print_aux:
             print(msg)
