@@ -8,7 +8,6 @@
 
 import os
 import sys
-import argparse
 import h5py
 import numpy as np
 
@@ -19,6 +18,7 @@ from mintpy.utils import (
     utils as ut,
     attribute as attr,
 )
+from mintpy.utils.arg_utils import create_argument_parser
 
 
 ###########################################################################################
@@ -47,10 +47,13 @@ EXAMPLE = """example:
   subset.py geomap_4rlks.trans --tight
 """
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Generate a subset from file/dataset',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=TEMPLATE+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Generate a subset from file/dataset'
+    epilog = TEMPLATE + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
+
     parser.add_argument('file', nargs='+', help='File(s) to subset/crop')
 
     parser.add_argument('-x','--sub-x','--subset-x', dest='subset_x', type=int, nargs=2,
@@ -77,8 +80,6 @@ def create_parser():
                         help="fill subset area out of data coverage with input value. i.e. \n"
                              "np.nan, 0, 1000, ... \n"
                              "By default, it's None for no-outfill.")
-    parser.add_argument('--no-parallel', dest='parallel', action='store_false', default=True,
-                        help='Disable parallel processing. Diabled auto for 1 input file.\n\n')
 
     parser.add_argument('-o', '--output', dest='outfile',
                         help='output file name\n' +
@@ -120,7 +121,7 @@ def get_coverage_box(atr):
     width = int(atr['WIDTH'])
 
     # Get geo box
-    try:
+    if all(x in atr.keys() for x in ['Y_STEP', 'X_STEP', 'Y_FIRST', 'X_FIRST']):
         lat_step = float(atr['Y_STEP'])
         lon_step = float(atr['X_STEP'])
         ul_lat = float(atr['Y_FIRST'])
@@ -128,16 +129,18 @@ def get_coverage_box(atr):
         lr_lat = ul_lat + lat_step*length
         lr_lon = ul_lon + lon_step*width
         geo_box = (ul_lon, ul_lat, lr_lon, lr_lat)
-    except ValueError:
+    else:
         geo_box = None
 
     # Get pixel box
-    try:
-        pix_box = (int(atr['SUBSET_XMIN']),
-                   int(atr['SUBSET_YMIN']),
-                   int(atr['SUBSET_XMAX']),
-                   int(atr['SUBSET_YMAX']))
-    except ValueError:
+    if all(f'SUBSET_{x}' in atr.keys() for x in ['YMIN', 'XMIN', 'YMAX', 'XMAX']):
+        pix_box = (
+            int(atr['SUBSET_XMIN']),
+            int(atr['SUBSET_YMIN']),
+            int(atr['SUBSET_XMAX']),
+            int(atr['SUBSET_YMAX']),
+        )
+    else:
         pix_box = None
 
     return pix_box, geo_box
