@@ -10,11 +10,12 @@ import os
 import sys
 import re
 from configparser import ConfigParser
-import argparse
 import h5py
 import numpy as np
+
 from mintpy.objects import timeseries, geometry
 from mintpy.utils import ptime, readfile, writefile, utils as ut
+from mintpy.utils.arg_utils import create_argument_parser
 
 try:
     import pyaps3 as pa
@@ -30,6 +31,21 @@ WEATHER_MODEL_HOURS = {
 
 
 ###############################################################
+REFERENCE = """reference:
+  Jolivet, R., R. Grandin, C. Lasserre, M.-P. Doin and G. Peltzer (2011), Systematic InSAR tropospheric
+  phase delay corrections from global meteorological reanalysis data, Geophys. Res. Lett., 38, L17311,
+  doi:10.1029/2011GL048757
+
+  Jolivet, R., P. S. Agram, N. Y. Lin, M. Simons, M. P. Doin, G. Peltzer, and Z. Li (2014), Improving
+  InSAR geodesy using global atmospheric models, Journal of Geophysical Research: Solid Earth, 119(3),
+  2324-2341, doi:10.1002/2013JB010588.
+
+  # ERA5
+  Hersbach, H., Bell, B., Berrisford, P., Hirahara, S., Horányi, A., Muñoz-Sabater, J., et al. (2020). 
+  The ERA5 global reanalysis. Quarterly Journal of the Royal Meteorological Society, 146(730), 1999–2049.
+  https://doi.org/10.1002/qj.3803
+"""
+
 EXAMPLE = """example:
   # download datasets, calculate tropospheric delays and correct time-series file.
   tropo_pyaps3.py -f timeseries.h5 -g inputs/geometryRadar.h5
@@ -52,21 +68,6 @@ SAFE_FILE = """SAFE_files.txt:
     ...
 """
 
-REFERENCE = """reference:
-  Jolivet, R., R. Grandin, C. Lasserre, M.-P. Doin and G. Peltzer (2011), Systematic InSAR tropospheric
-  phase delay corrections from global meteorological reanalysis data, Geophys. Res. Lett., 38, L17311,
-  doi:10.1029/2011GL048757
-
-  Jolivet, R., P. S. Agram, N. Y. Lin, M. Simons, M. P. Doin, G. Peltzer, and Z. Li (2014), Improving
-  InSAR geodesy using global atmospheric models, Journal of Geophysical Research: Solid Earth, 119(3),
-  2324-2341, doi:10.1002/2013JB010588.
-
-  # ERA5
-  Hersbach, H., Bell, B., Berrisford, P., Hirahara, S., Horányi, A., Muñoz-Sabater, J., et al. (2020). 
-  The ERA5 global reanalysis. Quarterly Journal of the Royal Meteorological Society, 146(730), 1999–2049.
-  https://doi.org/10.1002/qj.3803
-"""
-
 DATA_INFO = """Global Atmospheric Models:
   re-analysis_dataset      coverage  temp_resolution  spatial_resolution       latency       assimilation
   --------------------------------------------------------------------------------------------------------
@@ -94,11 +95,12 @@ atmosphere/
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Tropospheric correction using weather models\n' +
-                                     '  PyAPS is used to download and calculate the delay for each acquisition.',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=REFERENCE+'\n'+DATA_INFO+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Tropospheric correction using weather models via PyAPS'
+    epilog = REFERENCE + '\n' + DATA_INFO + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('-f', '--file', dest='dis_file',
                         help='timeseries HDF5 file, i.e. timeseries.h5')
@@ -556,7 +558,7 @@ def check_pyaps_account_config(tropo_model):
         for opt in SECTION_OPTS[section]:
             val = cfg.get(section, opt)
             if not val or val in default_values:
-                raise ValueError('PYAPS: No account info found for {} in {} section in file: {}'.format(tropo_model, section, cfg_file))
+                raise ValueError(f'PYAPS: No account info found for {tropo_model} in {section} section in file: {cfg_file}')
 
     return
 
@@ -837,9 +839,10 @@ def main(iargs=None):
     get_grib_info(inps)
 
     # download
-    inps.grib_files = dload_grib_files(inps.grib_files, 
-                                       tropo_model=inps.tropo_model,
-                                       snwe=inps.snwe)
+    inps.grib_files = dload_grib_files(
+        inps.grib_files, 
+        tropo_model=inps.tropo_model,
+        snwe=inps.snwe)
 
     # calculate tropo delay and save to h5 file
     if inps.geom_file:
@@ -852,21 +855,22 @@ def main(iargs=None):
     if inps.dis_file:
         ftype = inps.atr['FILE_TYPE']
         if ftype == 'timeseries':
-            correct_timeseries(dis_file=inps.dis_file,
-                               tropo_file=inps.tropo_file,
-                               cor_dis_file=inps.cor_dis_file)
+            correct_timeseries(
+                dis_file=inps.dis_file,
+                tropo_file=inps.tropo_file,
+                cor_dis_file=inps.cor_dis_file)
 
         elif ftype == '.unw':
-            correct_single_ifgram(dis_file=inps.dis_file,
-                                  tropo_file=inps.tropo_file,
-                                  cor_dis_file=inps.cor_dis_file)
+            correct_single_ifgram(
+                dis_file=inps.dis_file,
+                tropo_file=inps.tropo_file,
+                cor_dis_file=inps.cor_dis_file)
         else:
             print('input file {} is not timeseries nor .unw, correction is not supported yet.'.format(ftype))
 
     else:
         print('No input displacement file, skip correcting tropospheric delays.')
 
-    
     return
 
 ###############################################################
