@@ -12,10 +12,10 @@ import sys
 import argparse
 import numpy as np
 from scipy import linalg, stats
-from matplotlib import pyplot as plt, ticker, widgets, patches
+from matplotlib import pyplot as plt, widgets, patches
 
 from mintpy.objects import timeseries, giantTimeseries, HDFEOS
-from mintpy.utils import arg_group, ptime, time_func, readfile, utils as ut, plot as pp
+from mintpy.utils import arg_utils, ptime, time_func, readfile, utils as ut, plot as pp
 from mintpy.multilook import multilook_data
 from mintpy import subset, view, timeseries2velocity as ts2vel
 
@@ -26,6 +26,7 @@ EXAMPLE = """example:
   tsview.py timeseries.h5  --wrap
   tsview.py timeseries.h5  --yx 300 400 --zero-first  --nodisplay
   tsview.py geo_timeseries.h5  --lalo 33.250 131.665  --nodisplay
+  tsview.py slcStack.h5 -u dB -v 20 60 -c gray
 
   # press left / right key to slide images
 
@@ -35,31 +36,50 @@ EXAMPLE = """example:
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Interactive time-series viewer',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Interactive time-series viewer'
+    epilog = EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = arg_utils.create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
+
     parser.add_argument('file', nargs='+',
                         help='time-series file to display\n'
                              'i.e.: timeseries_ERA5_ramp_demErr.h5 (MintPy)\n'
                              '      LS-PARAMS.h5 (GIAnT)\n'
                              '      S1_IW12_128_0593_0597_20141213_20180619.he5 (HDF-EOS5)')
-    parser.add_argument('--label', dest='file_label', nargs='*', help='labels to display for multiple input files')
-    parser.add_argument('--ylim', dest='ylim', nargs=2, metavar=('YMIN', 'YMAX'), type=float, help='Y limits for point plotting.')
-    parser.add_argument('--tick-right', dest='tick_right', action='store_true', help='set tick and tick label to the right')
-    parser.add_argument('-l','--lookup', dest='lookup_file', type=str, help='lookup table file')
+    parser.add_argument('--label', dest='file_label', nargs='*',
+                        help='labels to display for multiple input files')
+    parser.add_argument('--ylim', dest='ylim', nargs=2, metavar=('YMIN', 'YMAX'), type=float,
+                        help='Y limits for point plotting.')
+    parser.add_argument('--tick-right', dest='tick_right', action='store_true',
+                        help='set tick and tick label to the right')
+    parser.add_argument('-l','--lookup', dest='lookup_file', type=str,
+                        help='lookup table file')
+    parser.add_argument('--no-show-img','--not-show-image', dest='disp_fig_img', action='store_false',
+                        help='do NOT show the map figure.\n'
+                             'Useful for plotting a point time series only.\n'
+                             'This option requires --yx/lalo input.')
 
-    parser.add_argument('-n', dest='idx', metavar='NUM', type=int, help='Epoch/slice number for initial display.')
-    parser.add_argument('--error', dest='error_file', help='txt file with error for each date.')
+    parser.add_argument('-n', dest='idx', metavar='NUM', type=int,
+                        help='Epoch/slice number for initial display.')
+    parser.add_argument('--error', dest='error_file',
+                        help='txt file with error for each date.')
 
     # time info
-    parser.add_argument('--start-date', dest='start_date', type=str, help='start date of displacement to display')
-    parser.add_argument('--end-date', dest='end_date', type=str, help='end date of displacement to display')
-    parser.add_argument('--exclude', '--ex', dest='ex_date_list', nargs='*', default=['exclude_date.txt'], help='Exclude date shown as gray.')
-    parser.add_argument('--zf', '--zero-first', dest='zero_first', action='store_true', help='Set displacement at first acquisition to zero.')
-    parser.add_argument('--off','--offset', dest='offset', type=float, help='Offset for each timeseries file.')
+    parser.add_argument('--start-date', dest='start_date', type=str,
+                        help='start date of displacement to display')
+    parser.add_argument('--end-date', dest='end_date', type=str,
+                        help='end date of displacement to display')
+    parser.add_argument('--exclude', '--ex', dest='ex_date_list', nargs='*', default=['exclude_date.txt'],
+                        help='Exclude date shown as gray.')
+    parser.add_argument('--zf', '--zero-first', dest='zero_first', action='store_true',
+                        help='Set displacement at first acquisition to zero.')
+    parser.add_argument('--off','--offset', dest='offset', type=float,
+                        help='Offset for each timeseries file.')
 
-    parser.add_argument('--noverbose', dest='print_msg', action='store_false', help='Disable the verbose message printing.')
+    parser.add_argument('--noverbose', dest='print_msg', action='store_false',
+                        help='Disable the verbose message printing.')
 
     # temporal model fitting
     parser.add_argument('--nomodel', '--nofit', dest='plot_model', action='store_false',
@@ -67,31 +87,38 @@ def create_parser():
     parser.add_argument('--plot-model-conf-int', '--plot-fit-conf-int', dest='plot_model_conf_int', action='store_true',
                         help='Plot the time function prediction confidence intervals.\n'
                              '[!-- Preliminary feature alert! --!]\n'
-                             '[!-- This feature is NOT throughly checked. Read the code before use. Interpret at your own risk! --!]')
+                             '[!-- This feature is NOT throughly checked. '
+                             'Read the code before use. Interpret at your own risk! --!]')
 
-    parser = arg_group.add_timefunc_argument(parser)
+    parser = arg_utils.add_timefunc_argument(parser)
 
     # pixel of interest
     pixel = parser.add_argument_group('Pixel Input')
-    pixel.add_argument('--yx', type=int, metavar=('Y', 'X'), nargs=2, help='initial pixel to plot in Y/X coord')
-    pixel.add_argument('--lalo', type=float, metavar=('LAT', 'LON'), nargs=2, help='initial pixel to plot in lat/lon coord')
+    pixel.add_argument('--yx', type=int, metavar=('Y', 'X'), nargs=2,
+                       help='initial pixel to plot in Y/X coord')
+    pixel.add_argument('--lalo', type=float, metavar=('LAT', 'LON'), nargs=2,
+                       help='initial pixel to plot in lat/lon coord')
 
-    pixel.add_argument('--marker', type=str, default='o', help='marker style (default: %(default)s).')
-    pixel.add_argument('--ms', '--markersize', dest='marker_size', type=float, default=6.0, help='marker size (default: %(default)s).')
-    pixel.add_argument('--lw', '--linewidth', dest='linewidth', type=float, default=0, help='line width (default: %(default)s).')
-    pixel.add_argument('--ew', '--edgewidth', dest='edge_width', type=float, default=1.0, help='Edge width for the error bar (default: %(default)s)')
+    pixel.add_argument('--marker', type=str, default='o',
+                       help='marker style (default: %(default)s).')
+    pixel.add_argument('--ms', '--markersize', dest='marker_size', type=float, default=6.0,
+                       help='marker size (default: %(default)s).')
+    pixel.add_argument('--lw', '--linewidth', dest='linewidth', type=float, default=0,
+                       help='line width (default: %(default)s).')
+    pixel.add_argument('--ew', '--edgewidth', dest='edge_width', type=float, default=1.0,
+                       help='Edge width for the error bar (default: %(default)s)')
 
     # other groups
-    parser = arg_group.add_data_disp_argument(parser)
-    parser = arg_group.add_dem_argument(parser)
-    parser = arg_group.add_figure_argument(parser)
-    parser = arg_group.add_gps_argument(parser)
-    parser = arg_group.add_mask_argument(parser)
-    parser = arg_group.add_map_argument(parser)
-    parser = arg_group.add_memory_argument(parser)
-    parser = arg_group.add_reference_argument(parser)
-    parser = arg_group.add_save_argument(parser)
-    parser = arg_group.add_subset_argument(parser)
+    parser = arg_utils.add_data_disp_argument(parser)
+    parser = arg_utils.add_dem_argument(parser)
+    parser = arg_utils.add_figure_argument(parser)
+    parser = arg_utils.add_gps_argument(parser)
+    parser = arg_utils.add_mask_argument(parser)
+    parser = arg_utils.add_map_argument(parser)
+    parser = arg_utils.add_memory_argument(parser)
+    parser = arg_utils.add_reference_argument(parser)
+    parser = arg_utils.add_save_argument(parser)
+    parser = arg_utils.add_subset_argument(parser)
 
     return parser
 
@@ -100,7 +127,10 @@ def cmd_line_parse(iargs=None):
     parser = create_parser()
     inps = parser.parse_args(args=iargs)
 
-    if '--gps-comp' in iargs:
+    if inps.flip_lr or inps.flip_ud:
+        inps.auto_flip = False
+
+    if inps.gps_component:
         msg = '--gps-comp is not supported for {}'.format(os.path.basename(__file__))
         raise NotImplementedError(msg)
 
@@ -116,20 +146,19 @@ def cmd_line_parse(iargs=None):
         inps.mask_file = 'no'
 
     # default value
-    if not inps.disp_unit:
-        inps.disp_unit = 'cm'
-    if not inps.colormap:
-        inps.colormap = 'jet'
-    if not inps.fig_size:
-        inps.fig_size = [8.0, 4.5]
-
-    # temporal model fitting, initialize the dicts of exp and log funcs
-    inps = ts2vel.init_exp_log_dicts(inps)
+    inps.disp_unit = inps.disp_unit if inps.disp_unit else 'cm'
+    inps.colormap = inps.colormap if inps.colormap else 'jet'
+    inps.fig_size = inps.fig_size if inps.fig_size else [8.0, 4.5]
 
     # verbose print using --noverbose option
     global vprint
     vprint = print if inps.print_msg else lambda *args, **kwargs: None
 
+    if not inps.disp_fig_img:
+        if not inps.yx and not inps.lalo:
+            inps.disp_fig_img = True
+            print('WARNING: NO --yx/lalo input found for --no-show-img, turn it OFF and continue')
+
     if not inps.disp_fig:
         plt.switch_backend('Agg')
 
@@ -140,6 +169,8 @@ def cmd_line_parse(iargs=None):
 def read_init_info(inps):
     # Time Series Info
     atr = readfile.read_attribute(inps.file[0])
+    atr['DATA_TYPE'] = atr.get('DATA_TYPE', 'float32')
+
     inps.key = atr['FILE_TYPE']
     if inps.key == 'timeseries':
         obj = timeseries(inps.file[0])
@@ -202,9 +233,6 @@ def read_init_info(inps):
     (inps.disp_unit,
      inps.unit_fac) = pp.scale_data2disp_unit(metadata=atr, disp_unit=inps.disp_unit)[1:3]
 
-    # Map info - coordinate unit
-    inps.coord_unit = atr.get('Y_UNIT', 'degrees').lower()
-
     # Read Error List
     inps.ts_plot_func = plot_ts_scatter
     inps.error_ts = None
@@ -243,6 +271,10 @@ def read_init_info(inps):
     vprint('subset coverage in lat/lon: '+str(inps.geo_box))
     vprint('------------------------------------------------------------------------')
 
+    # Map info - coordinate unit
+    inps.coord_unit = atr.get('Y_UNIT', 'degrees').lower()
+    inps = view.check_map_projection(inps, metadata=atr, print_msg=inps.print_msg)
+
     # calculate multilook_num
     # ONLY IF:
     #   inps.multilook is True (no --nomultilook input) AND
@@ -278,7 +310,7 @@ def read_init_info(inps):
 
     # do not plot native reference point if it's out of the coverage due to subset
     if (inps.ref_yx and 'Y_FIRST' in atr.keys()
-        and inps.ref_yx == (int(atr['REF_Y']), int(atr['REF_X']))
+        and inps.ref_yx == (int(atr.get('REF_Y',-999)), int(atr.get('REF_X',-999)))
         and not (    inps.pix_box[0] <= inps.ref_yx[1] < inps.pix_box[2]
                  and inps.pix_box[1] <= inps.ref_yx[0] < inps.pix_box[3])):
         inps.disp_ref_pixel = False
@@ -305,25 +337,29 @@ def read_init_info(inps):
         else:
             inps.transparency = 1.0
 
-    ## display unit ans wrap
+    ## display unit and wrap
     # if wrap_step == 2*np.pi (default value), set disp_unit_img = radian;
     # otherwise set disp_unit_img = disp_unit
     inps.disp_unit_img = inps.disp_unit
     if inps.wrap:
-        inps.range2phase = -4. * np.pi / float(atr['WAVELENGTH'])
-        if   'cm' == inps.disp_unit.split('/')[0]:   inps.range2phase /= 100.
-        elif 'mm' == inps.disp_unit.split('/')[0]:   inps.range2phase /= 1000.
-        elif 'm'  == inps.disp_unit.split('/')[0]:   inps.range2phase /= 1.
-        else:
-            raise ValueError('un-recognized display unit: {}'.format(inps.disp_unit))
+        inps.vlim = inps.wrap_range
 
         if (inps.wrap_range[1] - inps.wrap_range[0]) == 2*np.pi:
             inps.disp_unit_img = 'radian'
-        inps.vlim = inps.wrap_range
-    inps.cbar_label = 'Displacement [{}]'.format(inps.disp_unit_img)
+
+        if inps.disp_unit_img == 'radian':
+            inps.range2phase = -4. * np.pi / float(atr['WAVELENGTH'])
+            if   'cm' == inps.disp_unit.split('/')[0]:   inps.range2phase /= 100.
+            elif 'mm' == inps.disp_unit.split('/')[0]:   inps.range2phase /= 1000.
+            elif 'm'  == inps.disp_unit.split('/')[0]:   inps.range2phase /= 1.
+            else:
+                raise ValueError('un-recognized display unit: {}'.format(inps.disp_unit))
+
+    inps.cbar_label = 'Amplitude' if atr['DATA_TYPE'].startswith('complex') else 'Displacement'
+    inps.cbar_label += '[{}]'.format(inps.disp_unit_img)
 
     ## fit a suite of time func to the time series
-    inps.model, inps.num_param = ts2vel.read_inps2model(inps, date_list=inps.date_list)
+    inps.model = time_func.inps2model(inps, date_list=inps.date_list, print_msg=inps.print_msg)
 
     # dense TS for plotting
     inps.date_list_fit = ptime.get_date_range(inps.date_list[0], inps.date_list[-1])
@@ -389,6 +425,9 @@ def read_timeseries_data(inps):
                                   box=inps.pix_box,
                                   xstep=inps.multilook_num,
                                   ystep=inps.multilook_num)
+        if atr['DATA_TYPE'].startswith('complex'):
+            vprint('input data is complex, calculate its amplitude and continue')
+            data = np.abs(data)
 
         if inps.ref_yx and inps.ref_yx != (int(atr.get('REF_Y', -1)), int(atr.get('REF_X', -1))):
             (ry, rx) = subset_and_multilook_yx(inps.ref_yx, inps.pix_box, inps.multilook_num)
@@ -436,7 +475,7 @@ def read_timeseries_data(inps):
     if not inps.vlim:
         inps.cmap_lut, inps.vlim = pp.auto_adjust_colormap_lut_and_disp_limit(ts_data[0],
                                                                               num_multilook=10,
-                                                                              print_msg=inps.print_msg)
+                                                                              print_msg=inps.print_msg)[:2]
     vprint('data    range: {} {}'.format(inps.dlim, inps.disp_unit))
     vprint('display range: {} {}'.format(inps.vlim, inps.disp_unit))
 
@@ -540,26 +579,23 @@ def plot_ts_fit(ax, ts_fit, inps, ppar, m_strs=None, ts_fit_lim=None):
     return ax
 
 
-def get_model_param_str(model, ds_dict, unit_fac=100):
+def get_model_param_str(model, ds_dict, disp_unit='cm'):
     """Summary model parameters in a str paragraph.
-    Parameters: model    - dict, model dictionary
-                ds_dict  - dict, est. time func. param
-                unit_fac - float, unit scaling factor, to determine the unit
-    Returns:    ds_strs  - list of strings, each to summary the result of one time func
+    Parameters: model     - dict, model dictionary
+                ds_dict   - dict, est. time func. param
+                disp_unit - float, display unit for length, which can be scaled
+    Returns:    ds_strs   - list of strings, each to summary the result of one time func
     """
 
     # dataset unit dict
     ds_unit_dict = ts2vel.model2hdf5_dataset(model)[2]
+    ds_names = list(ds_unit_dict.keys())
 
-    # update unit based on unit_fac
-    for ds_name, ds_unit in ds_unit_dict.items():
-        units = ds_unit.split('/')
-        if units[0] == 'm':
-            if   unit_fac == 1000 : units[0] = 'mm'
-            elif unit_fac == 100  : units[0] = 'cm'
-            elif unit_fac == 10   : units[0] = 'dm'
-            elif unit_fac == 1    : units[0] = 'm'
-            elif unit_fac == 0.001: units[0] = 'km'
+    # update ds_unit_dict based on disp_unit
+    for ds_name in ds_names:
+        units = ds_unit_dict[ds_name].split('/')
+        if units[0] == 'm' and disp_unit != 'm':
+            units[0] = disp_unit
             ds_unit_dict[ds_name] = '/'.join(units)
 
     # list of dataset names
@@ -583,13 +619,13 @@ def get_model_param_str(model, ds_dict, unit_fac=100):
     return ds_strs
 
 
-def fit_time_func(model, date_list, ts_dis, unit_fac=100, G_fit=None, conf_level=0.95, seconds=0):
+def fit_time_func(model, date_list, ts_dis, disp_unit='cm', G_fit=None, conf_level=0.95, seconds=0):
     """Fit a suite of fime functions to the time series.
     Equations:  Gm = d
     Parameters: model      - dict of time functions, check utils.time_func.estimate_time_func() for details.
                 date_list  - list of dates in YYYYMMDD format
                 ts_dis     - 1D np.ndarray, displacement time series
-                unit_fac   - float, scaling factor due to different data and display units
+                disp_unit  - str, display unit for length, which can be scaled
                 G_fit      - 2D np.ndarray, design matrix for the dense time series prediction plot
                 conf_level - float in [0,1], confidence level of the plotted confidence intervals
     Returns:    m_strs     - dict, dictionary in {ds_name: ds_value}
@@ -624,7 +660,7 @@ def fit_time_func(model, date_list, ts_dis, unit_fac=100, G_fit=None, conf_level
     # 1.3 translate estimation result into HDF5 ready datasets
     # AND compose list of strings for printout
     m_dict = ts2vel.model2hdf5_dataset(model, m, m_std)[0]
-    m_strs = get_model_param_str(model, m_dict, unit_fac=unit_fac)
+    m_strs = get_model_param_str(model, m_dict, disp_unit=disp_unit)
 
     # 2. reconstruct the fine resolution function
     if G_fit is not None:
@@ -760,16 +796,15 @@ class timeseriesViewer():
                 disp_cbar=True,
                 disp_slider=True,
                 print_msg=self.print_msg)
-        self.fig_img = plt.figure(self.figname_img, figsize=self.figsize_img)
+        subplot_kw = dict(projection=self.map_proj_obj) if self.map_proj_obj is not None else {}
+        self.fig_img, self.ax_img = plt.subplots(figsize=self.figsize_img, subplot_kw=subplot_kw)
 
         # Figure 1 - Axes 1 - Displacement Map
-        self.ax_img = self.fig_img.add_axes([0.125, 0.25, 0.75, 0.65])
         img_data = np.array(self.ts_data[0][self.idx, :, :])
         img_data[self.mask == 0] = np.nan
         self.plot_init_image(img_data)
 
         # Figure 1 - Axes 2 - Time Slider
-        self.ax_tslider = self.fig_img.add_axes([0.125, 0.1, 0.75, 0.07])
         self.plot_init_time_slider(init_idx=self.idx, ref_idx=self.ref_idx)
         self.tslider.on_changed(self.update_time_slider)
 
@@ -783,8 +818,8 @@ class timeseriesViewer():
                 save_ts_data_and_plot(self.yx, d_ts, m_strs, self)
 
         # Final linking of the canvas to the plots.
-        self.fig_img.canvas.mpl_connect('button_press_event', self.update_plot_timeseries)
-        self.fig_img.canvas.mpl_connect('key_press_event', self.on_key_event)
+        self.fig_img.canvas.mpl_connect('button_press_event', self.update_point_timeseries)
+        self.fig_img.canvas.mpl_connect('key_press_event', self.update_image)
         if self.disp_fig:
             vprint('showing ...')
             msg = '\n------------------------------------------------------------------------'
@@ -793,11 +828,90 @@ class timeseriesViewer():
             msg += '\n2) Press left or right arrow key (if not responding, click the image and try again).'
             msg += '\n------------------------------------------------------------------------'
             vprint(msg)
+
+            # --no-show-map option
+            # requires --yx/lalo input
+            if self.yx and not self.disp_fig_img:
+                plt.close(self.fig_img)
+
             plt.show()
         return
 
 
+    ##---------- event functions
+    def update_point_timeseries(self, event):
+        """Event function to get y/x from button press"""
+        if event.inaxes == self.ax_img:
+            # get row/col number
+            if self.fig_coord == 'geo':
+                y, x = self.coord.geo2radar(event.ydata, event.xdata, print_msg=False)[0:2]
+            else:
+                y, x = int(event.ydata+0.5), int(event.xdata+0.5)
+
+            # plot time-series displacement
+            self.plot_point_timeseries((y, x))
+        return
+
+
+    def update_image(self, event):
+        """Slide images with left/right key on keyboard"""
+        if event.inaxes and event.inaxes.figure == self.fig_img:
+            idx = None
+            if event.key == 'left':
+                idx = max(self.idx - 1, 0)
+            elif event.key == 'right':
+                idx = min(self.idx + 1, self.num_date - 1)
+
+            if idx is not None and idx != self.idx:
+                # update title
+                disp_date = self.dates[idx].strftime(self.disp_date_format)
+                sub_title = 'N = {n}, Time = {t}'.format(n=idx, t=disp_date)
+                self.ax_img.set_title(sub_title, fontsize=self.font_size)
+
+                # read data
+                data_img = np.array(self.ts_data[0][idx, :, :])
+                data_img[self.mask == 0] = np.nan
+                if self.wrap:
+                    if self.disp_unit_img == 'radian':
+                        data_img *= self.range2phase
+                    data_img = ut.wrap(data_img, wrap_range=self.wrap_range)
+
+                # update
+                self.tslider.set_val(idx)         # update slider
+                self.img.set_data(data_img)       # update image
+                self.idx = idx
+                self.fig_img.canvas.draw_idle()
+                self.fig_img.canvas.flush_events()
+        return
+
+
+    def update_time_slider(self, val):
+        """Update Displacement Map using Slider"""
+        self.idx = self.tslider.val
+
+        # update title
+        disp_date = self.dates[self.idx].strftime(self.disp_date_format)
+        sub_title = 'N = {n}, Time = {t}'.format(n=self.idx, t=disp_date)
+        self.ax_img.set_title(sub_title, fontsize=self.font_size)
+
+        # read/update 2D image data
+        data_img = self.ts_data[0][self.idx, :, :]
+        data_img[self.mask == 0] = np.nan
+        if self.wrap:
+            if self.disp_unit_img == 'radian':
+                data_img *= self.range2phase
+            data_img = ut.wrap(data_img, wrap_range=self.wrap_range)
+        self.img.set_data(data_img)
+
+        # update figure
+        self.fig_img.canvas.draw_idle()
+        self.fig_img.canvas.flush_events()
+        return
+
+
+    ##---------- plot functions
     def plot_init_image(self, img_data):
+        """Plot the initial 2D image."""
         # prepare data
         if self.wrap:
             if self.disp_unit_img == 'radian':
@@ -805,7 +919,8 @@ class timeseriesViewer():
             img_data = ut.wrap(img_data, wrap_range=self.wrap_range)
 
         # Title and Axis Label
-        disp_date = self.dates[self.idx].strftime('%Y-%m-%d')
+        self.disp_date_format = ptime.get_compact_isoformat(self.date_list[0])
+        disp_date = self.dates[self.idx].strftime(self.disp_date_format)
         self.fig_title = 'N = {}, Time = {}'.format(self.idx, disp_date)
 
         # Initial Pixel of interest
@@ -818,63 +933,34 @@ class timeseriesViewer():
 
         # call view.py to plot
         self.img, self.cbar_img = view.plot_slice(self.ax_img, img_data, self.atr, self)[2:4]
+        self.fig_img.canvas.set_window_title(self.figname_img)
+        self.fig_img.tight_layout(rect=(0,0,1,0.97))
+
         return self.img, self.cbar_img
 
 
     def plot_init_time_slider(self, init_idx=-1, ref_idx=None):
-        val_step = np.min(np.diff(self.yearList))
-        val_min = self.yearList[0]
-        val_max = self.yearList[-1]
+        """Plot the initial slider."""
+        # initiate axes
+        self.fig_img.subplots_adjust(bottom=0.16)
+        self.ax_tslider = self.fig_img.add_axes([0.125, 0.05, 0.75, 0.03])
 
+        # plot slider
         self.tslider = widgets.Slider(
-            self.ax_tslider,
-            label='Time',
-            valinit=self.yearList[init_idx],
-            valmin=val_min,
-            valmax=val_max,
-            valstep=val_step)
+            ax=self.ax_tslider,
+            label='Image',
+            valinit=init_idx,
+            valmin=0,
+            valmax=self.num_date-1,
+            valstep=1)
 
-        bar_width = val_step / 4.
-        datex = np.array(self.yearList) - bar_width / 2.
-        self.tslider.ax.bar(datex, np.ones(len(datex)), bar_width, facecolor='black', ecolor=None)
+        # plot reference date as a gray dot
         if ref_idx is not None:
-            self.tslider.ax.bar(datex[ref_idx], 1., bar_width*3, facecolor='crimson', ecolor=None)
+            self.tslider.ax.scatter(ref_idx, 0.5, s=8**2, marker='o', color='gray', edgecolors='w')
 
-        # xaxis tick format
-        if np.floor(val_max) == np.floor(val_min):
-            digit = 10.
-        else:
-            digit = 1.
-        self.tslider.ax.set_xticks(np.round(np.linspace(val_min, val_max, num=5) * digit) / digit)
-        self.tslider.ax.xaxis.set_minor_locator(ticker.MultipleLocator(1./12.))
-        self.tslider.ax.set_xlim([val_min, val_max])
-        self.tslider.ax.set_yticks([])
-        self.tslider.valtext.set_visible(False)   #hide slider values
         return self.tslider
 
 
-    def update_time_slider(self, val):
-        """Update Displacement Map using Slider"""
-        idx = np.argmin(np.abs(np.array(self.yearList) - self.tslider.val))
-        # update title
-        disp_date = self.dates[idx].strftime('%Y-%m-%d')
-        sub_title = 'N = {n}, Time = {t}'.format(n=idx, t=disp_date)
-        self.ax_img.set_title(sub_title, fontsize=self.font_size)
-
-        # read data
-        data_img = np.array(self.ts_data[0][idx, :, :])
-        data_img[self.mask == 0] = np.nan
-        if self.wrap:
-            if self.disp_unit_img == 'radian':
-                data_img *= self.range2phase
-            data_img = ut.wrap(data_img, wrap_range=self.wrap_range)
-
-        # update data
-        self.img.set_data(data_img)
-        self.idx = idx
-        self.fig_img.canvas.draw()
-        return
-
     def plot_point_timeseries(self, yx):
         """Plot point displacement time-series at pixel [y, x]
         Parameters: yx     : list of 2 int
@@ -903,7 +989,7 @@ class timeseriesViewer():
                 model=self.model,
                 date_list=self.date_list,
                 ts_dis=ts_dis,
-                unit_fac=self.unit_fac,
+                disp_unit=self.disp_unit,
                 G_fit=self.G_fit,
                 seconds=self.seconds)
 
@@ -920,8 +1006,11 @@ class timeseriesViewer():
             if not np.all(np.isnan(ts_dis)):
                 ppar = argparse.Namespace()
                 ppar.label = self.file_label[i]
-                ppar.ms = self.marker_size - ms_step * (num_file - 1 - i)
                 ppar.mfc = pp.mplColors[num_file - 1 - i] if self.mask[y, x] != 0 else 'gray'
+                ppar.ms = self.marker_size - ms_step * (num_file - 1 - i)
+                # use smaller marker size for very long time series
+                if self.num_date > 1e3:
+                    ppar.ms /= 10
                 self.ts_plot_func(ax, ts_dis, self, ppar)
 
                 # plot model prediction
@@ -941,7 +1030,8 @@ class timeseriesViewer():
         # axis format
         ax.tick_params(which='both', direction='in', labelsize=self.font_size, bottom=True, top=True, left=True, right=True)
         pp.auto_adjust_xaxis_date(ax, self.yearList, fontsize=self.font_size)
-        ax.set_ylabel('Displacement [{}]'.format(self.disp_unit), fontsize=self.font_size)
+        cbar_label = 'Amplitude' if self.atr['DATA_TYPE'].startswith('complex') else 'Displacement'
+        ax.set_ylabel(f'{cbar_label} [{self.disp_unit}]', fontsize=self.font_size)
         ax.set_ylim(self.ylim)
         if self.tick_right:
             ax.yaxis.tick_right()
@@ -961,7 +1051,8 @@ class timeseriesViewer():
         vprint('\n---------------------------------------')
         vprint(title)
         float_formatter = lambda x: [float('{:.2f}'.format(i)) for i in x]
-        vprint(float_formatter(ts_dis))
+        if self.num_date <= 1e3:
+            vprint(float_formatter(ts_dis))
 
         if not np.all(np.isnan(ts_dis)):
             # min/max displacement
@@ -973,56 +1064,14 @@ class timeseriesViewer():
                 vprint(f'    {m_str}')
 
             # update figure
-            self.fig_pts.canvas.draw()
+            # use fig.canvas.draw_idel() instead of fig.canvas.draw()
+            # reference: https://stackoverflow.com/questions/64789437
+            self.fig_pts.canvas.draw_idle()
+            self.fig_pts.canvas.flush_events()
 
         return ts_dis, m_strs
 
 
-    def update_plot_timeseries(self, event):
-        """Event function to get y/x from button press"""
-        if event.inaxes == self.ax_img:
-            # get row/col number
-            if self.fig_coord == 'geo':
-                y, x = self.coord.geo2radar(event.ydata, event.xdata, print_msg=False)[0:2]
-            else:
-                y, x = int(event.ydata+0.5), int(event.xdata+0.5)
-
-            # plot time-series displacement
-            self.plot_point_timeseries((y, x))
-        return
-
-
-    def on_key_event(self, event):
-        """Slide images with left/right key on keyboard"""
-        if event.inaxes and event.inaxes.figure == self.fig_img:
-            idx = None
-            if event.key == 'left':
-                idx = max(self.idx - 1, 0)
-            elif event.key == 'right':
-                idx = min(self.idx + 1, self.num_date - 1)
-
-            if idx is not None and idx != self.idx:
-                # update title
-                disp_date = self.dates[idx].strftime('%Y-%m-%d')
-                sub_title = 'N = {n}, Time = {t}'.format(n=idx, t=disp_date)
-                self.ax_img.set_title(sub_title, fontsize=self.font_size)
-
-                # read data
-                data_img = np.array(self.ts_data[0][idx, :, :])
-                data_img[self.mask == 0] = np.nan
-                if self.wrap:
-                    if self.disp_unit_img == 'radian':
-                        data_img *= self.range2phase
-                    data_img = ut.wrap(data_img, wrap_range=self.wrap_range)
-
-                # update
-                self.tslider.set_val(self.yearList[idx]) # update slider
-                self.img.set_data(data_img)              # update image
-                self.idx = idx
-                self.fig_img.canvas.draw()
-        return
-
-
 ###########################################################################################
 def main(iargs=None):
     obj = timeseriesViewer(iargs=iargs)
