@@ -8,11 +8,10 @@
 
 import os
 import sys
-import argparse
 import time
 import h5py
 import numpy as np
-from matplotlib import pyplot as plt, ticker
+from matplotlib import pyplot as plt
 
 try:
     from cvxopt import matrix
@@ -26,6 +25,7 @@ except ImportError:
 from mintpy.objects import ifgramStack, conncomp
 from mintpy.defaults.template import get_template_content
 from mintpy.utils import ptime, readfile, writefile, utils as ut, plot as pp
+from mintpy.utils.arg_utils import create_argument_parser
 from mintpy.utils.solvers import l1regls
 from mintpy import ifgram_inversion as ifginv
 
@@ -33,6 +33,15 @@ from mintpy import ifgram_inversion as ifginv
 key_prefix = 'mintpy.unwrapError.'
 
 ##########################################################################################
+TEMPLATE1 = get_template_content('quick_overview')
+TEMPLATE2 = get_template_content('correct_unwrap_error')
+
+REFERENCE = """reference:
+  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis:
+  Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
+  doi:10.1016/j.cageo.2019.104331.
+"""
+
 EXAMPLE = """example:
   # correct phase unwrapping error with phase closure
   unwrap_error_phase_closure.py  ./inputs/ifgramStack.h5  --cc-mask maskConnComp.h5  -t smallbaselineApp.cfg   --update
@@ -54,27 +63,20 @@ NOTE = """
      unwrapping errors, then the minor right interferograms will turn into wrong.
 """
 
-REFERENCE = """reference:
-  Yunjun, Z., H. Fattahi, and F. Amelung (2019), Small baseline InSAR time series analysis:
-  Unwrapping error correction and noise reduction, Computers & Geosciences, 133, 104331,
-  doi:10.1016/j.cageo.2019.104331.
-"""
-
-TEMPLATE1 = get_template_content('quick_overview')
-TEMPLATE2 = get_template_content('correct_unwrap_error')
-
-
-
-def create_parser():
-    parser = argparse.ArgumentParser(description='Unwrapping Error Correction based on Phase Closure'+NOTE,
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=REFERENCE+'\n'+TEMPLATE1+'\n'+TEMPLATE2+'\n'+EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Unwrapping Error Correction based on Phase Closure'
+    epilog = REFERENCE + '\n' + TEMPLATE1 + '\n' + TEMPLATE2 + '\n' + EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = create_argument_parser(
+        name, synopsis=synopsis, description=synopsis+NOTE, epilog=epilog, subparsers=subparsers)
 
     parser.add_argument('ifgram_file', help='interferograms file to be corrected')
     parser.add_argument('-c','--cc-mask', dest='cc_mask_file', default='maskConnComp.h5',
                         help='common connected components file, required for --action correct')
     parser.add_argument('-n','--num-sample', dest='numSample', type=int, default=100,
                         help='Number of randomly samples/pixels for each common connected component.')
+    parser.add_argument('-m', '--min-area', dest='connCompMinArea', type=float, default=2.5e3,
+                        help='minimum region/area size of a single connComponent.')
 
     parser.add_argument('-a','--action', dest='action', type=str, default='correct',
                         choices={'correct', 'calculate'},
@@ -146,6 +148,8 @@ def read_template2inps(template_file, inps=None):
                 inpsDict[key] = value
             elif key in ['numSample']:
                 inpsDict[key] = int(value)
+            elif key in ['connCompMinArea']:
+                inpsDict[key] = float(value)
 
     return inps
 
@@ -302,78 +306,21 @@ def calc_num_triplet_with_nonzero_integer_ambiguity(ifgram_file, mask_file=None,
     writefile.write(num_nonzero_closure, out_file, meta)
 
     # plot
-    plot_num_triplet_with_nonzero_integer_ambiguity(out_file)
+    pp.plot_num_triplet_with_nonzero_integer_ambiguity(out_file)
 
     return out_file
 
 
-def plot_num_triplet_with_nonzero_integer_ambiguity(fname, display=False, font_size=12, fig_size=[9,3]):
-    """Plot the histogram for the number of triplets with non-zero integer ambiguity
-
-    Fig. 3d-e in Yunjun et al. (2019, CAGEO).
-    """
-
-    # read data
-    data, atr = readfile.read(fname)
-    vmax = int(np.nanmax(data))
-
-    # plot
-    fig, axs = plt.subplots(nrows=1, ncols=2, figsize=fig_size)
-
-    # subplot 1 - map
-    ax = axs[0]
-    im = ax.imshow(data, cmap='RdBu_r', interpolation='nearest')
-
-    # reference point
-    if all(key in atr.keys() for key in ['REF_Y','REF_X']):
-        ax.plot(int(atr['REF_X']), int(atr['REF_Y']), 's', color='white', ms=3)
-
-    # format
-    pp.auto_flip_direction(atr, ax=ax, print_msg=False)
-    fig.colorbar(im, ax=ax)
-    ax.set_title(r'$T_{int}$', fontsize=font_size)
-
-    # subplot 2 - histogram
-    ax = axs[1]
-    ax.hist(data[~np.isnan(data)].flatten(), range=(0, vmax), log=True, bins=vmax)
-
-    # axis format
-    ax.set_xlabel(r'# of triplets w non-zero int ambiguity $T_{int}$', fontsize=font_size)
-    ax.set_ylabel('# of pixels', fontsize=font_size)
-    ax.xaxis.set_minor_locator(ticker.AutoMinorLocator())
-    ax.yaxis.set_major_locator(ticker.LogLocator(base=10.0, numticks=15))
-    ax.yaxis.set_minor_locator(ticker.LogLocator(base=10.0, numticks=15,
-                                                 subs=(0.1,0.2,0.3,0.4,0.5,0.6,0.7,0.8,0.9)))
-    ax.yaxis.set_minor_formatter(ticker.NullFormatter())
-
-    for ax in axs:
-        ax.tick_params(which='both', direction='in', labelsize=font_size,
-                       bottom=True, top=True, left=True, right=True)
-
-    fig.tight_layout()
-
-    # output
-    out_fig = '{}.png'.format(os.path.splitext(fname)[0])
-    print('plot and save figure to file', out_fig)
-    fig.savefig(out_fig, bbox_inches='tight', transparent=True, dpi=300)
-
-    if display:
-        plt.show()
-    else:
-        plt.close(fig)
-
-    return
-
-
 ##########################################################################################
 def get_common_region_int_ambiguity(ifgram_file, cc_mask_file, water_mask_file=None, num_sample=100,
-                                    dsNameIn='unwrapPhase'):
+                                    dsNameIn='unwrapPhase', cc_min_area=2.5e3):
     """Solve the phase unwrapping integer ambiguity for the common regions among all interferograms
     Parameters: ifgram_file     : str, path of interferogram stack file
                 cc_mask_file    : str, path of common connected components file
                 water_mask_file : str, path of water mask file
                 num_sample      : int, number of pixel sampled for each region
                 dsNameIn        : str, dataset name of the unwrap phase to be corrected
+                cc_min_area     : float, minimum region/area size
     Returns:    common_regions  : list of skimage.measure._regionprops._RegionProperties object
                     modified by adding two more variables:
                     sample_coords : 2D np.ndarray in size of (num_sample, 2) in int64 format
@@ -399,64 +346,69 @@ def get_common_region_int_ambiguity(ifgram_file, cc_mask_file, water_mask_file=N
         print('refine common mask based on water mask file', water_mask_file)
         cc_mask[water_mask == 0] = 0
 
-    label_img, num_label = conncomp.label_conn_comp(cc_mask, min_area=2.5e3, print_msg=True)
+    label_img, num_label = conncomp.label_conn_comp(cc_mask, min_area=cc_min_area, print_msg=True)
     common_regions = measure.regionprops(label_img)
     print('number of common regions:', num_label)
+    if num_label == 0:
+        msg = 'WARNING: NO common region found! '
+        msg += 'Try a smaller value for the mintpy.unwrapError.connCompMinArea ({}) option.'.format(cc_min_area)
+        print(msg)
 
-    # add sample_coords / int_ambiguity
-    print('number of samples per region:', num_sample)
-    print('solving the phase-unwrapping integer ambiguity for {}'.format(dsNameIn))
-    print('\tbased on the closure phase of interferograms triplets (Yunjun et al., 2019)')
-    print('\tusing the L1-norm regularzed least squares approximation (LASSO) ...')
-    for i in range(num_label):
-        common_reg = common_regions[i]
-        # sample_coords
-        idx = sorted(np.random.choice(common_reg.area, num_sample, replace=False))
-        common_reg.sample_coords = common_reg.coords[idx, :].astype(int)
+    else:
+        # add sample_coords / int_ambiguity
+        print('number of samples per region:', num_sample)
+        print('solving the phase-unwrapping integer ambiguity for {}'.format(dsNameIn))
+        print('\tbased on the closure phase of interferograms triplets (Yunjun et al., 2019)')
+        print('\tusing the L1-norm regularzed least squares approximation (LASSO) ...')
+        for i in range(num_label):
+            common_reg = common_regions[i]
+            # sample_coords
+            idx = sorted(np.random.choice(common_reg.area, num_sample, replace=False))
+            common_reg.sample_coords = common_reg.coords[idx, :].astype(int)
 
-        # solve for int_ambiguity
-        U = np.zeros((num_ifgram, num_sample))
-        if common_reg.label == label_img[stack_obj.refY, stack_obj.refX]:
-            print('{}/{} skip calculation for the reference region'.format(i+1, num_label))
-        else:
-            prog_bar = ptime.progressBar(maxValue=num_sample, prefix='{}/{}'.format(i+1, num_label))
-            for j in range(num_sample):
-                # read unwrap phase
-                y, x = common_reg.sample_coords[j, :]
-                unw = ifginv.read_stack_obs(stack_obj,
-                                            box=(x, y, x+1, y+1),
-                                            ref_phase=ref_phase,
-                                            obs_ds_name=dsNameIn,
-                                            dropIfgram=True,
-                                            print_msg=False).reshape(num_ifgram, -1)
+            # solve for int_ambiguity
+            U = np.zeros((num_ifgram, num_sample))
+            if common_reg.label == label_img[stack_obj.refY, stack_obj.refX]:
+                print('{}/{} skip calculation for the reference region'.format(i+1, num_label))
+            else:
+                prog_bar = ptime.progressBar(maxValue=num_sample, prefix='{}/{}'.format(i+1, num_label))
+                for j in range(num_sample):
+                    # read unwrap phase
+                    y, x = common_reg.sample_coords[j, :]
+                    unw = ifginv.read_stack_obs(stack_obj,
+                                                box=(x, y, x+1, y+1),
+                                                ref_phase=ref_phase,
+                                                obs_ds_name=dsNameIn,
+                                                dropIfgram=True,
+                                                print_msg=False).reshape(num_ifgram, -1)
 
-                # calculate closure_int
-                closure_pha = np.dot(C, unw)
-                closure_int = matrix(np.round((closure_pha - ut.wrap(closure_pha)) / (2.*np.pi)))
+                    # calculate closure_int
+                    closure_pha = np.dot(C, unw)
+                    closure_int = matrix(np.round((closure_pha - ut.wrap(closure_pha)) / (2.*np.pi)))
 
-                # solve for U
-                U[:,j] = np.round(l1regls(-C, closure_int, alpha=1e-2, show_progress=0)).flatten()
-                prog_bar.update(j+1, every=5)
-            prog_bar.close()
-        # add int_ambiguity
-        common_reg.int_ambiguity = np.median(U, axis=1)
-        common_reg.date12_list = date12_list
+                    # solve for U
+                    U[:,j] = np.round(l1regls(-C, closure_int, alpha=1e-2, show_progress=0)).flatten()
+                    prog_bar.update(j+1, every=5)
+                prog_bar.close()
+            # add int_ambiguity
+            common_reg.int_ambiguity = np.median(U, axis=1)
+            common_reg.date12_list = date12_list
 
-    #sort regions by size to facilitate the region matching later
-    common_regions.sort(key=lambda x: x.area, reverse=True)
+        #sort regions by size to facilitate the region matching later
+        common_regions.sort(key=lambda x: x.area, reverse=True)
 
-    # plot sample result
-    fig_size = pp.auto_figure_size(label_img.shape, disp_cbar=False)
-    fig, ax = plt.subplots(figsize=fig_size)
-    ax.imshow(label_img, cmap='jet')
-    for common_reg in common_regions:
-        ax.plot(common_reg.sample_coords[:,1],
-                common_reg.sample_coords[:,0], 'k.', ms=2)
-    pp.auto_flip_direction(stack_obj.metadata, ax, print_msg=False)
-    out_img = 'common_region_sample.png'
-    fig.savefig(out_img, bbox_inches='tight', transparent=True, dpi=300)
-    print('saved common regions and sample pixels to file', out_img)
-    plt.close(fig)
+        # plot sample result
+        fig_size = pp.auto_figure_size(label_img.shape, disp_cbar=False)
+        fig, ax = plt.subplots(figsize=fig_size)
+        ax.imshow(label_img, cmap='jet')
+        for common_reg in common_regions:
+            ax.plot(common_reg.sample_coords[:,1],
+                    common_reg.sample_coords[:,0], 'k.', ms=2)
+        pp.auto_flip_direction(stack_obj.metadata, ax, print_msg=False)
+        out_img = 'common_region_sample.png'
+        fig.savefig(out_img, bbox_inches='tight', transparent=True, dpi=300)
+        print('saved common regions and sample pixels to file', out_img)
+        plt.close(fig)
 
     return common_regions
 
@@ -553,13 +505,17 @@ def main(iargs=None):
                                                          cc_mask_file=inps.cc_mask_file,
                                                          water_mask_file=inps.waterMaskFile,
                                                          num_sample=inps.numSample,
-                                                         dsNameIn=inps.datasetNameIn)
+                                                         dsNameIn=inps.datasetNameIn,
+                                                         cc_min_area=inps.connCompMinArea)
 
         # apply the integer ambiguity from common conn comp to the whole ifgram
-        run_unwrap_error_phase_closure(inps.ifgram_file, common_regions,
-                                       water_mask_file=inps.waterMaskFile,
-                                       dsNameIn=inps.datasetNameIn,
-                                       dsNameOut=inps.datasetNameOut)
+        if len(common_regions) == 0:
+            print('skip phase closure correction ...')
+        else:
+            run_unwrap_error_phase_closure(inps.ifgram_file, common_regions,
+                                           water_mask_file=inps.waterMaskFile,
+                                           dsNameIn=inps.datasetNameIn,
+                                           dsNameOut=inps.datasetNameOut)
 
     else:
         # calculate the number of triplets with non-zero integer ambiguity
@@ -570,7 +526,7 @@ def main(iargs=None):
         # for debug
         debug_mode = False
         if debug_mode:
-            plot_num_triplet_with_nonzero_integer_ambiguity(out_file)
+            pp.plot_num_triplet_with_nonzero_integer_ambiguity(out_file)
 
     m, s = divmod(time.time()-start_time, 60)
     print('time used: {:02.0f} mins {:02.1f} secs\nDone.'.format(m, s))
