@@ -5,6 +5,7 @@
 ############################################################
 # 2020-07: Talib Oliver-Cabrera, add UAVSAR support w/in stripmapStack
 # 2020-10: Cunren Liang, add alosStack support
+# 2022-06: Yujie Zheng, add standard processing from isce2
 # Group contents:
 #     metadata
 #     geometry
@@ -16,13 +17,24 @@
 
 
 import os
-import re
+import datetime
 import glob
 import shelve
-import datetime
+import re
+import time
+
 import numpy as np
+from scipy import ndimage
+
+from mintpy.objects.constants import SPEED_OF_LIGHT, EARTH_RADIUS
 from mintpy.objects import sensor
-from mintpy.utils import ptime, readfile, writefile, utils1 as ut
+from mintpy.utils import (
+    ptime,
+    readfile,
+    writefile,
+    attribute as attr,
+    utils1 as ut,
+)
 
 # suppress matplotlib DEBUG message
 import logging
@@ -30,10 +42,6 @@ mpl_logger = logging.getLogger('matplotlib')
 mpl_logger.setLevel(logging.WARNING)
 
 
-SPEED_OF_LIGHT = 299792458  # m/s
-EARTH_RADIUS = 6378122.65   # m
-
-
 
 def get_processor(meta_file):
     """
@@ -360,16 +368,16 @@ def extract_alosStack_metadata(meta_file, geom_dir):
     width = img.width
     length = img.length
     data = np.memmap(lat_file, dtype='float64', mode='r', shape=(length, width))
-    meta['LAT_REF1'] = str(data[0+edge, 0+edge])
-    meta['LAT_REF2'] = str(data[0+edge, -1-edge])
-    meta['LAT_REF3'] = str(data[-1-edge, 0+edge])
+    meta['LAT_REF1'] = str(data[ 0+edge,  0+edge])
+    meta['LAT_REF2'] = str(data[ 0+edge, -1-edge])
+    meta['LAT_REF3'] = str(data[-1-edge,  0+edge])
     meta['LAT_REF4'] = str(data[-1-edge, -1-edge])
 
     lon_file = glob.glob(os.path.join(geom_dir, '*_{}rlks_{}alks.lon'.format(rlooks, alooks)))[0]
     data = np.memmap(lon_file, dtype='float64', mode='r', shape=(length, width))
-    meta['LON_REF1'] = str(data[0+edge, 0+edge])
-    meta['LON_REF2'] = str(data[0+edge, -1-edge])
-    meta['LON_REF3'] = str(data[-1-edge, 0+edge])
+    meta['LON_REF1'] = str(data[ 0+edge,  0+edge])
+    meta['LON_REF2'] = str(data[ 0+edge, -1-edge])
+    meta['LON_REF3'] = str(data[-1-edge,  0+edge])
     meta['LON_REF4'] = str(data[-1-edge, -1-edge])
 
     los_file = glob.glob(os.path.join(geom_dir, '*_{}rlks_{}alks.los'.format(rlooks, alooks)))[0]
@@ -477,6 +485,9 @@ def extract_geometry_metadata(geom_dir, meta=dict(), box=None, fbase_list=['hgt'
 
     extract A/RLOOKS by comparing hgt.xml and hgt.full.xml file
     update azimuthPixelSize / rangePixelSize based on A/RLOOKS
+
+    extract LENGTH/WIDTH from the first geom file
+    update corresponding metadata if box is not None
     """
 
     def get_nonzero_row_number(data, buffer=2):
@@ -516,6 +527,15 @@ def extract_geometry_metadata(geom_dir, meta=dict(), box=None, fbase_list=['hgt'
     meta['rangePixelSize'] *= meta['RLOOKS']
     meta['azimuthPixelSize'] *= meta['ALOOKS']
 
+    # get LENGTH/WIDTH
+    atr = readfile.read_attribute(geom_files[0])
+    meta['LENGTH'] = atr['LENGTH']
+    meta['WIDTH'] = atr['WIDTH']
+
+    # update due to subset
+    if box:
+        meta = attr.update_attribute4subset(meta, box)
+
     # get LAT/LON_REF1/2/3/4 into metadata
     for geom_file in geom_files:
         if 'lat' in os.path.basename(geom_file):
@@ -570,7 +590,7 @@ def read_tops_baseline(baseline_file):
 def read_stripmap_baseline(baseline_file):
     """Read baseline file generated by ISCE/stripmapStack processor.
 
-    Example: 
+    Example:
     baselines/20200111_20200125.txt
         PERP_BASELINE_BOTTOM 173.97914535263297
         PERP_BASELINE_TOP 174.05612879066618
@@ -801,56 +821,6 @@ def get_IPF(proj_dir, ts_file):
     return date_list, IPF_IW1, IPF_IW2, IPF_IW3
 
 
-def safe_list_file2sensor_list(safe_list_file, date_list=None, print_msg=True):
-    """Get list of Sentinel-1 sensor names from txt file with SAFE file names.
-
-    Parameters: safe_list_file - str, path of the text file with Sentinel-1 SAFE file path
-                                 E.g. SAFE_files.txt
-                date_list      - list of str in YYYYMMDD format, reference list of dates
-    Returns:    sensor_list    - list of str in S1A or S1B
-                date_list      - list of str in YYYYMMDD format
-    Example:
-        date_list = timeseries('timeseries.h5').get_date_list()
-        sensor_list = safe_list_file2sensor_list('../SAFE_files.txt',
-                                                 date_list=date_list,
-                                                 print_msg=False)[0]
-        s1b_dates = [i for i, j in zip(date_list, sensor_list) if j == 'S1B']
-        np.savetxt('S1B_date.txt', np.array(s1b_dates).reshape(-1,1), fmt='%s')
-    """
-    # read txt file
-    fc = np.loadtxt(safe_list_file, dtype=str).astype(str).tolist()
-    safe_fnames = [os.path.basename(i) for i in fc]
-
-    # get date_list
-    date_list_out = [re.findall('_\d{8}T', i)[0][1:-1] for i in safe_fnames]
-    date_list_out = sorted(list(set(date_list_out)))
-
-    # get sensor_list
-    sensor_list = []
-    for d in date_list_out:
-        safe_fname = [i for i in safe_fnames if d in i][0]
-        sensor = safe_fname.split('_')[0]
-        sensor_list.append(sensor)
-
-    # update against date_list_ref
-    if date_list is not None:
-        # check possible missing dates
-        dates_missing = [i for i in date_list if i not in date_list_out]
-        if dates_missing:
-            raise ValueError('The following dates are missing:\n{}'.format(dates_missing))
-
-        # prune dates not-needed
-        flag = np.array([i in date_list for i in date_list_out], dtype=np.bool_)
-        if np.sum(flag) > 0:
-            sensor_list = np.array(sensor_list)[flag].tolist()
-            dates_removed = np.array(date_list_out)[~flag].tolist()
-            date_list_out = np.array(date_list_out)[flag].tolist()
-            if print_msg:
-                print('The following dates are not needed and removed:\n{}'.format(dates_removed))
-
-    return sensor_list, date_list
-
-
 def get_sensing_datetime_list(proj_dir, date_list=None):
     """Get the sensing datetime objects from ISCE stack results.
     It assumes the default directory structure from topsStack, as below:
@@ -917,3 +887,212 @@ def get_sensing_datetime_list(proj_dir, date_list=None):
 
     return sensingMid, sensingStart, sensingStop
 
+
+
+############################## Standard Processing ###########################################
+
+def gaussian_kernel(sx, sy, sig_x, sig_y):
+    '''Generate a guassian kernal (with all elements sum to 1).
+
+    Parameters: sx/y    - int, dimensions of kernal
+                sig_x/y - float, standard deviation of the guassian distribution
+    '''
+    # ensure sx/y are odd number
+    sx += 1 if np.mod(sx, 2) == 0 else 0
+    sy += 1 if np.mod(sy, 2) == 0 else 0
+
+    x, y = np.meshgrid(np.arange(sx), np.arange(sy))
+    x += 1
+    y += 1
+
+    xc = (sx + 1) / 2
+    yc = (sy + 1) / 2
+    fx = ((x-xc)**2.) / (2.*sig_x**2.)
+    fy = ((y-yc)**2.) / (2.*sig_y**2.)
+
+    k = np.exp(-1.0 * (fx+fy))
+    a = 1./np.sum(k)
+    k = a * k
+
+    return k
+
+
+def convolve(data, kernel):
+    '''Convolve / filter the complex data based on the given kernel.
+
+    Parameters: data   - 2D np.ndarray in complex
+                kernel - 2D np.ndarray in float, convolution kernel
+    '''
+    R = ndimage.convolve(data.real, kernel, mode='constant', cval=0.0)
+    I = ndimage.convolve(data.imag, kernel, mode='constant', cval=0.0)
+    return R + 1J*I
+
+
+def estimate_coherence(intfile, corfile):
+    '''Estimate the spatial coherence (phase sigma) of the wrapped interferogram.
+
+    Parameters: intfile - str, path to the *.int file
+                corfile - str, path to the output correlation file
+    '''
+    import isce
+    import isceobj
+    from mroipac.icu.Icu import Icu
+
+    # create filt interferogram file object
+    filtImage = isceobj.createIntImage()
+    filtImage.load(intfile + '.xml')
+    filtImage.setAccessMode('read')
+    filtImage.createImage()
+
+    # create phase sigma correlation file object
+    phsigImage = isceobj.createImage()
+    phsigImage.dataType='FLOAT'
+    phsigImage.bands = 1
+    phsigImage.setWidth(filtImage.getWidth())
+    phsigImage.setFilename(corfile)
+    phsigImage.setAccessMode('write')
+    phsigImage.createImage()
+
+    # setup Icu() object
+    icuObj = Icu(name='sentinel_filter_icu')
+    icuObj.configure()
+    icuObj.unwrappingFlag = False
+    icuObj.useAmplitudeFlag = False
+    #icuObj.correlationType = 'NOSLOPE'
+
+    # run
+    icuObj.icu(intImage=filtImage, phsigImage=phsigImage)
+    phsigImage.renderHdr()
+
+    # close
+    filtImage.finalizeImage()
+    phsigImage.finalizeImage()
+
+    return
+
+
+def unwrap_snaphu(int_file, cor_file, unw_file, defo_max=2.0, max_comp=32,
+                  init_only=True, init_method='MCF', cost_mode='SMOOTH'):
+    '''Unwrap interferograms using SNAPHU via isce2.
+
+    Modified from ISCE-2/topsStack/unwrap.py
+    Notes from Piyush:
+        SNAPHU is an iterative solver, starting from the initial solution. It can get
+            stuck in an infinite loop.
+        The initial solution is created using MCF or MST method. The MST initial solution
+            typically require lots of iterations and may not be a good starting point.
+        DEFO cost mode requires geometry info for the program to interpret the coherence
+            correctly and setup costs based on that. DEFO always sounds more theoretical
+            to me, but I haven not fully explored it. TOPO cost mode requires spatial baseline.
+            SMOOTH cost mode is purely data driven.
+        Amplitude of zero is a mask in all cost modes. For TOPO mode, amplitude is used to find
+            layover; for SMOOTH mode, only non-zero amplitude matters.
+
+    Default configurations in ISCE-2/topsStack:
+        init_only = True
+        init_method = 'MCF'
+        cost_mode = 'SMOOTH'
+    Default configurations in FRInGE:
+        init_only = False
+        init_method = 'MST'
+        cost_mode = 'DEFO'
+
+    Parameters: int_file    - str, path to the wrapped interferogram file
+                cor_file    - str, path to the correlation file
+                unw_file    - str, path to the output unwrapped interferogram file
+                defo_max    - float, maximum number of cycles for the deformation phase
+                max_comp    - int, maximum number of connected components
+                init_only   - bool, initlize-only mode
+                init_method - str, algo used for initialization: MCF, MST
+                cost_mode   - str, statistical-cost mode: TOPO, DEFO, SMOOTH, NOSTATCOSTS
+    Returns:    unw_file    - str, path to the output unwrapped interferogram file
+    '''
+    import isce
+    from contrib.Snaphu.Snaphu import Snaphu
+
+    start_time = time.time()
+
+    # configurations - atr
+    atr = readfile.read_attribute(int_file)
+    width = int(atr['WIDTH'])
+    length = int(atr['LENGTH'])
+    altitude = float(atr['HEIGHT'])
+    earth_radius = float(atr['EARTH_RADIUS'])
+    wavelength = float(atr['WAVELENGTH'])
+    rg_looks = int(atr['RLOOKS'])
+    az_looks = int(atr['ALOOKS'])
+    corr_looks = float(atr.get('NCORRLOOKS', rg_looks * az_looks / 1.94))
+
+    ## setup SNAPHU
+    # https://web.stanford.edu/group/radar/softwareandlinks/sw/snaphu/snaphu.conf.full
+    # https://github.com/isce-framework/isce2/blob/main/contrib/Snaphu/Snaphu.py
+    print('phase unwrapping with SNAPHU ...')
+    print('SNAPHU cost mode: {}'.format(cost_mode))
+    print('SNAPHU init only: {}'.format(init_only))
+    print('SNAPHU init method: {}'.format(init_method))
+    print('SNAPHU max number of connected components: {}'.format(max_comp))
+
+    snp = Snaphu()
+
+    # file IO
+    snp.setInput(int_file)
+    snp.setOutput(unw_file)
+    snp.setCorrfile(cor_file)
+    snp.setCorFileFormat('FLOAT_DATA')
+    snp.setWidth(width)
+
+    # runtime options
+    snp.setCostMode(cost_mode)
+    snp.setInitOnly(init_only)
+    snp.setInitMethod(init_method)
+
+    # geometry parameters
+    # baseline info is not used in deformation mode, but is very important in topography mode
+    snp.setAltitude(altitude)
+    snp.setEarthRadius(earth_radius)
+    snp.setWavelength(wavelength)
+    snp.setRangeLooks(rg_looks)
+    snp.setAzimuthLooks(az_looks)
+    snp.setCorrLooks(corr_looks)
+
+    # deformation mode parameters
+    snp.setDefoMaxCycles(defo_max)
+
+    # connected component control
+    # grow connectedc components if init_only is True
+    # https://github.com/isce-framework/isce2/blob/main/contrib/Snaphu/Snaphu.py#L413
+    snp.setMaxComponents(max_comp)
+
+    ## run SNAPHU
+    snp.prepare()
+    snp.unwrap()
+    print('finished SNAPHU running')
+
+    # mask out wired values from SNAPHU
+    # based on https://github.com/isce-framework/isce2/pull/326
+    flag = np.fromfile(int_file, dtype=np.complex64).reshape(length, width)
+    data = np.memmap(unw_file, dtype='float32', mode='r+', shape=(length*2, width))
+    data[0:length*2:2, :][np.nonzero(flag == 0)] = 0
+    data[1:length*2:2, :][np.nonzero(flag == 0)] = 0
+
+    ## render metadata
+    print('write metadata file: {}.xml'.format(unw_file))
+    atr['FILE_TYPE'] = '.unw'
+    atr['DATA_TYPE'] = 'float32'
+    atr['INTERLEAVE'] = 'BIL'
+    atr['BANDS'] = '2'
+    writefile.write_isce_xml(atr, unw_file)
+
+    if snp.dumpConnectedComponents:
+        print('write metadata file: {}.conncomp.xml'.format(unw_file))
+        atr['FILE_TYPE'] = '.conncomp'
+        atr['DATA_TYPE'] = 'uint8'
+        atr['INTERLEAVE'] = 'BIP'
+        atr['BANDS'] = '1'
+        writefile.write_isce_xml(atr, f'{unw_file}.conncomp')
+
+    # time usage
+    m, s = divmod(time.time() - start_time, 60)
+    print('time used: {:02.0f} mins {:02.1f} secs.'.format(m, s))
+
+    return unw_file
