@@ -10,6 +10,8 @@
 import os
 import sys
 import re
+import glob
+import datetime as dt
 import warnings
 import defusedxml.ElementTree as ET
 
@@ -26,32 +28,43 @@ from mintpy.objects import (
     timeseries,
     HDFEOS
 )
+from mintpy.objects import sensor
+from mintpy.utils import ptime, utils0 as ut
 
+SPEED_OF_LIGHT = 299792458  # meters per second
 
-SPEED_OF_LIGHT = 299792458.0   # meter/second
 
 standardMetadataKeys = {
     # ROI_PAC/MintPy attributes
     'ALOOKS'             : ['azimuth_looks'],
     'RLOOKS'             : ['range_looks'],
-    'AZIMUTH_PIXEL_SIZE' : ['azimuthPixelSize', 'azimuth_pixel_spacing', 'az_pixel_spacing'],
-    'RANGE_PIXEL_SIZE'   : ['rangePixelSize', 'range_pixel_spacing', 'rg_pixel_spacing'],
+    'AZIMUTH_PIXEL_SIZE' : ['azimuthPixelSize', 'azimuth_pixel_spacing', 'az_pixel_spacing', 'azimuth_spacing'],
+    'RANGE_PIXEL_SIZE'   : ['rangePixelSize', 'range_pixel_spacing', 'rg_pixel_spacing', 'range_spacing'],
     'CENTER_LINE_UTC'    : ['center_time'],
     'DATA_TYPE'          : ['dataType', 'data_type'],
     'EARTH_RADIUS'       : ['earthRadius', 'earth_radius_below_sensor', 'earth_radius'],
-    'HEADING'            : ['HEADING_DEG', 'heading'],
+    'HEADING'            : ['HEADING_DEG', 'heading', 'centre_heading'],
     'HEIGHT'             : ['altitude', 'SC_height'],
     'BANDS'              : ['number_bands', 'bands'],
     'INTERLEAVE'         : ['scheme', 'interleave'],
     'LENGTH'             : ['length', 'FILE_LENGTH', 'lines', 'azimuth_lines', 'nlines', 'az_samp',
-                            'interferogram_azimuth_lines'],
-    'ORBIT_DIRECTION'    : ['passDirection'],
-    'PLATFORM'           : ['spacecraftName', 'sensor'],
+                            'interferogram_azimuth_lines', 'num_output_lines'],
+    'LAT_REF1'           : ['first_near_lat'],
+    'LON_REF1'           : ['first_near_long'],
+    'LAT_REF2'           : ['first_far_lat'],
+    'LON_REF2'           : ['first_far_long'],
+    'LAT_REF3'           : ['last_near_lat'],
+    'LON_REF3'           : ['last_near_long'],
+    'LAT_REF4'           : ['last_far_lat'],
+    'LON_REF4'           : ['last_far_long'],
+    'ORBIT_DIRECTION'    : ['passDirection', 'pass'],
+    'NO_DATA_VALUE'      : ['NoDataValue'],
+    'PLATFORM'           : ['spacecraftName', 'sensor', 'mission'],
     'POLARIZATION'       : ['polarization'],
-    'PRF'                : ['prf'],
-    'STARTING_RANGE'     : ['startingRange', 'near_range_slc', 'near_range'],
+    'PRF'                : ['prf', 'pulse_repetition_frequency'],
+    'STARTING_RANGE'     : ['startingRange', 'near_range_slc', 'near_range', 'slant_range_to_first_pixel'],
     'WAVELENGTH'         : ['wavelength', 'Wavelength', 'radarWavelength', 'radar_wavelength'],
-    'WIDTH'              : ['width', 'Width', 'samples', 'range_samp', 'interferogram_width'],
+    'WIDTH'              : ['width', 'Width', 'samples', 'range_samp', 'interferogram_width', 'num_samples_per_line'],
     # from PySAR [MintPy<=1.1.1]
     'REF_DATE'           : ['ref_date'],
     'REF_LAT'            : ['ref_lat'],
@@ -75,48 +88,10 @@ standardMetadataKeys = {
     'relative_orbit' : ['trackNumber'],
 }
 
-GDAL2ISCE_DATATYPE = {
-    1 : 'BYTE',
-    2 : 'uint16',
-    3 : 'SHORT',
-    4 : 'uint32',
-    5 : 'INT',
-    6 : 'FLOAT',
-    7 : 'DOUBLE',
-    10: 'CFLOAT',
-    11: 'complex128',
-}
-
-GDAL2NUMPY_DATATYPE = {
-    1 : 'uint8',
-    2 : 'uint16',
-    3 : 'int16',
-    4 : 'uint32',
-    5 : 'int32',
-    6 : 'float32',
-    7 : 'float64',
-    10: 'complex64',
-    11: 'complex128',
-}
-
-NUMPY2GDAL_DATATYPE = {
-  "uint8"     : 1,
-  "int8"      : 1,
-  "uint16"    : 2,
-  "int16"     : 3,
-  "uint32"    : 4,
-  "int32"     : 5,
-  "float32"   : 6,
-  "float64"   : 7,
-  "complex64" : 10,
-  "complex128": 11,
-}
-
-# single file (data + attributes) supported by GDAL
-GDAL_FILE_EXTS = ['.tif', '.grd']
-
+## data type conventions [use numpy as reference]
+# 1 - ENVI
 # reference: https://subversion.renater.fr/efidir/trunk/efidir_soft/doc/Programming_C_EFIDIR/header_envi.pdf
-ENVI2NUMPY_DATATYPE = {
+DATA_TYPE_ENVI2NUMPY = {
     '1' : 'uint8',
     '2' : 'int16',
     '3' : 'int32',
@@ -130,6 +105,73 @@ ENVI2NUMPY_DATATYPE = {
     '15': 'uint64',
 }
 
+DATA_TYPE_NUMPY2ENVI = {
+    'uint8'     : '1',
+    'int16'     : '2',
+    'int32'     : '3',
+    'float32'   : '4',
+    'float64'   : '5',
+    'complex64' : '6',
+    'complex128': '9',
+    'uint16'    : '12',
+    'uint32'    : '13',
+    'int64'     : '14',
+    'uint64'    : '15',
+}
+
+# 2 - GDAL
+DATA_TYPE_GDAL2NUMPY = {
+    1 : 'uint8',
+    2 : 'uint16',
+    3 : 'int16',
+    4 : 'uint32',
+    5 : 'int32',
+    6 : 'float32',
+    7 : 'float64',
+    8 : 'cint16',       # for translation purpose only, as numpy does not support complex int
+    9 : 'cint32',       # for translation purpose only, as numpy does not support complex int
+    10: 'complex64',
+    11: 'complex128',
+}
+
+DATA_TYPE_NUMPY2GDAL = {
+    "uint8"     : 1,
+    "int8"      : 1,
+    "uint16"    : 2,
+    "int16"     : 3,
+    "uint32"    : 4,
+    "int32"     : 5,
+    "float32"   : 6,
+    "float64"   : 7,
+    "cint16"    : 8,    # for translation purpose only, as numpy does not support complex int
+    "cint32"    : 9,    # for translation purpose only, as numpy does not support complex int
+    "complex64" : 10,
+    "complex128": 11,
+}
+
+# 3 - ISCE
+DATA_TYPE_ISCE2NUMPY = {
+    'byte'  : 'uint8',
+    'short' : 'int16',
+    'int'   : 'int32',
+    'float' : 'float32',
+    'double': 'float64',
+    'cfloat': 'complex64',
+}
+
+DATA_TYPE_NUMPY2ISCE = {
+    'uint8'    : 'BYTE',
+    'int16'    : 'SHORT',
+    'int32'    : 'INT',
+    'float32'  : 'FLOAT',
+    'float64'  : 'DOUBLE',
+    'complex64': 'CFLOAT',
+}
+
+
+# single file (data + attributes) supported by GDAL
+GDAL_FILE_EXTS = ['.tiff', '.tif', '.grd']
+
 ENVI_BAND_INTERLEAVE = {
     'BAND' : 'BSQ',
     'LINE' : 'BIL',
@@ -142,6 +184,16 @@ ENVI_BYTE_ORDER = {
 }
 
 
+SPECIAL_STR2NUM = {
+    'yes'   : True,
+    'true'  : True,
+    'no'    : False,
+    'false' : False,
+    'none'  : None,
+    'nan'   : np.nan,
+}
+
+
 ###########################################################################
 ## Slice-based data identification for data reading:
 ##
@@ -202,7 +254,7 @@ def read(fname, box=None, datasetName=None, print_msg=True, xstep=1, ystep=1, da
                 box         : 4-tuple of int area to read, defined in (x0, y0, x1, y1) in pixel coordinate
                 x/ystep     : int, number of pixels to pick/multilook for each output pixel
                 data_type   : numpy data type, e.g. np.float32, np.bool_, etc.
-    Returns:    data        : 2/3-D matrix in numpy.array format, return None if failed
+    Returns:    data        : 2/3/4D matrix in numpy.array format, return None if failed
                 atr         : dictionary, attributes of data, return None if failed
     Examples:
         from mintpy.utils import readfile
@@ -272,7 +324,7 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
                     ...
                 box         : 4-tuple of int area to read, defined in (x0, y0, x1, y1) in pixel coordinate
                 x/ystep     : int, number of pixels to pick/multilook for each output pixel
-    Returns:    data        : 2D/3D array
+    Returns:    data        : 2/3/4D array
                 atr         : dict, metadata
     """
     # File Info: list of slice / dataset / dataset2d / dataset3d
@@ -305,7 +357,8 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
     with h5py.File(fname, 'r') as f:
         # get dataset object
         dsNames = [i for i in [datasetName[0], dsFamily] if i in f.keys()]
-        dsNamesOld = [i for i in slice_list if '/{}'.format(datasetName[0]) in i] # support for old mintpy files
+        # support for old mintpy-v0.x files
+        dsNamesOld = [i for i in slice_list if '/{}'.format(datasetName[0]) in i]
         if len(dsNames) > 0:
             ds = f[dsNames[0]]
         elif len(dsNamesOld) > 0:
@@ -313,6 +366,10 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
         else:
             raise ValueError('input dataset {} not found in file {}'.format(datasetName, fname))
 
+        # output size for >=2D dataset if x/ystep > 1
+        xsize = int((box[2] - box[0]) / xstep)
+        ysize = int((box[3] - box[1]) / ystep)
+
         # 2D dataset
         if ds.ndim == 2:
             # read data
@@ -321,11 +378,6 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
 
             # sampling / nearest interplation in y/xstep
             if xstep * ystep > 1:
-                # output size if x/ystep > 1
-                xsize = int((box[2] - box[0]) / xstep)
-                ysize = int((box[3] - box[1]) / ystep)
-
-                # sampling
                 data = data[int(ystep/2)::ystep,
                             int(xstep/2)::xstep]
                 data = data[:ysize, :xsize]
@@ -343,30 +395,35 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
                     slice_flag[date_list.index(d)] = True
 
             # read data
+            num_slice = np.sum(slice_flag)
+            inds = np.where(slice_flag)[0].tolist()
+
             if xstep * ystep == 1:
-                data = ds[slice_flag,
-                          box[1]:box[3],
-                          box[0]:box[2]]
+                if num_slice / slice_flag.size < 0.05:
+                    # single indexing if only a small fraction is read
+                    data = np.zeros((num_slice, ysize, xsize), dtype=ds.dtype)
+                    for i, ind in enumerate(inds):
+                        data[i] = ds[ind,
+                                     box[1]:box[3],
+                                     box[0]:box[2]]
+                else:
+                    data = ds[:,
+                              box[1]:box[3],
+                              box[0]:box[2]][slice_flag]
 
             else:
-                # output size if x/ystep > 1
-                xsize = int((box[2] - box[0]) / xstep)
-                ysize = int((box[3] - box[1]) / ystep)
-
                 # sampling / nearest interplation in y/xstep
                 # use for loop to save memory
-                num_slice = np.sum(slice_flag)
                 data = np.zeros((num_slice, ysize, xsize), ds.dtype)
 
-                inds = np.where(slice_flag)[0]
-                for i in range(num_slice):
+                for i, ind in enumerate(inds):
                     # print out msg
                     if print_msg:
-                        sys.stdout.write('\r' + f'reading slice {i+1}/{num_slice}...')
+                        sys.stdout.write('\r' + f'reading 2D slices {i+1}/{num_slice}...')
                         sys.stdout.flush()
 
                     # read and index
-                    d2 = ds[inds[i],
+                    d2 = ds[ind,
                             box[1]:box[3],
                             box[0]:box[2]]
                     d2 = d2[int(ystep/2)::ystep,
@@ -379,6 +436,42 @@ def read_hdf5_file(fname, datasetName=None, box=None, xstep=1, ystep=1, print_ms
             if any(i == 1 for i in data.shape):
                 data = np.squeeze(data)
 
+        # 4D dataset
+        elif ds.ndim == 4:
+            # custom flag for the time domain is ignore for now
+            # a.k.a. read the entire first 2 dimensions
+
+            num1, num2 = ds.shape[0], ds.shape[1]
+            shape = (num1, num2, ysize, xsize)
+            if print_msg:
+                ram_size = num1 * num2 * ysize * xsize * ds.dtype.itemsize / 1024**3
+                print(f'initiate a 4D matrix in size of {shape} in {ds.dtype} in the memory ({ram_size:.1f} GB) ...')
+            data = np.zeros(shape, ds.dtype) * np.nan
+
+            # loop over the 1st dimension [for more verbose print out msg]
+            for i in range(num1):
+                if print_msg:
+                    sys.stdout.write('\r' + f'reading 3D cubes {i+1}/{num1}...')
+                    sys.stdout.flush()
+
+                d3 = ds[i, :,
+                        box[1]:box[3],
+                        box[0]:box[2]]
+
+                # sampling / nearest interpolation in y/xstep
+                if xstep * ystep > 1:
+                    d3 = d3[:,
+                            int(ystep/2)::ystep,
+                            int(xstep/2)::xstep]
+
+                data[i, :, :, :] = d3[:, :ysize, :xsize]
+
+            if print_msg:
+                print('')
+
+            if any(i == 1 for i in data.shape):
+                data = np.squeeze(data)
+
     return data
 
 
@@ -426,7 +519,7 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
             data_type = dataTypeDict[data_type]
 
         k = atr['FILE_TYPE'].lower().replace('.', '')
-        if k in ['unw', 'cor']:
+        if k in ['unw', 'cor', 'ion']:
             band = min(2, num_band)
             if datasetName and datasetName in ['band1','intensity','magnitude']:
                 band = 1
@@ -547,8 +640,12 @@ def read_binary_file(fname, datasetName=None, box=None, xstep=1, ystep=1):
             byte_order = 'little-endian'
 
     # GDAL / GMTSAR / ASF HyP3
-    elif processor in ['gdal', 'gmtsar', 'hyp3', 'cosicorr']:
-        pass
+    elif processor in ['gdal', 'gmtsar', 'hyp3', 'cosicorr', 'uavsar']:
+        # try to recognize custom dataset names if specified and recognized.
+        if datasetName:
+            slice_list = get_slice_list(fname)
+            if datasetName in slice_list:
+                band = slice_list.index(datasetName) + 1
 
     else:
         print('Unknown InSAR processor: {}'.format(processor))
@@ -632,6 +729,11 @@ def get_slice_list(fname, no_complex=False):
             obj.open(print_msg=False)
             slice_list = obj.sliceList
 
+        elif k == 'timeseries' and 'slc' in d1_list:
+            with h5py.File(fname, 'r') as f:
+                dates = f['date'][:]
+            slice_list = ['slc-{}'.format(i.decode('UTF-8')) for i in dates]
+
         else:
             ## Find slice by walking through the file structure
             length, width = int(atr['LENGTH']), int(atr['WIDTH'])
@@ -659,7 +761,7 @@ def get_slice_list(fname, no_complex=False):
             # isce los file
             slice_list = ['incidenceAngle', 'azimuthAngle']
 
-        elif fext in ['.unw']:
+        elif fext in ['.unw', '.ion']:
             slice_list = ['magnitude', 'phase']
 
         elif fext in ['.int', '.slc']:
@@ -669,11 +771,19 @@ def get_slice_list(fname, no_complex=False):
                 slice_list = ['complex']
 
         elif fbase.startswith('off') and fext in ['.bip'] and num_band == 2:
+            # ampcor offset file
             slice_list = ['azimuthOffset', 'rangeOffset']
 
         elif fbase.startswith('off') and fname.endswith('cov.bip') and num_band == 3:
+            # ampcor offset covariance file
             slice_list = ['azimuthOffsetVar', 'rangeOffsetVar', 'offsetCovar']
 
+        elif fext in ['.lkv']:
+            slice_list = ['east', 'north', 'up']
+
+        elif fext in ['.llh']:
+            slice_list = ['latitude', 'longitude', 'height']
+
         else:
             slice_list = ['band{}'.format(i+1) for i in range(num_band)]
 
@@ -719,6 +829,20 @@ def get_hdf5_compression(fname):
     return compression
 
 
+def get_no_data_value(fname):
+    """Grab the NO_DATA_VALUE of the input file."""
+    val = read_attribute(fname).get('NO_DATA_VALUE', None)
+    val = str(val).lower()
+
+    if ut.is_number(val):
+        val = float(val)
+    elif val in SPECIAL_STR2NUM.keys():
+        val = SPECIAL_STR2NUM[val]
+    else:
+        raise ValueError(f'Un-recognized no-data-value type: {val}')
+    return val
+
+
 #########################################################################
 def read_attribute(fname, datasetName=None, metafile_ext=None):
     """Read attributes of input file into a dictionary
@@ -731,6 +855,7 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
                          ...
     Returns:    atr : dict, attributes dictionary
     """
+    fdir = os.path.dirname(fname)
     fbase, fext = os.path.splitext(os.path.basename(fname))
     fext = fext.lower()
     if not os.path.isfile(fname):
@@ -877,6 +1002,45 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
             msg += '{} * {} in {}!'.format(atr['LENGTH'], atr['WIDTH'], atr['DATA_TYPE'])
             raise ValueError(msg)
 
+    elif fext in ['.lkv', '.llh']:
+        # UAVSAR geometry file
+        # link: https://uavsar.jpl.nasa.gov/science/documents/stack-format.html
+        site, line, version, bcorr = fbase.split('_')[:4]
+        ann_files = glob.glob(os.path.join(fdir, f'{site}_{line}_*_{version}_{bcorr}.ann'))
+        if len(ann_files) > 0:
+            ann = read_uavsar_ann(ann_files[0])
+            atr = {}
+
+            # data size
+            seg, mli = fbase.split('_')[-2:]
+            if seg.startswith('s'):
+                # single segment file
+                atr['LENGTH'] = ann[f'slc_{seg[1:]}_{mli} Rows']
+                atr['WIDTH']  = ann[f'slc_{seg[1:]}_{mli} Columns']
+
+            else:
+                # merged/concatenated file
+                num_seg = int(ann['Number of Segments'])
+                length, width = 0, 0
+                for i in range(1, num_seg+1):
+                    length += int(ann[f'slc_{i}_{mli} Rows'])
+                    width = int(ann[f'slc_{i}_{mli} Columns'])
+                atr['LENGTH'] = str(length)
+                atr['WIDTH'] = str(width)
+
+            atr['ANTENNA_SIDE'] = '1' if ann['Look Direction'].lower() == 'left' else '-1'
+            atr['PROCESSOR'] = 'uavsar'
+            atr['PLATFORM'] = 'uavsar'
+            atr['FILE_TYPE'] = fext
+            atr['DATA_TYPE'] = 'float32'
+            atr['RLOOKS'] = mli.split('x')[0]
+            atr['ALOOKS'] = mli.split('x')[1]
+            atr['BANDS'] = '3'
+            atr['INTERLEAVE'] = 'BIP'
+
+        else:
+            raise FileNotFoundError('No UAVSAR *.ann file found!')
+
     else:
         # grab all existed potential metadata file given the data file in prefered order/priority
         # .aux.xml file does not have geo-coordinates info
@@ -895,13 +1059,12 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
         if metafile_ext:
             metafiles = [i for i in metafiles if i.endswith(metafile_ext)]
 
-        # use the GDAL supported data file is no metadata file found
-        if len(metafiles) == 0:
-            # for .tif/.grd files, extract metadata from the file itself
-            if fext in GDAL_FILE_EXTS:
-                metafiles = [fname]
-            else:
-                raise FileNotFoundError('No metadata file found for data file: {}'.format(fname))
+        # for .tif/.grd files, priority:
+        # .rsc > file itself > .xml/.aux.xml/.hdr etc.
+        if fext in GDAL_FILE_EXTS and not os.path.isfile(fname + '.rsc'):
+            metafiles = [fname]
+        elif len(metafiles) == 0:
+            raise FileNotFoundError('No metadata file found for data file: {}'.format(fname))
 
         atr = {}
         # PROCESSOR
@@ -1000,6 +1163,9 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
 
     elif datasetName and datasetName in datasetUnitDict.keys():
         atr['UNIT'] = datasetUnitDict[datasetName]
+        # SLC stack
+        if datasetName == 'timeseries' and atr.get('DATA_TYPE', 'float32').startswith('complex'):
+            atr['UNIT'] = '1'
 
     elif 'UNIT' not in atr.keys():
         if k in datasetUnitDict.keys():
@@ -1007,6 +1173,10 @@ def read_attribute(fname, datasetName=None, metafile_ext=None):
         else:
             atr['UNIT'] = '1'
 
+    # NO_DATA_VALUE
+    no_data_value = atr.get('NO_DATA_VALUE', None)
+    atr['NO_DATA_VALUE'] = str(no_data_value).lower()
+
     # FILE_PATH
     if 'FILE_PATH' in atr.keys() and 'OG_FILE_PATH' not in atr.keys():
         # need to check original source file to successfully subset legacy-sensor products
@@ -1043,91 +1213,50 @@ def standardize_metadata(metaDictIn, standardKeys=None):
 
 
 #########################################################################
-def read_template(fname, delimiter='=', print_msg=True):
-    """Reads the template file into a python dictionary structure.
-    Parameters: fname : str
-                    full path to the template file
-                delimiter : str
-                    string to separate the key and value
-                print_msg : bool
-                    print message or not
-    Returns:    template_dict : dict
-                    file content
-    Examples:
-        tmpl = read_template(KyushuT424F610_640AlosA.template)
-        tmpl = read_template(R1_54014_ST5_L0_F898.000.pi, ':')
+def read_template(fname, delimiter='=', skip_chars=None):
+    """Read the template file into a dictionary structure.
+
+    Parameters: fname      - str, full path to the template file
+                delimiter  - str, string to separate the key and value
+                skip_chars - list of str, skip certain charaters in values
+    Returns:    template   - dict, file content
+    Examples:   template = read_template('KyushuAlosAT424.txt')
+                template = read_template('smallbaselineApp.cfg')
     """
-    template_dict = {}
 
-    # insarmaps: the below logic for plotattributes object can be made much more simple
-    # if we assume that any plot attribute coming after a > belongs to the
-    # same object. Must Ask Falk and Yunjun if we can assume this to eliminate
-    # all these conditionals
-    plotAttributeDict = {}
-    insidePlotObject = False
-    plotAttributes = []
+    if skip_chars and isinstance(skip_chars, str):
+        skip_chars = [skip_chars]
 
-    def is_plot_attribute(attribute):
-        tokens = attribute.split(".")
-        if tokens is None:
-            return False
-        return tokens[0] == "plot" and len(tokens) > 1
-
-    # read input text file or string
-    lines = None
+    # read input text file / string
     if os.path.isfile(fname):
         with open(fname, 'r') as f:
             lines = f.readlines()
     elif isinstance(fname, str):
         lines = fname.split('\n')
+    lines = [x.strip() for x in lines]
 
-    # loop to parser/read each line
+    # parse line by line
+    template = {}
     for line in lines:
-        line = line.strip()
         # split on the 1st occurrence of delimiter
         c = [i.strip() for i in line.split(delimiter, 1)]
 
         # ignore commented lines or those without variables
-        if len(c) < 2 or line.startswith(('%', '#', '!')):
-            # next
-
-            # insarmaps:
-            if line.startswith(">"):
-                plotAttributeDict = {}
-                insidePlotObject = True
-            # otherwise, if previously inside attributes object, we are now outside
-            # unless the line is a comment
-            elif insidePlotObject and not line.startswith('%') and not line.startswith('#'):
-                # just came from being inside plot object, but now we are outside
-                insidePlotObject = False
-                plotAttributes.append(plotAttributeDict)
-
-        else:
+        if len(c) >= 2 and not line.startswith(('%', '#', '!')):
             key = c[0]
             value = str.replace(c[1], '\n', '').split("#")[0].strip()
             value = os.path.expanduser(value)  # translate ~ symbol
             value = os.path.expandvars(value)  # translate env variables
 
+            # skip certain characters by replacing them with empty str
+            if skip_chars:
+                for skip_char in skip_chars:
+                    value.replace(skip_char, '')
+
             if value != '':
-                template_dict[key] = value
+                template[key] = value
 
-            # insarmaps:
-            if insidePlotObject:
-                if is_plot_attribute(key):
-                    plotAttributeDict[key] = value
-                else:
-                    # just came from being inside plot object, but now we are outside
-                    insidePlotObject = False
-                    plotAttributes.append(plotAttributeDict)
-                    template_dict[key] = value
-
-    # insarmaps: what if no \n at end of file? write out last plot attributes dict
-    if insidePlotObject:
-        plotAttributes.append(plotAttributeDict)
-    if len(plotAttributes) > 0:
-        template_dict["plotAttributes"] = json.dumps(plotAttributes)
-
-    return template_dict
+    return template
 
 
 def read_roipac_rsc(fname, delimiter=' '):
@@ -1261,21 +1390,25 @@ def read_isce_xml(fname):
     # imageFile, e.g. filt_fine.unw.xml
     if root.tag.startswith('image'):
         for child in root.findall('property'):
-            key = child.get('name')
+            key = child.get('name').lower()
             value = child.find('value').text
             xmlDict[key] = value
 
         # Read lat/lon info for geocoded file
         # in form: root/component coordinate*/property name/value
         for coord_name, prefix in zip(['coordinate1', 'coordinate2'], ['X', 'Y']):
-            child = root.find("./component[@name='{}']".format(coord_name))
-            v_step  = float(child.find("./property[@name='delta']").find('value').text)
-            v_first = float(child.find("./property[@name='startingvalue']").find('value').text)
-            if abs(v_step) < 1. and abs(v_step) > 1e-7:
+            e_step  = root.find(f"./component[@name='{coord_name}']/property[@name='delta']")
+            e_first = root.find(f"./component[@name='{coord_name}']/property[@name='startingvalue']")
+            v_step  = float(e_step.find('value').text)  if e_step  is not None else None
+            v_first = float(e_first.find('value').text) if e_first is not None else None
+            if v_step and v_first and abs(v_step) < 1. and abs(v_step) > 1e-7:
                 xmlDict['{}_STEP'.format(prefix)] = v_step
                 xmlDict['{}_FIRST'.format(prefix)] = v_first - v_step / 2.
                 xmlDict['{}_UNIT'.format(prefix)] = 'degrees'
 
+        # data_type
+        xmlDict['data_type'] = DATA_TYPE_ISCE2NUMPY[xmlDict['data_type'].lower()]
+
     # PAMDataset, e.g. hgt.rdr.aux.xml
     elif root.tag == 'PAMDataset':
         meta = root.find("./Metadata[@domain='ENVI']")
@@ -1283,8 +1416,16 @@ def read_isce_xml(fname):
             key = child.get('key')
             value = child.text
             xmlDict[key] = value
-        xmlDict['data_type'] = ENVI2NUMPY_DATATYPE[xmlDict['data_type']]
 
+        # data_type
+        xmlDict['data_type'] = DATA_TYPE_ENVI2NUMPY[xmlDict['data_type']]
+
+        # NoDataValue
+        meta = root.find("./PAMRasterBand/NoDataValue")
+        if meta is not None:
+            xmlDict['NoDataValue'] = meta.text
+
+    # standardize metadata keys
     xmlDict = standardize_metadata(xmlDict)
 
     return xmlDict
@@ -1293,7 +1434,7 @@ def read_isce_xml(fname):
 def read_envi_hdr(fname):
     """Read ENVI .hdr file into a python dict structure"""
     atr = read_template(fname, delimiter='=')
-    atr['DATA_TYPE'] = ENVI2NUMPY_DATATYPE[atr.get('data type', '4')]
+    atr['DATA_TYPE'] = DATA_TYPE_ENVI2NUMPY[atr.get('data type', '4')]
     atr['BYTE_ORDER'] = ENVI_BYTE_ORDER[atr.get('byte order', '1')]
 
     if 'map info' in atr.keys():
@@ -1326,7 +1467,9 @@ def read_gdal_vrt(fname):
         raise ImportError('Cannot import gdal and osr!')
 
     # read dataset using gdal
-    ds = gdal.Open(fname, gdal.GA_ReadOnly)
+    # Using os.fspath to convert Path objects to str, recommended by
+    # https://github.com/OSGeo/gdal/issues/1613#issuecomment-824703596
+    ds = gdal.Open(os.fspath(fname), gdal.GA_ReadOnly)
 
     atr = {}
     atr['WIDTH']  = ds.RasterXSize
@@ -1334,8 +1477,7 @@ def read_gdal_vrt(fname):
     atr['BANDS'] = ds.RasterCount
 
     # data type
-    data_type = ds.GetRasterBand(1).DataType
-    atr['DATA_TYPE'] = GDAL2ISCE_DATATYPE[data_type]
+    atr['DATA_TYPE'] = DATA_TYPE_GDAL2NUMPY[ds.GetRasterBand(1).DataType]
 
     # interleave
     interleave = ds.GetMetadata('IMAGE_STRUCTURE').get('INTERLEAVE', 'PIXEL')
@@ -1370,11 +1512,35 @@ def read_gdal_vrt(fname):
         if atr['X_FIRST'] > 180.:
             atr['X_FIRST'] -= 360.
 
+    # no data value
+    atr['NoDataValue'] = ds.GetRasterBand(1).GetNoDataValue()
+
     atr = standardize_metadata(atr)
 
     return atr
 
 
+def read_uavsar_ann(fname, comment=';', delimiter='='):
+    """Read the UAVSAR annotation file into dictionary.
+    """
+    # read the entirer text file into list of strings
+    lines = None
+    with open(fname, 'r') as f:
+        lines = f.readlines()
+
+    # convert the list of strings into a dict object
+    meta = {}
+    for line in lines:
+        line = line.strip()
+        c = [x.strip() for x in line.split(delimiter, 1)]
+        if len(c) >= 2 and not line.startswith(comment):
+            key = c[0].split('(')[0].strip()
+            value = str.replace(c[1], '\n', '').split(comment)[0].strip()
+            meta[key] = value
+
+    return meta
+
+
 def read_gmtsar_prm(fname, delimiter='='):
     """Read GMTSAR .prm file into a python dict structure.
     Parameters: fname : str.
@@ -1439,6 +1605,108 @@ def attribute_gmtsar2roipac(prm_dict_in):
     return prm_dict
 
 
+def read_snap_dim(fname):
+    """Read SNAP *.dim file into a python dict structure.
+    Parameters: fname - str, path of the *.dim file
+    Returns:    dim_dict - dict, ditionary of keys and values
+    Examples:   from mintpy.utils import readfile
+                atr = readfile.read_snap_dim('20190303_20190408_unw_tc.dim')
+    """
+    root = ET.parse(fname).getroot()
+    ds = root.find("Dataset_Sources/MDElem[@name='metadata']/MDElem[@name='Abstracted_Metadata']")
+
+    # initial dict elements
+    dim_dict = {}
+    dim_dict['PROCESSOR'] = 'snap'
+    dim_dict['FILE_TYPE'] = os.path.basename(fname).split('_')[-2].lower()
+
+    # read abstracted_metadata into dict
+    for child in ds.findall('MDATTR'):
+        key = child.get('name').lower()
+        value = child.text
+        dim_dict[key] = value
+
+    ## 1. standardize
+    dim_dict = standardize_metadata(dim_dict)
+
+    ## 2. extra calculations
+    # antenna_side
+    dim_dict['ANTENNA_SIDE'] = '-1' if dim_dict['antenna_pointing'] == 'right' else '1'
+
+    # center_line_utc
+    start_utc = dt.datetime.strptime(dim_dict['first_line_time'], '%d-%b-%Y %H:%M:%S.%f')
+    end_utc   = dt.datetime.strptime(dim_dict['last_line_time'],  '%d-%b-%Y %H:%M:%S.%f')
+    center_utc = start_utc + ((end_utc - start_utc) / 2)
+    center_secs = (center_utc.hour * 3600.0 +
+                   center_utc.minute * 60. +
+                   center_utc.second)
+    dim_dict['CENTER_LINE_UTC'] = center_secs
+
+    # data_type / length / width
+    band = root.find("Image_Interpretation/Spectral_Band_Info")
+    dim_dict['DATA_TYPE'] = band.find("DATA_TYPE").text
+    dim_dict['LENGTH']    = band.find("BAND_RASTER_HEIGHT").text
+    dim_dict['WIDTH']     = band.find("BAND_RASTER_WIDTH").text
+
+    # earth_radius + height
+    orbit = ds.find("MDElem[@name='Orbit_State_Vectors']/MDElem[@name='orbit_vector1']")
+    x_pos = float(orbit.find("MDATTR[@name='x_pos']").text)
+    y_pos = float(orbit.find("MDATTR[@name='y_pos']").text)
+    z_pos = float(orbit.find("MDATTR[@name='z_pos']").text)
+    height, radius = ut.xyz_to_local_radius((x_pos, y_pos, z_pos))
+    dim_dict['HEIGHT'] = height
+    dim_dict['EARTH_RADIUS'] = radius
+
+    # platform
+    dim_dict['PLATFORM'] = sensor.standardize_sensor_name(dim_dict['PLATFORM'])
+
+    # wavelength
+    # radar_frequency is in the unit of MHz
+    dim_dict['WAVELENGTH'] = SPEED_OF_LIGHT / (float(dim_dict['radar_frequency']) * 1e6)
+
+    # x/y_first/step_unit
+    transform = root.find("Geoposition/IMAGE_TO_MODEL_TRANSFORM").text.split(',')
+    transform = [str(float(i)) for i in transform]     # Convert 3.333e-4 to 0.0003333
+    dim_dict["X_STEP"]  = transform[0]
+    dim_dict["Y_STEP"]  = transform[3]
+    dim_dict["X_FIRST"] = transform[4]
+    dim_dict["Y_FIRST"] = transform[5]
+    dim_dict["X_UNIT"]  = "degrees"
+    dim_dict["Y_UNIT"]  = "degrees"
+
+    ## 3. geometry datasets
+    # incidence_angle
+    dim_dict['INCIDENCE_ANGLE'] = (float(dim_dict['incidence_near']) + float(dim_dict['incidence_near'])) / 2.0
+
+    # slant_range_distance
+    dim_dict['SLANT_RANGE_DISTANCE'] = ut.incidence_angle2slant_range_distance(dim_dict, dim_dict['INCIDENCE_ANGLE'])
+
+    ## 4. specific to each interferogram
+    bases = ds.find("MDElem[@name='Baselines']").findall("MDElem")[0].findall("MDElem")
+
+    # date12
+    dates = [x.get('name').split(':')[1].strip() for x in bases]
+    [date1, date2] = sorted([dt.datetime.strptime(x, '%d%b%Y').strftime('%Y%m%d') for x in dates])
+    dim_dict['DATE12'] = f'{date1[2:]}-{date2[2:]}'
+
+    # p_baseline
+    pbases = [x.find("MDATTR[@name='Perp Baseline']").text for x in bases]
+    pbase = [x for x in pbases if float(x) != 0][0]
+    dim_dict['P_BASELINE_TOP_HDR'] = pbase
+    dim_dict['P_BASELINE_BOTTOM_HDR'] = pbase
+
+    # convert all key & value in string format
+    for key, value in dim_dict.items():
+        dim_dict[key] = str(value)
+
+    # ensure int type metadata value
+    for key in ['ALOOKS', 'RLOOKS', 'LENGTH', 'WIDTH']:
+        if key in dim_dict.keys():
+            dim_dict[key] = str(int(float(dim_dict[key])))
+
+    return dim_dict
+
+
 
 #########################################################################
 def read_binary(fname, shape, box=None, data_type='float32', byte_order='l',
@@ -1551,7 +1819,9 @@ def read_gdal(fname, box=None, band=1, cpx_band='phase', xstep=1, ystep=1):
         raise ImportError('Cannot import gdal!')
 
     # open data file
-    ds = gdal.Open(fname, gdal.GA_ReadOnly)
+    # Using os.fspath to convert Path objects to str, recommended by
+    # https://github.com/OSGeo/gdal/issues/1613#issuecomment-824703596
+    ds = gdal.Open(os.fspath(fname), gdal.GA_ReadOnly)
     bnd = ds.GetRasterBand(band)
 
     # box
@@ -1567,7 +1837,7 @@ def read_gdal(fname, box=None, band=1, cpx_band='phase', xstep=1, ystep=1):
     data = bnd.ReadAsArray()[box[1]:box[3], box[0]:box[2]]
 
     # adjust output band for complex data
-    data_type = GDAL2ISCE_DATATYPE[bnd.DataType]
+    data_type = DATA_TYPE_GDAL2NUMPY[bnd.DataType]
     if data_type.replace('>', '').startswith('c'):
         if cpx_band.startswith('real'):
             data = data.real
@@ -1596,6 +1866,89 @@ def read_gdal(fname, box=None, band=1, cpx_band='phase', xstep=1, ystep=1):
     return data
 
 
+
+############################  Read GMT Faults  ################################
+
+def read_gmt_lonlat_file(ll_file, SNWE=None, min_dist=10):
+    """Read GMT lonlat file into list of 2D np.ndarray.
+
+    Parameters: ll_file  - str, path to the GMT lonlat file
+                SNWE     - tuple of 4 float, area of interest in lat/lon
+                min_dist - float, minimum distance in km of fault segments
+    Returns:    faults   - list of 2D np.ndarray in size of [num_point, 2] in float32
+                           with each row for one point in [lon, lat] in degrees
+    Examples:
+        # prepare GMT lonlat file
+        cd ~/data/aux/faults
+        gmt kml2gmt UCERF3_Fault.kml > UCERF3_Fault.lonlat
+
+        # read faults data
+        ll_file = os.path.expanduser('~/data/aux/faults/UCERF3_Fault.lonlat')
+        faults = read_gmt_lonlat_file(ll_file, SNWE=(31, 36, -118, -113), min_dist=0.1)
+
+        # add faults to the existing plot
+        fig, ax = plt.subplots(figsize=[7, 7], subplot_kw=dict(projection=ccrs.PlateCarree()))
+        data, atr, inps = view.prep_slice(cmd)
+        ax, inps, im, cbar = view.plot_slice(ax, data, atr, inps)
+
+        prog_bar = ptime.progressBar(maxValue=len(faults))
+        for i, fault in enumerate(faults):
+            ax.plot(fault[:,0], fault[:,1], 'k-', lw=0.2)
+            prog_bar.update(i+1, every=10)
+        prog_bar.close()
+        ax.set_xlim(inps.geo_box[0], inps.geo_box[2])
+        ax.set_ylim(inps.geo_box[3], inps.geo_box[1])
+
+    """
+    # read text file
+    lines = None
+    with open(ll_file, 'r') as f:
+        lines = f.readlines()
+    lines = [x for x in lines if not x.startswith('#')]
+
+    debug_mode = False
+    if debug_mode:
+        lines = lines[:1000]
+
+    # loop to extract/organize the data into list of arrays
+    num_line = len(lines)
+    faults = []
+    fault = []
+    prog_bar = ptime.progressBar(maxValue=num_line)
+    for i, line in enumerate(lines):
+        line = line.strip().replace('\n','').replace('\t', ' ')
+        if line.startswith('>'):
+            fault = []
+        else:
+            fault.append([float(x) for x in line.split()[:2]])
+
+        # save if 1) this is the last line OR 2) the next line starts a new fault
+        if i == num_line - 1 or lines[i+1].startswith('>'):
+            fault = np.array(fault, dtype=np.float32)
+            s = np.nanmin(fault[:,1]); n = np.nanmax(fault[:,1])
+            w = np.nanmin(fault[:,0]); e = np.nanmax(fault[:,0])
+
+            if fault is not None and SNWE:
+                S, N, W, E = SNWE
+                if e < W or w > E or s > N or n < S:
+                    # check overlap of two rectangles
+                    # link: https://stackoverflow.com/questions/40795709
+                    fault = None
+
+            if fault is not None and min_dist > 0:
+                dist = abs(n - s) * 108 * abs(e - w) * 108 * np.cos((n+s)/2 * np.pi/180)
+                if dist < min_dist:
+                    fault = None
+
+            if fault is not None:
+                faults.append(fault)
+
+        prog_bar.update(i+1, every=1000, suffix=f'line {i+1} / {num_line}')
+    prog_bar.close()
+    return faults
+
+
+
 ############################ Obsolete Functions ###############################
 def read_float32(fname, box=None, byte_order='l'):
     """Reads roi_pac data (RMG format, interleaved line by line).
@@ -1858,5 +2211,4 @@ def read_GPS_USGS(fname):
 
     return east, north, up, dates, YYYYMMDD
 
-
 #########################################################################
