@@ -19,115 +19,128 @@ from mintpy.objects import (
     timeseries,
 )
 
-from mintpy.utils import ptime, readfile
+from mintpy.utils import ptime, readfile, attribute as attr
 from mintpy.utils.utils0 import *
 from mintpy.utils.utils1 import *
 from mintpy.objects.coord import coordinate
+from mintpy.objects.resample import resample
 
 
 #################################################################################
 def check_loaded_dataset(work_dir='./', print_msg=True, relpath=False):
-    """Check the result of loading data for the following two rules:
+    """Check the loaded input files, following two rules:
         1. file existance
         2. file attribute readability
 
-    Parameters: work_dir  : string, MintPy working directory
-                print_msg : bool, print out message
-    Returns:    True, if all required files and dataset exist; otherwise, ERROR
-                    If True, PROCESS, SLC folder could be removed.
-                stack_file  :
-                geom_file   :
-                lookup_file :
+    Parameters: work_dir    - str, MintPy working directory
+                print_msg   - bool, print out message
+    Returns:    stack_file  - str, path to the interferogram stack file
+                geom_file   - str, path to the geometry file
+                lookup_file - str, path to the look up table file, for radar-coord dataset only.
+                ion_file    - str, path to the ionosphere stack file
     Example:    work_dir = os.path.expandvars('./FernandinaSenDT128/mintpy')
-                ut.check_loaded_dataset(work_dir)
+                stack_file, geom_file, lookup_file = ut.check_loaded_dataset(work_dir)[:3]
     """
-    load_complete = True
-
     if not work_dir:
         work_dir = os.getcwd()
     work_dir = os.path.abspath(work_dir)
 
-    # 1. interferograms stack file: unwrapPhase, coherence
-    ds_list = ['unwrapPhase', 'rangeOffset', 'azimuthOffset']
-    flist = [os.path.join(work_dir, 'inputs/ifgramStack.h5')]
-    stack_file = is_file_exist(flist, abspath=True)
-    if stack_file is not None:
+    # 1. [required] interferograms stack file: unwrapPhase, coherence
+    stack_file = os.path.join(work_dir, 'inputs/ifgramStack.h5')
+    dnames = ['unwrapPhase', 'rangeOffset', 'azimuthOffset']
+
+    if is_file_exist(stack_file, abspath=True):
         obj = ifgramStack(stack_file)
         obj.open(print_msg=False)
-        if all(x not in obj.datasetNames for x in ds_list):
-            msg = 'required dataset is missing in file {}:'.format(stack_file)
-            msg += '\n' + ' OR '.join(ds_list)
+
+        if all(x not in obj.datasetNames for x in dnames):
+            msg = f'required dataset is missing in file {stack_file}:\n'
+            msg += ' OR '.join(dnames)
             raise ValueError(msg)
+
         # check coherence for phase stack
         if 'unwrapPhase' in obj.datasetNames and 'coherence' not in obj.datasetNames:
-            print('WARNING: "coherence" is missing in file {}'.format(stack_file))
+            print(f'WARNING: "coherence" is missing in file {stack_file}')
     else:
-        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), './inputs/ifgramStack.h5')
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), stack_file)
 
+    # get coordinate type of the loaded dataset
     atr = readfile.read_attribute(stack_file)
+    coord_type = 'GEO' if 'Y_FIRST' in atr.keys() else 'RADAR'
+    processor = atr['PROCESSOR']
 
-    # 2. geom_file: height
-    if 'X_FIRST' in atr.keys():
-        flist = [os.path.join(work_dir, 'inputs/geometryGeo.h5')]
-    else:
-        flist = [os.path.join(work_dir, 'inputs/geometryRadar.h5')]
-    geom_file = is_file_exist(flist, abspath=True)
-    if geom_file is not None:
+    # 2. [required] geom_file: height
+    geom_file = os.path.join(work_dir, 'inputs', f'geometry{coord_type.capitalize()}.h5')
+    dname = geometryDatasetNames[0]
+
+    if is_file_exist(geom_file, abspath=True):
         obj = geometry(geom_file)
         obj.open(print_msg=False)
-        dname = geometryDatasetNames[0]
         if dname not in obj.datasetNames:
-            raise ValueError('required dataset "{}" is missing in file {}'.format(dname, geom_file))
+            raise ValueError(f'required dataset "{dname}" is missing in file {geom_file}')
     else:
-        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), './inputs/geometry*.h5')
+        raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), geom_file)
 
-    # 3. lookup_file: latitude,longitude or rangeCoord,azimuthCoord
+    # 3. [required for radar-coord] lookup_file: latitude,longitude or rangeCoord,azimuthCoord
     # could be different than geometry file in case of roipac and gamma
-    flist = [os.path.join(work_dir, 'inputs/geometry*.h5')]
-    lookup_file = get_lookup_file(flist, abspath=True, print_msg=print_msg)
-    if 'X_FIRST' not in atr.keys():
+    lookup_file = os.path.join(work_dir, 'inputs/geometry*.h5')
+    lookup_file = get_lookup_file(lookup_file, abspath=True, print_msg=print_msg)
+    if coord_type == 'RADAR':
         if lookup_file is not None:
             obj = geometry(lookup_file)
             obj.open(print_msg=False)
 
-            if atr['PROCESSOR'] in ['isce', 'doris']:
+            # get the proper lookup table dataset names
+            if processor in ['isce', 'doris']:
                 dnames = geometryDatasetNames[1:3]
-            elif atr['PROCESSOR'] in ['gamma', 'roipac']:
+            elif processor in ['gamma', 'roipac']:
                 dnames = geometryDatasetNames[3:5]
             else:
-                raise AttributeError('InSAR processor: {}'.format(atr['PROCESSOR']))
+                msg = f'Unknown InSAR processor: {processor} to locate look up table!'
+                raise AttributeError(msg)
 
             for dname in dnames:
                 if dname not in obj.datasetNames:
-                    load_complete = False
-                    raise Exception('required dataset "{}" is missing in file {}'.format(dname, lookup_file))
+                    raise Exception(f'required dataset "{dname}" is missing in file {lookup_file}')
         else:
-            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), './inputs/geometry*.h5')
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT), lookup_file)
     else:
         print("Input data seems to be geocoded. Lookup file not needed.")
 
+    # 4. [optional] ionosphere stack file: unwrapPhase, coherence
+    ion_file = os.path.join(work_dir, 'inputs/ionStack.h5')
+    dname = 'unwrapPhase'
+
+    if is_file_exist(ion_file, abspath=True):
+        obj = ifgramStack(ion_file)
+        obj.open(print_msg=False)
+        if dname not in obj.datasetNames:
+            raise ValueError(f'required dataset "{dname}" is missing in file {ion_file}')
+
+        # check coherence for phase stack
+        if 'unwrapPhase' in obj.datasetNames and 'coherence' not in obj.datasetNames:
+            print(f'WARNING: "coherence" is missing in file {ion_file}')
+    else:
+        ion_file = None
+
     if relpath:
         stack_file  = os.path.relpath(stack_file)  if stack_file  else stack_file
         geom_file   = os.path.relpath(geom_file)   if geom_file   else geom_file
         lookup_file = os.path.relpath(lookup_file) if lookup_file else lookup_file
+        ion_file    = os.path.relpath(ion_file)    if ion_file    else ion_file
 
     # print message
     if print_msg:
-        print(('Loaded dataset are processed by '
-               'InSAR software: {}'.format(atr['PROCESSOR'])))
-        if 'X_FIRST' in atr.keys():
-            print('Loaded dataset is in GEO coordinates')
-        else:
-            print('Loaded dataset is in RADAR coordinates')
-        print('Interferograms Stack: {}'.format(stack_file))
-        print('Geometry File       : {}'.format(geom_file))
-        print('Lookup Table File   : {}'.format(lookup_file))
-        if load_complete:
-            print('-'*50)
-            print('All data needed found/loaded/copied. Processed 2-pass InSAR data can be removed.')
-        print('-'*50)
+        msg  = f'Loaded dataset are processed by InSAR software: {processor}'
+        msg += f'\nLoaded dataset are in {coord_type} coordinates'
+        msg += f'\nInterferogram Stack: {stack_file}'
+        msg += f'\nIonosphere    Stack: {ion_file}' if ion_file else ''
+        msg += f'\nGeometry      File : {geom_file}'
+        msg += f'\nLookup Table  File : {lookup_file}'
+        msg += '\n' + '-' * 50
+        print(msg)
 
-    return load_complete, stack_file, geom_file, lookup_file
+    return stack_file, geom_file, lookup_file, ion_file
 
 
 #################################################################################
@@ -170,7 +183,6 @@ def read_timeseries_lalo(lat, lon, ts_file, lookup_file=None, ref_lat=None, ref_
     return dates, dis, dis_std
 
 
-#################################################################################
 def read_timeseries_yx(y, x, ts_file, ref_y=None, ref_x=None, zero_first=True,
                        win_size=1, unit='m', method='mean', print_msg=True):
     """ Read time-series of one pixel with input y/x
@@ -228,10 +240,10 @@ def read_timeseries_yx(y, x, ts_file, ref_y=None, ref_x=None, zero_first=True,
         pass
     elif unit == 'cm':
         dis *= 100.
-        dis_std *= 100.
+        dis_std = None if dis_std is None else dis_std * 100.
     elif unit == 'mm':
         dis *= 1000.
-        dis_std *= 1000.
+        dis_std = None if dis_std is None else dis_std * 1000.
     else:
         raise ValueError('un-supported output unit: {}'.format(unit))
 
@@ -302,14 +314,20 @@ def transect_yx(z, atr, start_yx, end_yx, interpolation='nearest'):
 
     # Calculate Distance along the line
     earth_radius = 6.3781e6    # in meter
+    dist_unit = 'm'
     if 'Y_FIRST' in atr.keys():
         [lat0, lat1] = coordinate(atr).yx2lalo([y0, y1], coord_type='y')
         lat_c = (lat0 + lat1) / 2.
         x_step = float(atr['X_STEP']) * np.pi/180.0 * earth_radius * np.cos(lat_c * np.pi/180)
         y_step = float(atr['Y_STEP']) * np.pi/180.0 * earth_radius
     else:
-        x_step = range_ground_resolution(atr)
-        y_step = azimuth_ground_resolution(atr)
+        try:
+            x_step = range_ground_resolution(atr)
+            y_step = azimuth_ground_resolution(atr)
+        except KeyError:
+            x_step = 1
+            y_step = 1
+            dist_unit = 'pixel'
     dist_line = np.hypot((xs - x0) * x_step,
                          (ys - y0) * y_step)
 
@@ -323,6 +341,8 @@ def transect_yx(z, atr, start_yx, end_yx, interpolation='nearest'):
     transect['X'] = xs[mask]
     transect['value'] = z_line[mask]
     transect['distance'] = dist_line[mask]
+    transect['distance_unit'] = dist_unit
+
     return transect
 
 
@@ -380,3 +400,80 @@ def transect_lines(z, atr, lines):
 
 #################################################################################
 
+def prepare_geo_los_geometry(geom_file, unit='rad'):
+    """Prepare LOS geometry data/info in geo-coordinates.
+
+    Parameters: geom_file  - str, path of geometry file
+                unit       - str, rad or deg, output angle unit
+    Returns:    inc_angle  - 2D np.ndarray, incidence angle in radians / degrees
+                head_angle - 2D np.ndarray, heading   angle in radians / degrees
+                atr        - dict, metadata in geo-coordinate
+    """
+
+    print('prepare LOS geometry in geo-coordinates from file: {}'.format(geom_file))
+    atr = readfile.read_attribute(geom_file)
+
+    print('read incidenceAngle from file: {}'.format(geom_file))
+    inc_angle = readfile.read(geom_file, datasetName='incidenceAngle')[0]
+
+    if 'azimuthAngle' in readfile.get_dataset_list(geom_file):
+        print('read azimuthAngle   from file: {}'.format(geom_file))
+        print('convert azimuth angle to heading angle')
+        az_angle  = readfile.read(geom_file, datasetName='azimuthAngle')[0]
+        head_angle = azimuth2heading_angle(az_angle)
+    else:
+        print('use the HEADING attribute as the mean heading angle')
+        head_angle = np.ones(inc_angle.shape, dtype=np.float32) * float(atr['HEADING'])
+
+    # geocode inc/az angle data if in radar-coord
+    if 'Y_FIRST' not in atr.keys():
+        print('-'*50)
+        print('geocoding the incidence / heading angles ...')
+        res_obj = resample(lut_file=geom_file, src_file=geom_file)
+        res_obj.open()
+        res_obj.prepare()
+
+        # resample data
+        box = res_obj.src_box_list[0]
+        inc_angle  = res_obj.run_resample(src_data=inc_angle[box[1]:box[3], box[0]:box[2]])
+        head_angle = res_obj.run_resample(src_data=head_angle[box[1]:box[3], box[0]:box[2]])
+
+        # update attribute
+        atr = attr.update_attribute4radar2geo(atr, res_obj=res_obj)
+
+    # for 'Y_FIRST' not in 'degree'
+    # e.g. meters for UTM projection from ASF HyP3
+    if not atr['Y_UNIT'].lower().startswith('deg'):
+        # get SNWE in meter
+        length, width = int(atr['LENGTH']), int(atr['WIDTH'])
+        N = float(atr['Y_FIRST'])
+        W = float(atr['X_FIRST'])
+        y_step = float(atr['Y_STEP'])
+        x_step = float(atr['X_STEP'])
+        S = N + y_step * length
+        E = W + x_step * width
+
+        # SNWE in meter --> degree
+        lat0, lon0 = to_latlon(atr['OG_FILE_PATH'], W, N)
+        lat1, lon1 = to_latlon(atr['OG_FILE_PATH'], E, S)
+        lat_step = (lat1 - lat0) / length
+        lon_step = (lon1 - lon0) / width
+
+        # update Y/X_FIRST/STEP/UNIT
+        atr['Y_FIRST'] = lat0
+        atr['X_FIRST'] = lon0
+        atr['Y_STEP'] = lat_step
+        atr['X_STEP'] = lon_step
+        atr['Y_UNIT'] = 'degrees'
+        atr['X_UNIT'] = 'degrees'
+
+    # set invalid values to nan
+    inc_angle[inc_angle == 0] = np.nan
+    head_angle[head_angle == 90] = np.nan
+
+    # unit: degree to radian
+    if unit.startswith('rad'):
+        inc_angle *= np.pi / 180.
+        head_angle *= np.pi / 180.
+
+    return inc_angle, head_angle, atr
