@@ -16,6 +16,7 @@
 
 
 import os
+import math
 import h5py
 import numpy as np
 import matplotlib.pyplot as plt
@@ -126,7 +127,7 @@ def incidence_angle(atr, dem=None, dimension=2, print_msg=True):
         # consider the local variable due to topography
         if dem is not None:
             range_dist = range_distance(atr, dimension=2, print_msg=False)
-            inc_angle = (np.pi - np.arccos(((r+dem)**2 + range_dist**2 - (r+H)**2) / 
+            inc_angle = (np.pi - np.arccos(((r+dem)**2 + range_dist**2 - (r+H)**2) /
                                            (2*(r+dem)*range_dist))) * 180.0/np.pi
         else:
             inc_angle = np.tile(np.linspace(inc_angle_n, inc_angle_f, num=width,
@@ -203,6 +204,47 @@ def azimuth_ground_resolution(atr):
     return az_step
 
 
+def auto_lat_lon_step_size(atr, lat_c=None):
+    """Get the default lat/lon step size for geocoding.
+
+    Treat the pixel in radar coordinates as an rotated rectangle. Use the bounding box
+    of the rotated rectangle for the ratio between lat and lon steps. Then scale the
+    lat and lon step size to ensure the same area between the pixels in radar and geo
+    coordinates.
+
+    Link: https://math.stackexchange.com/questions/4001034
+
+    Parameters: atr      - dict, standard mintpy metadata
+                lat_c    - float, central latitude in degree
+    Returns:    lat_step - float, latitude  step size in degree
+                lon_step - float, longitude step size in degree
+    """
+    # azimuth angle (rotation angle) in radian
+    az_angle = np.deg2rad(abs(heading2azimuth_angle(float(atr['HEADING']))))
+
+    # radar pixel size in meter
+    az_step = azimuth_ground_resolution(atr)
+    rg_step = range_ground_resolution(atr)
+
+    # geo pixel size in meter
+    x_step = rg_step * abs(np.cos(az_angle)) + az_step * abs(np.sin(az_angle))
+    y_step = rg_step * abs(np.sin(az_angle)) + az_step * abs(np.cos(az_angle))
+    scale_factor = np.sqrt((rg_step * az_step) / (x_step * y_step))
+    x_step *= scale_factor
+    y_step *= scale_factor
+
+    # geo pixel size in degree
+    if lat_c is None:
+        if 'LAT_REF1' in atr.keys():
+            lat_c = (float(atr['LAT_REF1']) + float(atr['LAT_REF3'])) / 2.
+        else:
+            lat_c = 0
+    lon_step = np.rad2deg(x_step / (EARTH_RADIUS * np.cos(np.deg2rad(lat_c))))
+    lat_step = np.rad2deg(y_step / EARTH_RADIUS) * -1.
+
+    return lat_step, lon_step
+
+
 
 #################################### File Operation ##########################################
 def touch(fname_list, times=None):
@@ -275,17 +317,68 @@ def to_latlon(infile, x, y):
     return y, x
 
 
-def get_lat_lon(meta, geom_file=None, box=None, dimension=2):
+def xyz_to_local_radius(xyz):
+    """Calculate satellite height and ellpsoid local radius from orbital state vector.
+
+    This is a simplified version of the following functions from ISCE-2:
+    + isce.isceobj.Planet.xyz_to_llh()
+    + isce.isceobj.Ellipsoid.localRadius()
+
+    Parameters: xyz    - tuple of 3 float, orbital state vector
+    Reference:  height - float, satellite altitude in m
+                radius - float, Earth radius in m
+    """
+
+    # paramters from isce.isceobj.Planet.AstronomicalHandbook
+    a = 6378137.000       # WGS84 semimajor
+    e2 = 0.0066943799901  # WGS84 eccentricity squared
+
+    # xyz --> llh
+    a2 = a**2
+    e4 = e2**2
+    r_llh =  [0]*3
+    d_llh = [[0]*3 for i in range(len(xyz))]
+    xy2 = xyz[0]**2 + xyz[1]**2
+    p = (xy2) / a2
+    q = (1. - e2) * xyz[2]**2 / a2
+    r = (p + q - e4) / 6.
+    s = e4 * p * q / (4. * r**3)
+    t = (1. + s + math.sqrt(s * (2. + s))) **(1. / 3.)
+    u = r * (1. + t + 1./t)
+    v = math.sqrt(u**2 + e4 * q)
+    w = e2 * (u + v - q) / (2. * v)
+    k = math.sqrt(u + v + w**2) - w
+    D = k * math.sqrt(xy2) / (k + e2)
+    r_llh[0] = math.atan2(xyz[2], D)
+    r_llh[1] = math.atan2(xyz[1], xyz[0])
+    r_llh[2] = (k + e2 - 1.) * math.sqrt(D**2 + xyz[2]**2) / k
+    d_llh[0] = math.degrees(r_llh[0])
+    d_llh[1] = math.degrees(r_llh[1])
+    d_llh[2] = r_llh[2]
+    height = r_llh[2]
+
+    # calculate local radius
+    b = a * (1.0 - e2)**0.5
+    latg = math.atan( math.tan(math.radians(d_llh[0])) * a**2 / b**2 )
+    arg = (math.cos(latg)**2 / a**2) + (math.sin(latg)**2 / b**2)
+    radius = 1.0 / math.sqrt(arg)
+
+    return height, radius
+
+
+def get_lat_lon(meta, geom_file=None, box=None, dimension=2, ystep=1, xstep=1):
     """Extract precise pixel-wise lat/lon.
 
     For meta dict in geo-coordinates OR geom_file with latitude/longitude dataset
 
     Returned lat/lon are corresponds to the pixel center
 
-    Parameters: meta : dict, including LENGTH, WIDTH and Y/X_FIRST/STEP
-                box  : 4-tuple of int for (x0, y0, x1, y1)
-    Returns:    lats : 2D np.array for latitude  in size of (length, width)
-                lons : 2D np.array for longitude in size of (length, width)
+    Parameters: meta      - dict, including LENGTH, WIDTH and Y/X_FIRST/STEP
+                box       - 4-tuple of int for (x0, y0, x1, y1)
+                dimension - int, output lat/lon matrix dimension, 1 or 2
+                y/xstep   - int, number of pixels to skip for each output pixel
+    Returns:    lats      - 1/2D np.array for latitude  in size of (length, _width_)
+                lons      - 1/2D np.array for longitude in size of (_length_, width)
     """
     length, width = int(meta['LENGTH']), int(meta['WIDTH'])
     if box is None:
@@ -334,6 +427,16 @@ def get_lat_lon(meta, geom_file=None, box=None, dimension=2):
 
     lats = np.array(lats, dtype=np.float32)
     lons = np.array(lons, dtype=np.float32)
+
+    # y/xstep
+    if ystep * xstep > 1:
+        if lats.ndim == 1:
+            lats = lats[::ystep]
+            lons = lons[::xstep]
+        elif lats.ndim == 2:
+            lats = lats[::ystep, ::xstep]
+            lons = lons[::ystep, ::xstep]
+
     return lats, lons
 
 
@@ -386,15 +489,15 @@ def heading2azimuth_angle(head_angle):
 
 
 def enu2los(e, n, u, inc_angle, head_angle=None, az_angle=None):
-    """
+    """Project east/north/up motion into the line-of-sight direction defined by incidence/azimuth angle.
     Parameters: e          - np.ndarray or float, displacement in east-west direction, east as positive
                 n          - np.ndarray or float, displacement in north-south direction, north as positive
                 u          - np.ndarray or float, displacement in vertical direction, up as positive
-                inc_angle  - np.ndarray or float, local incidence angle from vertical
+                inc_angle  - np.ndarray or float, local incidence angle from vertical, in the unit of degrees
                 head_angle - np.ndarray or float, azimuth angle of the SAR platform along track direction
-                             measured from the north with clockwise direction as positive in the unit of degrees
+                             measured from the north with clockwise direction as positive, in the unit of degrees
                 az_angle   - np.ndarray or float, azimuth angle of the LOS vector from the ground to the SAR platform
-                             measured from the north with anti-clockwise direction as positive in the unit of degrees
+                             measured from the north with anti-clockwise direction as positive, in the unit of degrees
                              head_angle = 90 - az_angle
     Returns:    v_los      - np.ndarray or float, displacement in line-of-sight direction, moving toward satellite as positive
 
@@ -410,20 +513,23 @@ def enu2los(e, n, u, inc_angle, head_angle=None, az_angle=None):
         # head_angle -> az_angle
         if head_angle is not None:
             az_angle = heading2azimuth_angle(head_angle)
-    if az_angle is None:
-        raise ValueError('az_angle can not be None!')
+        else:
+            raise ValueError('az_angle can not be None!')
 
-    inc_angle *= np.pi/180.
-    az_angle *= np.pi/180.
-    v_los = (  e * np.sin(inc_angle) * np.sin(az_angle) * -1
-             + n * np.sin(inc_angle) * np.cos(az_angle)
-             + u * np.cos(inc_angle))
+    v_los = (  e * np.sin(np.deg2rad(inc_angle)) * np.sin(np.deg2rad(az_angle)) * -1
+             + n * np.sin(np.deg2rad(inc_angle)) * np.cos(np.deg2rad(az_angle))
+             + u * np.cos(np.deg2rad(inc_angle)))
 
     return v_los
 
 
 def four_corners(atr):
-    """Return 4 corners lat/lon"""
+    """Get the 4 corners coordinates from metadata dict in geo-coordinates.
+    Parameters: atr - dict
+    Returns:    south, north, west, east - float, in degrees or meters
+    Examples:   S, N, W, E = ut.four_corners(atr)
+                SNWE = ut.four_corners(atr)
+    """
     width  = int(atr['WIDTH'])
     length = int(atr['LENGTH'])
     lon_step = float(atr['X_STEP'])
@@ -432,7 +538,7 @@ def four_corners(atr):
     north = float(atr['Y_FIRST'])
     south = north + lat_step * length
     east  = west  + lon_step * width
-    return west, east, south, north
+    return south, north, west, east
 
 
 def get_circular_mask(x, y, radius, shape):
@@ -632,7 +738,7 @@ def yes_or_no(question):
     else:
         return yes_or_no("Uhhhh... please enter ")
 
-    
+
 def update_attribute_or_not(atr_new, atr_orig):
     """Compare new attributes with exsiting ones"""
     update = False
@@ -676,7 +782,7 @@ def check_parallel(file_num=1, print_msg=True, maxParallelNum=8):
     if file_num <= 1:
         enable_parallel = False
         if print_msg:
-            print('parallel processing is diabled for one input file')
+            print('parallel processing is disabled for one input file')
         return 1, enable_parallel, None, None
 
     # Check required python module
@@ -703,6 +809,24 @@ def check_parallel(file_num=1, print_msg=True, maxParallelNum=8):
         return num_cores, enable_parallel, None, None
 
 
+def print_command_line(script_name, args):
+    """print the command line with "" for arguments containing *.
+
+    Parameters: script_name - str, e.g. prep_isce.py
+                args        - list(str), list of input arguments
+    """
+    cmd = script_name
+    for arg in args:
+        # for option values containing *, add "" in the print out msg
+        # so that it can be copied and pasted to run directly.
+        if not arg.startswith('-') and '*' in arg:
+            cmd += f' "{arg}"'
+        else:
+            cmd += f' {arg}'
+
+    print(cmd)
+    return
+
 
 #################################### Math / Statistics ###################################
 def median_abs_deviation(data, center=None, scale=0.67449):
@@ -790,6 +914,14 @@ def round_to_1(x):
     return round(x, -digit)
 
 
+def round_up_to_odd(x):
+    """Round a float up to the next odd integer .
+    Link: https://stackoverflow.com/questions/31648729
+    """
+    y = np.ceil(x) // 2 * 2 + 1
+    return y.astype(np.int16)
+
+
 def highest_power_of_2(x):
     """Given a number x, find the highest power of 2 that <= x"""
     res = np.power(2, np.floor(np.log2(x)))
@@ -824,5 +956,3 @@ def is_number(string):
         return True
     except ValueError:
         return False
-
-
