@@ -519,15 +519,24 @@ def update_template_file(template_file, extra_dict, delimiter='='):
         if not line.startswith(('%', '#')) and len(c) > 1:
             key = c[0]
             value = str.replace(c[1], '\n', '').split("#")[0].strip()
-            value = value.replace('*', '\*')  # interpret * as character
+
             if key in extra_dict.keys() and extra_dict[key] != value:
-                # use "= {OLD_VALUE}" for search/replace to be more robust
+                # prepare value string to search & replace following "re" expression syntax
+                # link: https://docs.python.org/3/library/re.html
+                value2search = value
+                # 1. interpret special symbols as characters
+                for symbol in ['*', '[', ']', '(', ')']:
+                    value2search = value2search.replace(symbol, "\{}".format(symbol))
+                # 2. use "= {OLD_VALUE}" for search/replace to be more robust
                 # against the scenario when key name contains {OLD_VALUE}
                 # i.e. mintpy.load.autoPath
-                old_value_str = re.findall(delimiter+'[\s]*'+value, line)[0]
+                value2search = delimiter+'[\s]*'+value2search
+
+                old_value_str = re.findall(value2search, line)[0]
                 new_value_str = old_value_str.replace(value, extra_dict[key])
                 line = line.replace(old_value_str, new_value_str, 1)
                 print('    {}: {} --> {}'.format(key, value, extra_dict[key]))
+
         f_tmp.write(line)
     f_tmp.close()
 
@@ -616,7 +625,7 @@ def check_file_size(fname_list, mode_width=None, mode_length=None):
 #################################### Interaction ##########################################
 def is_file_exist(file_list, abspath=True):
     """Check if any file in the file list 1) exists and 2) readable
-    Parameters: file_list : list of string, file name with/without wildcards
+    Parameters: file_list : str or list(str), file name with/without wildcards
                 abspath   : bool, return absolute file name/path or not
     Returns:    file_path : string, found file name/path; None if not.
     """
