@@ -10,6 +10,9 @@ import collections
 ###########################################################################
 Tag = collections.namedtuple('Tag', 'version date')
 release_history = (
+    Tag('1.4.1', '2022-08-15'),
+    Tag('1.4.0', '2022-08-04'),
+    Tag('1.3.3', '2022-04-14'),
     Tag('1.3.2', '2021-11-21'),
     Tag('1.3.1', '2021-08-02'),
     Tag('1.3.0', '2021-03-06'),
@@ -32,19 +35,19 @@ release_history = (
 release_version = release_history[0].version
 release_date = release_history[0].date
 
-def get_version_info(version='v{}'.format(release_version), date=release_date):
+def get_version_info():
     """Grab version and date of the latest commit from a git repository"""
     # go to the repository directory
     dir_orig = os.getcwd()
     os.chdir(os.path.dirname(os.path.dirname(__file__)))
 
-    # grab git info into string
     try:
+        # grab from git cmd
         cmd = "git describe --tags"
         version = subprocess.check_output(cmd.split(), stderr=subprocess.DEVNULL)
-        version = version.decode('utf-8').strip()
+        version = version.decode('utf-8').strip()[1:]
 
-        #if there are new commits after the latest release
+        # if there are new commits after the latest release
         if '-' in version:
             version, num_commit = version.split('-')[:2]
             version += '-{}'.format(num_commit)
@@ -52,8 +55,11 @@ def get_version_info(version='v{}'.format(release_version), date=release_date):
         cmd = "git log -1 --date=short --format=%cd"
         date = subprocess.check_output(cmd.split(), stderr=subprocess.DEVNULL)
         date = date.decode('utf-8').strip()
+
     except:
-        pass
+        # use the latest release version/date
+        version = release_version
+        date = release_date
 
     # go back to the original directory
     os.chdir(dir_orig)
@@ -62,9 +68,9 @@ def get_version_info(version='v{}'.format(release_version), date=release_date):
 
 ###########################################################################
 
-version_num, version_date = get_version_info()
+version, version_date = get_version_info()
 version_description = """MintPy version {v}, date {d}""".format(
-    v=version_num,
+    v=version,
     d=version_date,
 )
 
@@ -85,7 +91,7 @@ ___________________________________________________________
    Miami InSAR Time-series software in Python    \______/ 
           MintPy {v}, {d}
 ___________________________________________________________
-""".format(v=version_num, d=version_date)
+""".format(v=version, d=version_date)
 
 website = 'https://github.com/insarlab/MintPy'
 
