@@ -11,7 +11,6 @@
 import os
 import sys
 import re
-import argparse
 import datetime as dt
 import numpy as np
 import matplotlib.pyplot as plt
@@ -23,15 +22,12 @@ import cartopy.crs as ccrs
 
 from mintpy.objects import (
     giantIfgramStack,
-    geometryDatasetNames,
-    ifgramDatasetNames,
     ifgramStack,
     timeseriesKeyNames,
-    timeseries,
 )
 from mintpy.objects.gps import GPS
 from mintpy.utils import (
-    arg_group,
+    arg_utils,
     ptime,
     readfile,
     utils as ut,
@@ -88,10 +84,12 @@ PLOT_TEMPLATE = """Plot Setting:
 """
 
 
-def create_parser():
-    parser = argparse.ArgumentParser(description='Plot InSAR Product in 2D',
-                                     formatter_class=argparse.RawTextHelpFormatter,
-                                     epilog=EXAMPLE)
+def create_parser(subparsers=None):
+    synopsis = 'Plot InSAR Product in 2D'
+    epilog = EXAMPLE
+    name = __name__.split('.')[-1]
+    parser = arg_utils.create_argument_parser(
+        name, synopsis=synopsis, description=synopsis, epilog=epilog, subparsers=subparsers)
 
     infile = parser.add_argument_group('Input File', 'File/Dataset to display')
     infile.add_argument('file', type=str, help='file for display')
@@ -120,17 +118,17 @@ def create_parser():
                              '  reverse = x * -1\n'
                              '  inverse = 1 / x')
 
-    parser = arg_group.add_data_disp_argument(parser)
-    parser = arg_group.add_dem_argument(parser)
-    parser = arg_group.add_figure_argument(parser)
-    parser = arg_group.add_gps_argument(parser)
-    parser = arg_group.add_mask_argument(parser)
-    parser = arg_group.add_map_argument(parser)
-    parser = arg_group.add_memory_argument(parser)
-    parser = arg_group.add_point_argument(parser)
-    parser = arg_group.add_reference_argument(parser)
-    parser = arg_group.add_save_argument(parser)
-    parser = arg_group.add_subset_argument(parser)
+    parser = arg_utils.add_data_disp_argument(parser)
+    parser = arg_utils.add_dem_argument(parser)
+    parser = arg_utils.add_figure_argument(parser)
+    parser = arg_utils.add_gps_argument(parser)
+    parser = arg_utils.add_mask_argument(parser)
+    parser = arg_utils.add_map_argument(parser)
+    parser = arg_utils.add_memory_argument(parser)
+    parser = arg_utils.add_point_argument(parser)
+    parser = arg_utils.add_reference_argument(parser)
+    parser = arg_utils.add_save_argument(parser)
+    parser = arg_utils.add_subset_argument(parser)
 
     return parser
 
@@ -289,16 +287,14 @@ def update_inps_with_file_metadata(inps, metadata):
             inps.ref_lalo = []
 
         else:
-            inps.ref_yx = coord.geo2radar(inps.ref_lalo[0], inps.ref_lalo[1])
+            inps.ref_yx = coord.geo2radar(inps.ref_lalo[0], inps.ref_lalo[1])[:2]
             vprint('input reference point in y  /x  : {}'.format(inps.ref_yx))
 
     # ref_lalo
     if inps.ref_yx and inps.geo_box:
-        inps.ref_lalo = [coord.yx2lalo(inps.ref_yx[0], coord_type='y'),
-                         coord.yx2lalo(inps.ref_yx[1], coord_type='x')]
+        inps.ref_lalo = coord.radar2geo(inps.ref_yx[0], inps.ref_yx[1])[:2]
     elif 'REF_LAT' in metadata.keys():
-        inps.ref_lalo = [float(metadata['REF_LAT']),
-                         float(metadata['REF_LON'])]
+        inps.ref_lalo = [float(metadata['REF_LAT']), float(metadata['REF_LON'])]
     else:
         inps.ref_lalo = None
 
@@ -406,22 +402,27 @@ def check_map_projection(inps, metadata, print_msg=True):
 ##################################################################################################
 def update_data_with_plot_inps(data, metadata, inps):
     # 1. spatial referencing with respect to the seed point
-    if inps.ref_yx:   # and inps.ref_yx != [int(metadata['REF_Y']), int(metadata['REF_X'])]:
+    if inps.ref_yx:
+        inps.ref_box = [inps.ref_yx[1],     inps.ref_yx[0],
+                        inps.ref_yx[1] + 1, inps.ref_yx[0] + 1]
+
         # update ref_y/x to subset
-        try:
-            ref_y = inps.ref_yx[0] - inps.pix_box[1]
-            ref_x = inps.ref_yx[1] - inps.pix_box[0]
-        except:
-            pass
+        ref_y = inps.ref_yx[0] - inps.pix_box[1]
+        ref_x = inps.ref_yx[1] - inps.pix_box[0]
 
         # update ref_y/x for multilooking
         if inps.multilook_num > 1:
             ref_y = int((ref_y - int(inps.multilook_num / 2)) / inps.multilook_num)
             ref_x = int((ref_x - int(inps.multilook_num / 2)) / inps.multilook_num)
 
-        # applying spatial referencing
         if len(data.shape) == 2:
-            ref_val = data[ref_y, ref_x]
+            # read ref_val
+            if 0 <= ref_y < data.shape[-2] and 0 <= ref_x < data.shape[-1]:
+                ref_val = data[ref_y, ref_x]
+            else:
+                ref_val = readfile.read(inps.file, datasetName=inps.dset[0], box=inps.ref_box, print_msg=False)[0]
+
+            # applying spatial referencing
             if not np.ma.is_masked(ref_val) and not np.isnan(ref_val):
                 data -= ref_val
                 vprint('set reference pixel to: {}'.format(inps.ref_yx))
@@ -436,13 +437,11 @@ def update_data_with_plot_inps(data, metadata, inps):
             if 0 <= ref_y < data.shape[-2] and 0 <= ref_x < data.shape[-1]:
                 ref_val = np.squeeze(data[:, ref_y, ref_x])
             elif inps.key == 'timeseries':
-                ref_box = [inps.ref_yx[1],     inps.ref_yx[0],
-                           inps.ref_yx[1] + 1, inps.ref_yx[0] + 1]
-                ref_val = readfile.read(inps.file, datasetName=inps.dset, box=ref_box, print_msg=False)[0]
+                ref_val = readfile.read(inps.file, datasetName=inps.dset, box=inps.ref_box, print_msg=False)[0]
             else:
                 raise ValueError('input reference point {} is out of data coverage!'.format(inps.ref_yx))
 
-            # apply referencing
+            # apply spatial referencing
             if not np.ma.is_masked(ref_val) and np.all(~np.isnan(ref_val)):
                 data -= np.tile(ref_val.reshape(-1, 1, 1), (1, data.shape[1], data.shape[2]))
                 vprint('set reference pixel to: {}'.format(inps.ref_yx))
@@ -488,7 +487,9 @@ def update_data_with_plot_inps(data, metadata, inps):
     # 4. update display min/max
     inps.dlim = [np.nanmin(data), np.nanmax(data)]
     if not inps.vlim: # and data.ndim < 3:
-        inps.cmap_lut, inps.vlim = pp.auto_adjust_colormap_lut_and_disp_limit(data, print_msg=inps.print_msg)
+        (inps.cmap_lut,
+         inps.vlim,
+         inps.unique_values) = pp.auto_adjust_colormap_lut_and_disp_limit(data, print_msg=inps.print_msg)
     vprint('data    range: {} {}'.format(inps.dlim, inps.disp_unit))
     vprint('display range: {} {}'.format(inps.vlim, inps.disp_unit))
 
@@ -546,7 +547,7 @@ def plot_slice(ax, data, metadata, inps=None):
         # Draw coastline using cartopy resolution parameters
         if inps.coastline:
             vprint('draw coast line with resolution: {}'.format(inps.coastline))
-            ax.coastlines(resolution=inps.coastline)
+            ax.coastlines(resolution=inps.coastline, linewidth=inps.coastline_linewidth)
 
         # Plot DEM
         if inps.dem_file:
@@ -563,8 +564,8 @@ def plot_slice(ax, data, metadata, inps=None):
             y, x = coord.geo2radar(ref_site_lalo[0], ref_site_lalo[1])[0:2]
             ref_data = data[y - inps.pix_box[1], x - inps.pix_box[0]]
             data -= ref_data
-            vprint(('referencing InSAR data to the pixel nearest to '
-                    f'GPS station: {inps.ref_gps_site} at {ref_site_lalo} '
+            vprint(('referencing InSAR data to the pixel nearest to GNSS station: '
+                    f'{inps.ref_gps_site} at [{ref_site_lalo[0]:.6f}, {ref_site_lalo[1]:.6f}] '
                     f'by substrating {ref_data:.3f} {inps.disp_unit}'))
             # do not show the original InSAR reference point
             inps.disp_ref_pixel = False
@@ -592,11 +593,13 @@ def plot_slice(ax, data, metadata, inps=None):
 
         # Lat Lon labels
         if inps.lalo_label:
-            pp.draw_lalo_label(inps.geo_box, ax,
+            pp.draw_lalo_label(ax,
+                               geo_box=inps.geo_box,
                                lalo_step=inps.lalo_step,
                                lalo_loc=inps.lalo_loc,
                                lalo_max_num=inps.lalo_max_num,
-                               font_size=inps.font_size,
+                               lalo_offset=inps.lalo_offset,
+                               font_size=inps.lalo_font_size if inps.lalo_font_size else inps.font_size,
                                projection=inps.map_proj_obj,
                                print_msg=inps.print_msg)
         else:
@@ -753,8 +756,7 @@ def plot_slice(ax, data, metadata, inps=None):
 
     # 3.2 Title
     if inps.disp_title:
-        ax.set_title(inps.fig_title, fontsize=inps.font_size,
-                     color=inps.font_color)
+        ax.set_title(inps.fig_title, fontsize=inps.font_size, color=inps.font_color)
 
     # 3.3 Flip Left-Right / Up-Down
     if inps.flip_lr:
@@ -770,18 +772,27 @@ def plot_slice(ax, data, metadata, inps=None):
         ax.axis('off')
         vprint('turn off axis display')
 
-    # 3.5 Turn off tick label
+    # 3.5 Tick labels
     if inps.disp_tick:
         # manually turn ON to enable tick labels for UTM with cartopy
         # link: https://github.com/SciTools/cartopy/issues/491
         ax.xaxis.set_visible(True)
         ax.yaxis.set_visible(True)
     else:
-        # ax.set_xticklabels([])
-        # ax.set_yticklabels([])
+        # turn off tick labels
         ax.get_xaxis().set_ticks([])
         ax.get_yaxis().set_ticks([])
 
+    # rotate Y-axis tick labels
+    # link: https://stackoverflow.com/questions/10998621
+    if inps.ylabel_rot:
+        kwargs = dict(rotation=inps.ylabel_rot)
+        # center the vertical alignment for vertical tick labels
+        if inps.ylabel_rot % 90 == 0:
+            kwargs['va'] = 'center'
+        plt.setp(ax.get_yticklabels(), **kwargs)
+        vprint(f'rotate Y-axis tick labels by {inps.ylabel_rot} deg')
+
     return ax, inps, im, cbar
 
 
@@ -1093,6 +1104,10 @@ def read_data4figure(i_start, i_end, inps, metadata):
                               xstep=inps.multilook_num,
                               ystep=inps.multilook_num,
                               print_msg=False)[0]
+            # reference pixel info in unwrapPhase
+            if inps.dset[i].startswith('unwrapPhase') and inps.file_ref_yx:
+                ref_y, ref_x = inps.file_ref_yx
+                d[d!=0] -= d[ref_y, ref_x]
             data[i - i_start, :, :] = d
             prog_bar.update(i - i_start + 1, suffix=inps.dset[i].split('/')[-1])
         prog_bar.close()
@@ -1146,9 +1161,11 @@ def read_data4figure(i_start, i_end, inps, metadata):
             and all(arg not in inps.argv for arg in ['-v', '--vlim', '--wrap'])
             and not (inps.dsetFamilyList[0].startswith('unwrap') and not inps.file_ref_yx)
             and inps.dsetFamilyList[0] not in ['bperp']):
-        inps.cmap_lut, inps.vlim = pp.auto_adjust_colormap_lut_and_disp_limit(data,
-                                                                              num_multilook=10,
-                                                                              print_msg=False)
+        (inps.cmap_lut,
+         inps.vlim,
+         inps.unique_values) = pp.auto_adjust_colormap_lut_and_disp_limit(data,
+                                                                          num_multilook=10,
+                                                                          print_msg=False)
 
     return data
 
@@ -1481,7 +1498,7 @@ def prep_slice(cmd, auto_fig=False):
         cmd += f'--sub-lon {W} {E} --sub-lat {S} {N} -c jet -v -3 10 '
         cmd += '--cbar-loc bottom --cbar-nbins 3 --cbar-ext both --cbar-size 5% '
         cmd += '--lalo-step 0.2 --lalo-loc 1 0 1 0 --scalebar 0.3 0.80 0.05 --notitle'
-        data, atr ,inps = view.prep_slice(cmd)
+        data, atr, inps = view.prep_slice(cmd)
         ax, inps, im, cbar = view.plot_slice(ax, data, atr, inps)
         plt.show()
     """
@@ -1541,7 +1558,7 @@ def prep_slice(cmd, auto_fig=False):
     if auto_fig == True:
         figsize = [i/2.0 for i in inps.fig_size]
         subplot_kw = dict(projection=inps.map_proj_obj) if inps.map_proj_obj is not None else {}
-        fig, ax = plt.subplots(figsize=figsize, num='Figure', subplot_kw=subplot_kw)
+        fig, ax = plt.subplots(figsize=figsize, subplot_kw=subplot_kw)
         return data, atr, inps, ax
     else:
         return data, atr, inps
@@ -1651,7 +1668,7 @@ class viewer():
 
             # prepare figure
             subplot_kw = dict(projection=self.map_proj_obj) if self.map_proj_obj is not None else {}
-            fig, ax = plt.subplots(figsize=self.fig_size, num='Figure', subplot_kw=subplot_kw)
+            fig, ax = plt.subplots(figsize=self.fig_size, subplot_kw=subplot_kw)
             if not self.disp_whitespace:
                 fig.subplots_adjust(left=0,right=1,bottom=0,top=1)
 
